@@ -212,6 +212,10 @@ while [ $# -gt 0 ]; do
 		LyxBuildDir=$(echo ${1}|cut -d= -f2)
 		shift
 		;;
+	--with-util-dir=*)
+		LyXUtilitiesDir=$(echo ${1}|cut -d= -f2)
+		shift
+		;;
 	--help|--help=*)
 		usage
 		;;
@@ -251,19 +255,21 @@ aspellstrip=
 LyxBuildDir=${LyxBuildDir:-$(dirname "${LyxSourceDir}")/lyx-build}
 DMGLocation=${DMGLocation:-"${LyxBuildDir}"}
 
+LyXUtilitiesDir=${LyXUtilitiesDir:-"${LyxBuildDir}"/utilities}
+
 GettextSourceDir=${GETTEXTDIR:-$(dirname "${LyxSourceDir}")/${GettextSource}}
 GettextBuildDir="${LyxBuildDir}"/"${GettextSource}"
-GettextInstallDir=${GettextInstallDir:-"${LyxBuildDir}"/utilities}
+GettextInstallDir=${GettextInstallDir:-"${LyXUtilitiesDir}"}
 GettextInstallHdr="${GettextInstallDir}/include/libintl.h"
 
 ASpellSourceDir=${ASPELLDIR:-$(dirname "${LyxSourceDir}")/${ASpellSource}}
 ASpellBuildDir="${ASpellSourceDir}"
-ASpellInstallDir=${ASpellInstallDir:-"${LyxBuildDir}"/utilities}
+ASpellInstallDir=${ASpellInstallDir:-"${LyXUtilitiesDir}"}
 ASpellInstallHdr="${ASpellInstallDir}/include/aspell.h"
 
 HunSpellSourceDir=${HUNSPELLDIR:-$(dirname "${LyxSourceDir}")/${HunSpellSource}}
 HunSpellBuildDir="${HunSpellSourceDir}"
-HunSpellInstallDir=${HunSpellInstallDir:-"${LyxBuildDir}"/utilities}
+HunSpellInstallDir=${HunSpellInstallDir:-"${LyXUtilitiesDir}"}
 HunSpellInstallHdr="${HunSpellInstallDir}/include/hunspell/hunspell.h"
 
 Qt4SourceDir=${QT4SOURCEDIR:-$(dirname "${LyxSourceDir}")/${Qt4SourceVersion}}
@@ -771,6 +777,25 @@ convert_universal() {
 	done
 }
 
+deduplicate() {
+	find "$@" -type f -print | while read file ; do
+		echo $(md5 -q "$file") "$file"
+	done | sort | while read hash file ; do
+		ppath=$(dirname "$pfile")
+		path=$(dirname "$file")
+		if [ "$phash" = "$hash" -a "$ppath" = "$path" ]; then
+			pname=$(basename "$pfile")
+			name=$(basename "$file")
+			cmp -s "$pfile" "$file" && (
+				rm "$file"
+				cd "$path" && ln -s "$pname" "$name" && echo link for "$file" created
+			)
+		fi
+		phash="$hash"
+		pfile="$file"
+	done
+}
+
 copy_dictionaries() {
 	if [ -d "${ASpellInstallDir}" -a "yes" = "${aspell_dictionaries}" ]; then
 		ASpellResources="${LyxAppPrefix}/Contents/Resources"
@@ -786,11 +811,13 @@ copy_dictionaries() {
 		HunSpellResources="${LyxAppPrefix}/Contents/Resources"
 		if [ -d "${DictionarySourceDir}" ]; then
 			( cd "${DictionarySourceDir}" && find dicts -name .svn -prune -o -type f -print | cpio -pmdv "${HunSpellResources}" )
+			deduplicate "${HunSpellResources}"/dicts
 		fi
 	fi
 	if [ -d "${DictionarySourceDir}" -a "yes" = "${thesaurus_deployment}" ]; then
 		MyThesResources="${LyxAppPrefix}/Contents/Resources"
 		( cd "${DictionarySourceDir}" && find thes -name .svn -prune -o -type f -print | cpio -pmdv "${MyThesResources}" )
+		deduplicate "${MyThesResources}"/thes
 	fi
 }
 
