@@ -52,6 +52,7 @@
 #include "support/filetools.h"
 #include "support/gettext.h"
 #include "support/Messages.h"
+#include "support/mutex.h"
 #include "support/Translator.h"
 #include "support/lstrings.h"
 
@@ -87,11 +88,6 @@ static char const * const string_orientation[] = {
 };
 
 
-static char const * const string_footnotekinds[] = {
-	"footnote", "margin", "fig", "tab", "alg", "wide-fig", "wide-tab", ""
-};
-
-
 static char const * const tex_graphics[] = {
 	"default", "dvialw", "dvilaser", "dvipdf", "dvipdfm", "dvipdfmx",
 	"dvips", "dvipsone", "dvitops", "dviwin", "dviwindo", "dvi2ps", "emtex",
@@ -447,6 +443,14 @@ map<string, string> const & BufferParams::auto_packages()
 {
 	static map<string, string> packages;
 	if (packages.empty()) {
+		// We could have a race condition here that two threads
+		// discover an empty map at the same time and want to fill
+		// it, but that is no problem, since the same contents is
+		// filled in twice then. Having the locker inside the
+		// packages.empty() condition has the advantage that we
+		// don't need the mutex overhead for simple reading.
+		static Mutex mutex;
+		Mutex::Locker locker(&mutex);
 		// adding a package here implies a file format change!
 		packages["amsmath"] =
 			N_("The LaTeX package amsmath is only used if AMS formula types or symbols from the AMS math toolbars are inserted into formulas");
@@ -623,7 +627,7 @@ string BufferParams::readToken(Lexer & lex, string const & token,
 						 "See section 3.1.2.2 (Class Availability) of the\n"
 						 "User's Guide for more information."), desc, prereqs);
 			frontend::Alert::warning(_("Document class not available"),
-				       msg);
+				       msg, true);
 		}
 	} else if (token == "\\begin_preamble") {
 		readPreamble(lex);
@@ -1315,6 +1319,11 @@ bool BufferParams::writeLaTeX(otexstream & os, LaTeXFeatures & features,
 	// are doing!
 	if (features.mustProvide("fix-cm"))
 		os << "\\RequirePackage{fix-cm}\n";
+	// Likewise for fixltx2e. If other packages conflict with this policy,
+	// treat it as a package bug (and report it!)
+	// See http://www.latex-project.org/cgi-bin/ltxbugs2html?pr=latex/4407
+	if (features.mustProvide("fixltx2e"))
+		os << "\\RequirePackage{fixltx2e}\n";
 
 	os << "\\documentclass";
 
@@ -1765,6 +1774,9 @@ bool BufferParams::writeLaTeX(otexstream & os, LaTeXFeatures & features,
 			lyxpreamble += "\\synctex=-1\n";
 	}
 
+	// The package options (via \PassOptionsToPackage)
+	lyxpreamble += from_ascii(features.getPackageOptions());
+
 	// due to interferences with babel and hyperref, the color package has to
 	// be loaded (when it is not already loaded) before babel when hyperref
 	// is used with the colorlinks option, see
@@ -1937,7 +1949,7 @@ bool BufferParams::writeLaTeX(otexstream & os, LaTeXFeatures & features,
 			+ atlyxpreamble + "\\makeatother\n\n";
 
 	// We try to load babel late, in case it interferes with other packages.
-	// Jurabib, hyperref, varioref and listings (bug 8995) have to be
+	// Jurabib, hyperref, varioref, bicaption and listings (bug 8995) have to be
 	// called after babel, though.
 	if (use_babel && !features.isRequired("jurabib")
 	    && !features.isRequired("hyperref")
@@ -1950,6 +1962,8 @@ bool BufferParams::writeLaTeX(otexstream & os, LaTeXFeatures & features,
 						   features.needBabelLangOptions())) + '\n';
 		lyxpreamble += from_utf8(features.getBabelPostsettings());
 	}
+	if (features.isRequired("bicaption"))
+		lyxpreamble += "\\usepackage{bicaption}\n";
 	if (!listings_params.empty() || features.isRequired("listings"))
 		lyxpreamble += "\\usepackage{listings}\n";
 	if (!listings_params.empty()) {
@@ -2810,7 +2824,8 @@ void BufferParams::writeEncodingPreamble(otexstream & os,
 		// inputenc must be omitted.
 		// see http://www.mail-archive.com/lyx-devel@lists.lyx.org/msg129680.html
 		if ((!encodings.empty() || package == Encoding::inputenc)
-		    && !features.isRequired("japanese")) {
+		    && !features.isRequired("japanese")
+		    && !features.isProvided("inputenc")) {
 			os << "\\usepackage[";
 			set<string>::const_iterator it = encodings.begin();
 			set<string>::const_iterator const end = encodings.end();
@@ -2841,7 +2856,9 @@ void BufferParams::writeEncodingPreamble(otexstream & os,
 			break;
 		case Encoding::inputenc:
 			// do not load inputenc if japanese is used
-			if (features.isRequired("japanese"))
+			// or if the class provides inputenc
+			if (features.isRequired("japanese")
+			    || features.isProvided("inputenc"))
 				break;
 			os << "\\usepackage[" << from_ascii(encoding().latexName())
 			   << "]{inputenc}\n";
@@ -2977,7 +2994,8 @@ Encoding const & BufferParams::encoding() const
 	// FIXME: actually, we should check for the flavor
 	// or runparams.isFullyUnicode() here:
 	// This check will not work with XeTeX/LuaTeX and tex fonts.
-	// Thus we have to reset the encoding in Buffer::makeLaTeXFile.
+	// Thus we have to reset the encoding in Buffer::makeLaTeXFile
+	// (for export) and Buffer::writeLaTeXSource (for preview).
 	if (useNonTeXFonts)
 		return *(encodings.fromLyXName("utf8-plain"));
 	if (inputenc == "auto" || inputenc == "default")
