@@ -15,6 +15,7 @@
 #ifndef BUFFER_VIEW_H
 #define BUFFER_VIEW_H
 
+#include "DocumentClassPtr.h"
 #include "update_flags.h"
 
 #include "support/strfwd.h"
@@ -124,7 +125,7 @@ public:
 	/// \return Tool tip for the given position.
 	docstring toolTip(int x, int y) const;
 	/// \return the context menu for the given position.
-	docstring contextMenu(int x, int y) const;
+	std::string contextMenu(int x, int y) const;
 
 	/// Save the current position as bookmark.
 	/// if idx == 0, save to temp_bookmark
@@ -166,8 +167,6 @@ public:
 	/// Scroll to the cursor.
 	/// \param recenter Whether the cursor should be centered on screen
 	bool scrollToCursor(DocIterator const & dit, bool recenter);
-	/// LFUN_SCROLL Helper.
-	void lfunScroll(FuncRequest const & cmd);
 	/// scroll down document by the given number of pixels.
 	int scrollDown(int pixels);
 	/// scroll up document by the given number of pixels.
@@ -194,7 +193,7 @@ public:
 	void resetInlineCompletionPos();
 	/// set the inline completion postfix and its position in the buffer.
 	/// Updates the updateFlags in \c cur.
-	void setInlineCompletion(Cursor & cur, DocIterator const & pos,
+	void setInlineCompletion(Cursor const & cur, DocIterator const & pos,
 		docstring const & completion, size_t uniqueChars = 0);
 
 	/// translate and insert a character, using the correct keymap.
@@ -318,6 +317,8 @@ public:
 	void clearLastInset(Inset * inset) const;
 	/// Is the mouse hovering a clickable inset or element?
 	bool clickableInset() const;
+	///
+	void makeDocumentClass();
 
 private:
 	/// noncopyable
@@ -334,7 +335,7 @@ private:
 	/// when it is allowed to scroll below the document.
 	int minVisiblePart();
 
-	/// Search recursively for the the innermost inset that covers (x, y) position.
+	/// Search recursively for the innermost inset that covers (x, y) position.
 	/// \retval 0 if no inset is found.
 	Inset const * getCoveringInset(
 		Text const & text, //< The Text where we start searching.
@@ -347,7 +348,7 @@ private:
 	void updateHoveredInset() const;
 
 	///
-	void updateDocumentClass(DocumentClass const * const olddc);
+	void updateDocumentClass(DocumentClassConstPtr olddc);
 	///
 	int width_;
 	///
