@@ -98,13 +98,15 @@ size_t DocRange::length() const
 
 class DocPair {
 public:
-	DocPair() {}
+	DocPair()
+	{}
 
 	DocPair(DocIterator o_, DocIterator n_)
 		: o(o_), n(n_)
 	{}
 
-	bool operator!=(DocPair const & rhs) {
+	bool operator!=(DocPair const & rhs)
+	{
 		// this might not be intuitive but correct for our purpose
 		return o != rhs.o && n != rhs.n;
 	}
@@ -147,10 +149,16 @@ public:
 	{}
 
 	/// Returns the from pair
-	DocPair from() const { return DocPair(o.from, n.from); }
+	DocPair from() const
+	{
+		return DocPair(o.from, n.from);
+	}
 
 	/// Returns the to pair
-	DocPair to() const { return DocPair(o.to, n.to); }
+	DocPair to() const
+	{
+		return DocPair(o.to, n.to);
+	}
 
 	DocRange o;
 	DocRange n;
@@ -182,7 +190,8 @@ static DocRangePair stepIntoInset(DocPair const & inset_location)
 template<class T>
 class compl_vector {
 public:
-	compl_vector() {}
+	compl_vector()
+	{}
 
 	void reset(T const & def)
 	{
@@ -224,7 +233,8 @@ public:
 	{}
 
 	///
-	~Impl() {}
+	~Impl()
+	{}
 
 	// Algorithm to find the shortest edit string. This algorithm 
 	// only needs a linear amount of memory (linear with the sum
@@ -236,7 +246,8 @@ public:
 	bool abort_;
 
 	///
-	QString status() {
+	QString status()
+	{
 		QString status;
 		status += toqstr("recursion level:") + " " + QString::number(recursion_level_)
 			+ " " + toqstr("differences:") + " " + QString::number(D_);
@@ -259,7 +270,7 @@ private:
 	SnakeResult retrieveMiddleSnake(int k, int D, Direction direction,
 		DocPair & middle_snake);
 	
-	/// Find the the furthest reaching D-path (number of horizontal
+	/// Find the furthest reaching D-path (number of horizontal
 	/// and vertical steps; differences between the old and new
 	/// document) in the k-diagonal (vertical minus horizontal steps).
 	void furthestDpathKdiagonal(int D, int k,
@@ -273,7 +284,7 @@ private:
 	/// around the middle snake.
 	void diff_i(DocRangePair const & rp);
 
-	/// Processes the splitted chunks. It either adds them as deleted,
+	/// Processes the split chunks. It either adds them as deleted,
 	/// as added, or call diff_i for further processing.
 	void diffPart(DocRangePair const & rp);
 
@@ -452,7 +463,8 @@ static bool equal(Inset const * i_o, Inset const * i_n)
 }
 
 
-static bool equal(DocIterator & o, DocIterator & n) {
+static bool equal(DocIterator & o, DocIterator & n)
+{
 	// Explicitly check for this, so we won't call
 	// Paragraph::getChar for the last pos.
 	bool const o_lastpos = o.pos() == o.lastpos();
@@ -598,7 +610,7 @@ Compare::Impl::SnakeResult Compare::Impl::retrieveMiddleSnake(
 	if (os[k].empty() && os_r[kk].empty()) {
 		// No, there is no snake at all, in which case
 		// the length of the shortest edit script is M+N.
-		LASSERT(2 * D - odd_offset_ == M_ + N_, /**/);
+		LATTEST(2 * D - odd_offset_ == M_ + N_);
 		return NoSnake;
 	} 
 	
@@ -818,7 +830,7 @@ void Compare::Impl::processSnake(DocRangePair const & rp)
 			pit_type const pit = it.o.pit() - rp.o.from.pit();
 			pos_type const pos = pit ? it.o.pos() : it.o.pos() - rp.o.from.pos();
 			inset = pars[pit].getInset(pos);
-			LASSERT(inset, /**/);
+			LASSERT(inset, continue);
 			diffInset(inset, it);
 		}
 	}
