@@ -65,22 +65,6 @@ namespace lyx {
 
 namespace {
 
-bool positionable(DocIterator const & cursor, DocIterator const & anchor)
-{
-	// avoid deeper nested insets when selecting
-	if (cursor.depth() > anchor.depth())
-		return false;
-
-	// anchor might be deeper, should have same path then
-	for (size_t i = 0; i < cursor.depth(); ++i)
-		if (&cursor[i].inset() != &anchor[i].inset())
-			return false;
-
-	// position should be ok.
-	return true;
-}
-
-
 // Find position closest to (x, y) in cell given by iter.
 // Used only in mathed
 DocIterator bruteFind2(Cursor const & c, int x, int y)
@@ -274,6 +258,36 @@ CursorData::CursorData(DocIterator const & dit)
 {}
 
 
+
+
+ostream & operator<<(ostream & os, CursorData const & cur)
+{
+	os << "\n cursor:                                | anchor:\n";
+	for (size_t i = 0, n = cur.depth(); i != n; ++i) {
+		os << " " << cur[i] << " | ";
+		if (i < cur.anchor_.depth())
+			os << cur.anchor_[i];
+		else
+			os << "-------------------------------";
+		os << "\n";
+	}
+	for (size_t i = cur.depth(), n = cur.anchor_.depth(); i < n; ++i) {
+		os << "------------------------------- | " << cur.anchor_[i] << "\n";
+	}
+	os << " selection: " << cur.selection_
+//	   << " x_target: " << cur.x_target_
+	   << " boundary: " << cur.boundary() << endl;
+	return os;
+}
+
+
+LyXErr & operator<<(LyXErr & os, CursorData const & cur)
+{
+	os.stream() << cur;
+	return os;
+}
+
+
 // be careful: this is called from the bv's constructor, too, so
 // bv functions are not yet available!
 Cursor::Cursor(BufferView & bv)
@@ -1307,36 +1321,6 @@ bool Cursor::selHandle(bool sel)
 	setSelection(sel);
 	return true;
 }
-
-
-ostream & operator<<(ostream & os, Cursor const & cur)
-{
-	os << "\n cursor:                                | anchor:\n";
-	for (size_t i = 0, n = cur.depth(); i != n; ++i) {
-		os << " " << cur[i] << " | ";
-		if (i < cur.anchor_.depth())
-			os << cur.anchor_[i];
-		else
-			os << "-------------------------------";
-		os << "\n";
-	}
-	for (size_t i = cur.depth(), n = cur.anchor_.depth(); i < n; ++i) {
-		os << "------------------------------- | " << cur.anchor_[i] << "\n";
-	}
-	os << " selection: " << cur.selection_
-	   << " x_target: " << cur.x_target_
-	   << " boundary: " << cur.boundary() << endl;
-	return os;
-}
-
-
-LyXErr & operator<<(LyXErr & os, Cursor const & cur)
-{
-	os.stream() << cur;
-	return os;
-}
-
-
 } // namespace lyx
 
 
@@ -2368,7 +2352,10 @@ void Cursor::sanitize()
 {
 	setBuffer(&bv_->buffer());
 	DocIterator::sanitize();
-	anchor_.sanitize();
+	if (selection())
+		anchor_.sanitize();
+	else
+		resetAnchor();
 }
 
 
