@@ -88,6 +88,10 @@ public:
 	explicit CursorData(Buffer * buffer);
 	///
 	explicit CursorData(DocIterator const & dit);
+	/// output
+	friend std::ostream & operator<<(std::ostream & os, CursorData const & cur);
+	friend LyXErr & operator<<(LyXErr & os, CursorData const & cur);
+
 protected:
 	/// the anchor position
 	DocIterator anchor_;
@@ -345,10 +349,6 @@ public:
 	/// for stable iterators or Undo data.
 	void sanitize();
 
-	/// output
-	friend std::ostream & operator<<(std::ostream & os, Cursor const & cur);
-	friend LyXErr & operator<<(LyXErr & os, Cursor const & cur);
-
 	///
 	bool textUndo();
 	///
