@@ -16,7 +16,6 @@
 #include "support/types.h"
 
 #include <vector>
-#include <string>
 
 
 namespace lyx {
@@ -42,7 +41,7 @@ public:
 class ErrorList : private std::vector<ErrorItem>
 {
 public:
-	ErrorList() : std::vector<ErrorItem> () {};
+	ErrorList() : std::vector<ErrorItem> () {}
 
 	using std::vector<ErrorItem>::push_back;
 	using std::vector<ErrorItem>::end;
