@@ -408,11 +408,11 @@ string Formats::getFormatFromFile(FileName const & filename) const
 	if (filename.empty())
 		return string();
 
+	string format;
 #ifdef HAVE_MAGIC_H
 	if (filename.exists()) {
 		magic_t magic_cookie = magic_open(MAGIC_MIME);
 		if (magic_cookie) {
-			string format;
 			if (magic_load(magic_cookie, NULL) != 0) {
 				LYXERR(Debug::GRAPHICS, "Formats::getFormatFromFile\n"
 					<< "\tCouldn't load magic database - "
@@ -442,24 +442,38 @@ string Formats::getFormatFromFile(FileName const & filename) const
 				}
 			}
 			magic_close(magic_cookie);
-			if (!format.empty())
+			// libmagic recognizes as latex also some formats of ours
+			// such as pstex and pdftex. Therefore we have to perform
+			// additional checks in this case (bug 9244).
+			if (!format.empty() && format != "latex")
 				return format;
 		}
 	}
 #endif
 
-	string const format = guessFormatFromContents(filename);
 	string const ext = getExtension(filename.absFileName());
-	if (isZippedFileFormat(format) && !ext.empty()) {
-		string const & fmt_name = formats.getFormatFromExtension(ext);
-		if (!fmt_name.empty()) {
-			Format const * p_format = formats.getFormat(fmt_name);
-			if (p_format && p_format->zippedNative())
-				return p_format->name();
+	if (format.empty()) {
+		string const format = guessFormatFromContents(filename);
+		if (isZippedFileFormat(format) && !ext.empty()) {
+			string const & fmt_name = formats.getFormatFromExtension(ext);
+			if (!fmt_name.empty()) {
+				Format const * p_format = formats.getFormat(fmt_name);
+				if (p_format && p_format->zippedNative())
+					return p_format->name();
+			}
 		}
+		// Don't simply return latex (bug 9244).
+		if (!format.empty() && format != "latex")
+			return format;
+	}
+
+	// Both libmagic and our guessing from contents may return as latex
+	// also lyx files and our pstex and pdftex formats. In this case we
+	// give precedence to the format determined by the extension.
+	if (format == "latex") {
+		format = getFormatFromExtension(ext);
+		return format.empty() ? "latex" : format;
 	}
-	if (!format.empty())
-		return format;
 
 	// try to find a format from the file extension.
 	return getFormatFromExtension(ext);
@@ -680,8 +694,10 @@ bool Formats::view(Buffer const & buffer, FileName const & filename,
 	if (!contains(command, token_from_format))
 		command += ' ' + token_from_format;
 
-	command = subst(command, token_from_format, quoteName(onlyFileName(filename.toFilesystemEncoding())));
-	command = subst(command, token_path_format, quoteName(onlyPath(filename.toFilesystemEncoding())));
+	command = subst(command, token_from_format,
+	                quoteName(onlyFileName(filename.toFilesystemEncoding()), quote_shell_filename));
+	command = subst(command, token_path_format,
+	                quoteName(onlyPath(filename.toFilesystemEncoding()), quote_shell_filename));
 	command = subst(command, token_socket_format, quoteName(theServerSocket().address()));
 	LYXERR(Debug::FILES, "Executing command: " << command);
 	// FIXME UNICODE utf8 can be wrong for files
@@ -748,8 +764,10 @@ bool Formats::edit(Buffer const & buffer, FileName const & filename,
 	if (!contains(command, token_from_format))
 		command += ' ' + token_from_format;
 
-	command = subst(command, token_from_format, quoteName(filename.toFilesystemEncoding()));
-	command = subst(command, token_path_format, quoteName(onlyPath(filename.toFilesystemEncoding())));
+	command = subst(command, token_from_format,
+	                quoteName(filename.toFilesystemEncoding(), quote_shell_filename));
+	command = subst(command, token_path_format,
+	                quoteName(onlyPath(filename.toFilesystemEncoding()), quote_shell_filename));
 	command = subst(command, token_socket_format, quoteName(theServerSocket().address()));
 	LYXERR(Debug::FILES, "Executing command: " << command);
 	// FIXME UNICODE utf8 can be wrong for files
