@@ -51,14 +51,14 @@ enum FuncCode
 	LFUN_SELECTION_PASTE, // JMarc 2008/12/15
 	LFUN_CUT,
 	LFUN_COPY,
-	LFUN_CLOSE_TAB_GROUP,
+	LFUN_TAB_GROUP_CLOSE,
 	LFUN_NOTE_NEXT,
 	// 20
 	LFUN_INSET_TOGGLE,
-	LFUN_HFILL_INSERT,
 	LFUN_TEXTSTYLE_APPLY,
 	LFUN_TEXTSTYLE_UPDATE,
 	LFUN_FONT_EMPH,
+	LFUN_CLIPBOARD_PASTE_SIMPLE,    // tommaso, 20111028
 	// 25
 	LFUN_FONT_BOLD,
 	LFUN_FONT_BOLDSYMBOL,
@@ -106,7 +106,7 @@ enum FuncCode
 	LFUN_CHAR_DELETE_FORWARD,
 	LFUN_CHAR_DELETE_BACKWARD,
 	LFUN_NEWLINE_INSERT,		// renamed: JSpitzm, 20080325
-	LFUN_BREAK_PARAGRAPH,
+	LFUN_PARAGRAPH_BREAK,
 	// 65
 	LFUN_QUOTE_INSERT,
 	LFUN_ACCENT_CIRCUMFLEX,
@@ -132,11 +132,11 @@ enum FuncCode
 	LFUN_ACCENT_OGONEK,
 	LFUN_SELF_INSERT,
 	// 85
-	LFUN_GETBUFNAME,
 	LFUN_SERVER_GET_XY,
 	LFUN_SERVER_SET_XY,
-	LFUN_SPLIT_VIEW,
-	LFUN_LINEATCURSOR,
+	LFUN_VIEW_SPLIT,
+	LFUN_VIEW_CLOSE,		// Tommaso, 20120915
+	LFUN_BUFFER_FORALL,             // scottkostyshak, 20120720
 	// 90
 	LFUN_SERVER_GET_LAYOUT,
 	LFUN_SERVER_GET_FILENAME,
@@ -168,7 +168,7 @@ enum FuncCode
 	LFUN_WORD_DELETE_FORWARD,
 	LFUN_WORD_DELETE_BACKWARD,
 	// 115
-	LFUN_LINE_DELETE,
+	LFUN_LINE_DELETE_FORWARD,
 	LFUN_DEBUG_LEVEL_SET,
 	LFUN_MARK_OFF,
 	LFUN_MARK_ON,
@@ -205,7 +205,7 @@ enum FuncCode
 	LFUN_PARAGRAPH_DOWN,            // Asger 1996-10-01
 	// 145
 	LFUN_PARAGRAPH_DOWN_SELECT,     // Asger 1996-10-01
-	LFUN_INSET_FORALL,		// lasgouttes, 20091127
+	LFUN_INSET_FORALL,		          // lasgouttes, 20091127
 	LFUN_BUFFER_NEW_TEMPLATE,       // Asger 1997-02-02
 	LFUN_BUFFER_RELOAD,             // Asger 1997-02-02
 	LFUN_RECONFIGURE,
@@ -230,7 +230,7 @@ enum FuncCode
 	// 165
 	LFUN_VC_REPO_UPDATE,
 	LFUN_VC_COMPARE,
-	LFUN_HYPERLINK_INSERT,          // CFO-G 971121
+	LFUN_HREF_INSERT,               // CFO-G 971121
 	LFUN_WORD_FIND_FORWARD,         // Etienne 980216
 	LFUN_WORD_FIND_BACKWARD,        // Etienne 980220
 	// 170
@@ -246,16 +246,16 @@ enum FuncCode
 	LFUN_FOOTNOTE_INSERT,           // Jug 20000307
 	LFUN_MATH_AMS_MATRIX,           // uwestoehr 12-07-2009
 	// 180
+	LFUN_IPA_INSERT,                // spitz, 20120305
+	LFUN_IN_IPA,                    // spitz, 20120520
+	LFUN_IPAMACRO_INSERT,           // spitz, 20120822
 	LFUN_TABULAR_INSERT,            // Jug 20000412
-	LFUN_LOFVIEW,                   // Dekel 20000519
-	LFUN_LOTVIEW,                   // Dekel 20000519
-	LFUN_LOAVIEW,                   // Dekel 20000519
 	LFUN_SET_COLOR,                 // SLior 20000611
 	// 185
 	LFUN_MARGINALNOTE_INSERT,       // Lgb 20000626
 	LFUN_FLOAT_INSERT,              // Lgb 20000627
 	LFUN_FLOAT_WIDE_INSERT,         // Lgb 20010531
-	LFUN_CAPTION_INSERT,            // Lgb 20000718; inactive as of 20060905
+	LFUN_CAPTION_INSERT,            // Lgb 20000718
 	LFUN_BUFFER_SWITCH,
 	// 190
 	LFUN_BUFFER_CHKTEX,             // Asger 971030
@@ -402,11 +402,11 @@ enum FuncCode
 	LFUN_COMPLETION_POPUP,
 	LFUN_COMPLETION_INLINE,
 	// 310
-	LFUN_COMPLETION_COMPLETE,
+	LFUN_COMPLETE,
 	LFUN_SET_GRAPHICS_GROUP,
 	LFUN_COMPLETION_CANCEL,
 	LFUN_COMPLETION_ACCEPT,
- 	LFUN_COMMAND_ALTERNATIVES,
+	LFUN_COMMAND_ALTERNATIVES,
 	// 315
 	LFUN_TAB_INSERT,
 	LFUN_TAB_DELETE,
@@ -414,15 +414,15 @@ enum FuncCode
 	LFUN_REGEXP_MODE,               // Tommaso, 20081003
 	LFUN_LABEL_GOTO,                // Ale 970806
 	// 320
-	LFUN_LABEL_COPY_AS_REF,         // sts, 20081116
-	LFUN_LABEL_INSERT_AS_REF,       // vfr, 20090407
+	LFUN_LABEL_COPY_AS_REFERENCE,   // sts, 20081116
+	LFUN_LABEL_INSERT_AS_REFERENCE, // vfr, 20090407
 	LFUN_PHANTOM_INSERT,            // uwestoehr, 20090130
 	LFUN_INSET_BEGIN,               // JMarc, 20090316
 	LFUN_INSET_END,                 // JMarc, 20090316
 	// 325
 	LFUN_INSET_BEGIN_SELECT,        // JMarc, 20090316
 	LFUN_INSET_END_SELECT,          // JMarc, 20090316
-	LFUN_INSET_SELECT_ALL,			// vfr, 20090822
+	LFUN_INSET_SELECT_ALL,          // vfr, 20090822
 	LFUN_BUFFER_ZOOM_IN,            // vfr, 20090330
 	LFUN_BUFFER_ZOOM_OUT,           // vfr, 20090330
 	// 330
@@ -430,9 +430,9 @@ enum FuncCode
 	LFUN_SECTION_SELECT,            // vfr, 20090503
 	LFUN_FONT_UNDERLINE,
 	LFUN_FONT_STRIKEOUT,
-	LFUN_FONT_UULINE,
+	LFUN_FONT_UNDERUNDERLINE,
 	// 335
-	LFUN_FONT_UWAVE,
+	LFUN_FONT_UNDERWAVE,
 	LFUN_BUFFER_EXPORT,             // Lgb 97-07-29
 	LFUN_BUFFER_TOGGLE_COMPRESSION, // bpeng 20060427
 	LFUN_BRANCH_ADD,                // spitz 20090707
@@ -448,8 +448,15 @@ enum FuncCode
 	LFUN_PREVIEW_INSERT,            // vfr, 20100328
 	LFUN_FORWARD_SEARCH,
 	LFUN_SCRIPT_INSERT,             // gb, 20101123
+	LFUN_BUFFER_EXPORT_AS,          // tommaso 20111006
 	// 350
-
+	LFUN_BRANCH_MASTER_ACTIVATE,    // spitz 20120930
+	LFUN_BRANCH_MASTER_DEACTIVATE,  // spitz 20120930
+	LFUN_ENVIRONMENT_SPLIT,         // spitz 20121223
+	LFUN_VC_RENAME,                 // gb 20130205
+	LFUN_VC_COPY,                   // gb 20130205
+	// 355
+	LFUN_SPELLING_CONTINUOUSLY,     // vfr, 20130324
 	LFUN_LASTACTION                 // end of the table
 };
 
