@@ -426,9 +426,14 @@ void LaTeXFeatures::require(set<string> const & names)
 
 
 void LaTeXFeatures::useLayout(docstring const & layoutname)
+{
+	useLayout(layoutname, 0);
+}
+
+
+void LaTeXFeatures::useLayout(docstring const & layoutname, int level)
 {
 	// Some code to avoid loops in dependency definition
-	static int level = 0;
 	const int maxlevel = 30;
 	if (level > maxlevel) {
 		lyxerr << "LaTeXFeatures::useLayout: maximum level of "
@@ -448,9 +453,7 @@ void LaTeXFeatures::useLayout(docstring const & layoutname)
 		require(layout.requires());
 
 		if (!layout.depends_on().empty()) {
-			++level;
-			useLayout(layout.depends_on());
-			--level;
+			useLayout(layout.depends_on(), level + 1);
 		}
 		usedLayouts_.push_back(layoutname);
 	} else {
@@ -458,8 +461,6 @@ void LaTeXFeatures::useLayout(docstring const & layoutname)
 		       << to_utf8(layoutname) << "' does not exist in this class"
 		       << endl;
 	}
-
-	--level;
 }
 
 
@@ -741,7 +742,10 @@ char const * simplefeatures[] = {
 	"tfrupee",
 	"shapepar",
 	"rsphrase",
-	"algorithm2e"
+	"algorithm2e",
+	"sectionbox",
+	"tcolorbox",
+	"pdfcomment"
 };
 
 char const * bibliofeatures[] = {
@@ -827,6 +831,22 @@ string const LaTeXFeatures::getColorOptions() const
 }
 
 
+string const LaTeXFeatures::getPackageOptions() const
+{
+	ostringstream packageopts;
+	// Output all the package option stuff we have been asked to do.
+	map<string, string>::const_iterator it =
+		params_.documentClass().packageOptions().begin();
+	map<string, string>::const_iterator en =
+		params_.documentClass().packageOptions().end();
+	for (; it != en; ++it)
+		if (mustProvide(it->first))
+			packageopts << "\\PassOptionsToPackage{" << it->second << "}"
+				 << "{" << it->first << "}\n";
+	return packageopts.str();
+}
+
+
 string const LaTeXFeatures::getPackages() const
 {
 	ostringstream packages;
@@ -837,16 +857,6 @@ string const LaTeXFeatures::getPackages() const
 	// also unknown packages can be requested. They are silently
 	// swallowed now. We should change this eventually.
 
-	// Output all the package option stuff we have been asked to do.
-	map<string, string>::const_iterator it =
-	    params_.documentClass().packageOptions().begin();
-	map<string, string>::const_iterator en =
-	    params_.documentClass().packageOptions().end();
-	for (; it != en; ++it)
-		if (mustProvide(it->first))
-			packages << "\\PassOptionsToPackage{" << it->second << "}"
-			         << "{" << it->first << "}\n";
-
 	//  These are all the 'simple' includes.  i.e
 	//  packages which we just \usepackage{package}
 	for (int i = 0; i < nb_simplefeatures; ++i) {
@@ -878,28 +888,10 @@ string const LaTeXFeatures::getPackages() const
 	if (!params_.useNonTeXFonts && !use_newtxmath && !amsPackages.empty())
 		packages << amsPackages;
 
-	// fixltx2e must be loaded after amsthm, since amsthm produces an error with
-	// the redefined \[ command (bug 7233). Load it as early as possible, since
-	// other packages might profit from it.
-	if (mustProvide("fixltx2e"))
-		packages << "\\usepackage{fixltx2e}\n";
-
 	if (mustProvide("cancel") &&
 	    params_.use_package("cancel") != BufferParams::package_off)
 		packages << "\\usepackage{cancel}\n";
-	// wasysym is a simple feature, but it must be after amsmath if both
-	// are used
-	// wasysym redefines some integrals (e.g. iint) from amsmath. That
-	// leads to inconsistent integrals. We only load this package if
-	// the document does not contain integrals (then isRequired("esint")
-	// is false) or if esint is used, since esint redefines all relevant
-	// integral symbols from wasysym and amsmath.
-	// See http://www.lyx.org/trac/ticket/1942
-	if (mustProvide("wasysym") &&
-	    params_.use_package("wasysym") != BufferParams::package_off &&
-	    (params_.use_package("esint") != BufferParams::package_off || !isRequired("esint")))
-		packages << "\\usepackage{wasysym}\n";
-
+	
 	// accents must be loaded after amsmath
 	if (mustProvide("accents") &&
 	    params_.use_package("accents") != BufferParams::package_off)
@@ -957,14 +949,27 @@ string const LaTeXFeatures::getPackages() const
 	if (mustProvide("setspace") && !isProvided("SetSpace"))
 		packages << "\\usepackage{setspace}\n";
 
-	// we need to assure that mhchem is loaded before esint
-	// because esint must be loaded AFTER amslatex and mhchem loads amlatex
+	// we need to assure that mhchem is loaded before esint and every other
+	// package that redefines command of amsmath because mhchem loads amlatex
 	// (this info is from the author of mhchem from June 2013)
 	if (mustProvide("mhchem") &&
 	    params_.use_package("mhchem") != BufferParams::package_off)
 		packages << "\\PassOptionsToPackage{version=3}{mhchem}\n"
 			    "\\usepackage{mhchem}\n";
 
+	// wasysym is a simple feature, but it must be after amsmath if both
+	// are used
+	// wasysym redefines some integrals (e.g. iint) from amsmath. That
+	// leads to inconsistent integrals. We only load this package if
+	// the document does not contain integrals (then isRequired("esint")
+	// is false) or if esint is used, since esint redefines all relevant
+	// integral symbols from wasysym and amsmath.
+	// See http://www.lyx.org/trac/ticket/1942
+	if (mustProvide("wasysym") &&
+		params_.use_package("wasysym") != BufferParams::package_off &&
+		(params_.use_package("esint") != BufferParams::package_off || !isRequired("esint")))
+		packages << "\\usepackage{wasysym}\n"; 
+
 	// esint must be after amsmath (and packages requiring amsmath, like mhchem)
 	// and wasysym, since it will redeclare inconsistent integral symbols
 	if (mustProvide("esint") &&
@@ -1282,8 +1287,6 @@ bool LaTeXFeatures::needBabelLangOptions() const
 string const LaTeXFeatures::loadAMSPackages() const
 {
 	ostringstream tmp;
-	if (mustProvide("amsthm"))
-		tmp << "\\usepackage{amsthm}\n";
 
 	if (mustProvide("amsmath")
 	    && params_.use_package("amsmath") != BufferParams::package_off) {
@@ -1296,6 +1299,9 @@ string const LaTeXFeatures::loadAMSPackages() const
 			tmp << "\\usepackage{amstext}\n";
 	}
 
+	if (mustProvide("amsthm"))
+		tmp << "\\usepackage{amsthm}\n";
+
 	if (mustProvide("amssymb")
 	    && params_.use_package("amssymb") != BufferParams::package_off)
 		tmp << "\\usepackage{amssymb}\n";
@@ -1424,7 +1430,8 @@ docstring const getFloatI18nPreamble(docstring const & type,
 
 
 docstring const i18npreamble(docstring const & templ, Language const * lang,
-		Encoding const & enc, bool const polyglossia)
+                             Encoding const & enc, bool const polyglossia,
+                             bool const need_fixedwidth)
 {
 	if (templ.empty())
 		return templ;
@@ -1436,6 +1443,24 @@ docstring const i18npreamble(docstring const & templ, Language const * lang,
 	string const langenc = lang->encoding()->iconvName();
 	string const texenc = lang->encoding()->latexName();
 	string const bufenc = enc.iconvName();
+	Encoding const * testenc(&enc);
+	bool lang_fallback = false;
+	bool ascii_fallback = false;
+	if (need_fixedwidth && !enc.hasFixedWidth()) {
+		if (lang->encoding()->hasFixedWidth()) {
+			testenc = lang->encoding();
+			lang_fallback = true;
+		} else {
+			// We need a fixed width encoding, but both the buffer
+			// encoding and the language encoding are variable
+			// width. As a last fallback, try to convert to pure
+			// ASCII using the LaTeX commands defined in unicodesymbols.
+			testenc = encodings.fromLyXName("ascii");
+			if (!testenc)
+				return docstring();
+			ascii_fallback = true;
+		}
+	}
 	// First and second character of plane 15 (Private Use Area)
 	string const s1 = "\xf3\xb0\x80\x80"; // U+F0000
 	string const s2 = "\xf3\xb0\x80\x81"; // U+F0001
@@ -1449,16 +1474,18 @@ docstring const i18npreamble(docstring const & templ, Language const * lang,
 		docstring const name = lang->translateLayout(key);
 		// Check whether name can be encoded in the buffer encoding
 		bool encodable = true;
-		for (size_t i = 0; i < name.size(); ++i) {
-			if (!enc.encodable(name[i])) {
+		for (size_t i = 0; i < name.size() && encodable; ++i)
+			if (!testenc->encodable(name[i]))
 				encodable = false;
-				break;
-			}
-		}
-		string const translated = encodable ? to_utf8(name)
-			: "\\inputencoding{" + texenc + "}"
-			+ s1 + langenc + s2 + to_utf8(name)
-			+ s1 + bufenc + s2;
+		string translated;
+		if (encodable && !lang_fallback)
+			translated = to_utf8(name);
+		else if (ascii_fallback)
+			translated = to_ascii(testenc->latexString(name).first);
+		else
+			translated = "\\inputencoding{" + texenc + "}"
+				+ s1 + langenc + s2 + to_utf8(name)
+				+ s1 + bufenc + s2;
 		preamble = subst(preamble, sub.str(), translated);
 	}
 	return from_utf8(preamble);
@@ -1483,20 +1510,20 @@ docstring const LaTeXFeatures::getTClassI18nPreamble(bool use_babel, bool use_po
 		snippets.insert(i18npreamble(tclass[*cit].langpreamble(),
 						buffer().language(),
 						buffer().params().encoding(),
-						use_polyglossia));
+						use_polyglossia, false));
 		// commands for language changing (for multilanguage documents)
 		if ((use_babel || use_polyglossia) && !UsedLanguages_.empty()) {
 			snippets.insert(i18npreamble(
 						tclass[*cit].babelpreamble(),
 						buffer().language(),
 						buffer().params().encoding(),
-						use_polyglossia));
+						use_polyglossia, false));
 			for (lang_it lit = lbeg; lit != lend; ++lit)
 				snippets.insert(i18npreamble(
 						tclass[*cit].babelpreamble(),
 						*lit,
 						buffer().params().encoding(),
-						use_polyglossia));
+						use_polyglossia, false));
 		}
 	}
 	if ((use_babel || use_polyglossia) && !UsedLanguages_.empty()) {
@@ -1544,24 +1571,34 @@ docstring const LaTeXFeatures::getTClassI18nPreamble(bool use_babel, bool use_po
 		TextClass::InsetLayouts::const_iterator it = ils.find(*cit);
 		if (it == ils.end())
 			continue;
+		// The listings package does not work with variable width
+		// encodings, only with fixed width encodings. Therefore we
+		// need to force a fixed width encoding for
+		// \lstlistlistingname and \lstlistingname (bug 9382).
+		// This needs to be consistent with InsetListings::latex().
+		docstring const ilname = it->second.name();
+		bool const need_fixedwidth = !runparams_.isFullUnicode() &&
+				(ilname == "Listings" ||
+				 ilname == "Include:Listings" ||
+				 ilname == "TOC:Listings");
 		// language dependent commands (once per document)
 		snippets.insert(i18npreamble(it->second.langpreamble(),
 						buffer().language(),
 						buffer().params().encoding(),
-						use_polyglossia));
+						use_polyglossia, need_fixedwidth));
 		// commands for language changing (for multilanguage documents)
 		if ((use_babel || use_polyglossia) && !UsedLanguages_.empty()) {
 			snippets.insert(i18npreamble(
 						it->second.babelpreamble(),
 						buffer().language(),
 						buffer().params().encoding(),
-						use_polyglossia));
+						use_polyglossia, need_fixedwidth));
 			for (lang_it lit = lbeg; lit != lend; ++lit)
 				snippets.insert(i18npreamble(
 						it->second.babelpreamble(),
 						*lit,
 						buffer().params().encoding(),
-						use_polyglossia));
+						use_polyglossia, need_fixedwidth));
 		}
 	}
 
