@@ -18,14 +18,13 @@
 #ifndef LYXRC_H
 #define LYXRC_H
 
-#include "paper.h"
+#include "Length.h"
 
 #include "support/strfwd.h"
 
 #include <map>
 #include <set>
 #include <string>
-#include <vector>
 
 
 namespace lyx {
@@ -57,6 +56,7 @@ public:
 		RC_COMPLETION_INLINE_MATH,
 		RC_COMPLETION_INLINE_TEXT,
 		RC_COMPLETION_INLINE_DOTS,
+		RC_COMPLETION_MINLENGTH,
 		RC_COMPLETION_POPUP_DELAY,
 		RC_COMPLETION_POPUP_MATH,
 		RC_COMPLETION_POPUP_TEXT,
@@ -68,9 +68,9 @@ public:
 		RC_CURSOR_WIDTH,
 		RC_DATE_INSERT_FORMAT,
 		RC_DEFAULT_DECIMAL_POINT,
-		RC_DEFAULT_LANGUAGE,
+		RC_DEFAULT_LENGTH_UNIT,
+		RC_DEFAULT_OTF_VIEW_FORMAT,
 		RC_DEFAULT_VIEW_FORMAT,
-		RC_DEFAULT_PAPERSIZE,
 		RC_DEFFILE,
 		RC_DIALOGS_ICONIFY_WITH_MAIN,
 		RC_DISPLAY_GRAPHICS,
@@ -86,6 +86,7 @@ public:
 		RC_FORWARD_SEARCH_PDF,
 		RC_FULL_SCREEN_LIMIT,
 		RC_FULL_SCREEN_SCROLLBAR,
+		RC_FULL_SCREEN_STATUSBAR,
 		RC_FULL_SCREEN_TABBAR,
 		RC_FULL_SCREEN_MENUBAR,
 		RC_FULL_SCREEN_TOOLBARS,
@@ -115,7 +116,7 @@ public:
 		RC_LYXRCFORMAT,
 		RC_MACRO_EDIT_STYLE,
 		RC_MAC_DONTSWAP_CTRL_META,
-		RC_MAC_LIKE_WORD_MOVEMENT,
+		RC_MAC_LIKE_CURSOR_MOVEMENT,
 		RC_MAKE_BACKUP,
 		RC_MARK_FOREIGN_LANGUAGE,
 		RC_MOUSE_WHEEL_SPEED,
@@ -184,10 +185,13 @@ public:
 		RC_USE_SYSTEM_COLORS,
 		RC_USE_TOOLTIP,
 		RC_USE_PIXMAP_CACHE,
+		RC_USE_QIMAGE,
+		RC_USE_SYSTEM_THEME_ICONS,
 		RC_VIEWDVI_PAPEROPTION,
 		RC_VIEWER,
 		RC_VIEWER_ALTERNATIVES,
 		RC_VISUAL_CURSOR,
+		RC_CLOSE_BUFFER_WITH_LAST_VIEW,
 		RC_LAST
 	};
 
@@ -271,7 +275,6 @@ public:
 	/// option for telling the dvi viewer about the paper size
 	std::string view_dvi_paper_option;
 	/// default paper size for local xdvi/dvips/ghostview/whatever
-	PAPER_SIZE default_papersize;
 	/// command to run chktex incl. options
 	std::string chktex_command;
 	/// all available commands to run bibtex incl. options
@@ -361,6 +364,8 @@ public:
 	bool use_system_colors;
 	/// Use pixmap cache?
 	bool use_pixmap_cache;
+	/// Use QImage backend?
+	bool use_qimage;
 	/// Spellchecker engine: aspell, hunspell, etc
 	std::string spellchecker;
 	/// Alternate language for spellchecker
@@ -371,6 +376,8 @@ public:
 	bool spellcheck_continuously;
 	/// spellcheck notes and comments?
 	bool spellcheck_notes;
+	/// minimum length of words to complete
+	unsigned int completion_minlength;
 	///
 	bool use_kbmap;
 	///
@@ -413,10 +420,10 @@ public:
 	///
 	bool mark_foreign_language;
 	///
-	std::string default_language;
-	///
 	std::string gui_language;
 	///
+	std::string default_otf_view_format;
+	///
 	std::string default_view_format;
 	/// all available viewers
 	Alternatives viewer_alternatives;
@@ -425,7 +432,7 @@ public:
 	///
 	bool mac_dontswap_ctrl_meta;
 	///
-	bool mac_like_word_movement;
+	bool mac_like_cursor_movement;
 	///
 	bool cursor_follows_scrollbar;
 	///
@@ -462,6 +469,8 @@ public:
 	std::string user_email;
 	/// icon set name
 	std::string icon_set;
+	/// whether to use the icons from the theme
+	bool use_system_theme_icons;
 	/// True if the TeX engine cannot handle posix paths
 	bool windows_style_tex_paths;
 	/// True if the TeX engine can handle file names containing spaces
@@ -491,6 +500,8 @@ public:
 	bool full_screen_tabbar;
 	/// Toggle menubar in fullscreen mode?
 	bool full_screen_menubar;
+	/// Toggle statusbar in fullscreen mode?
+	bool full_screen_statusbar;
 	/// Limit the text width?
 	bool full_screen_limit;
 	/// Width of limited screen (in pixels) in fullscreen mode
@@ -530,6 +541,8 @@ public:
 	/// Default decimal point when aligning table columns on decimal
 	std::string default_decimal_point;
 	///
+	Length::UNIT default_length_unit;
+	///
 	enum ScrollWheelZoom {
 		SCROLL_WHEEL_ZOOM_OFF,
 		SCROLL_WHEEL_ZOOM_CTRL,
@@ -542,6 +555,8 @@ public:
 	bool force_paint_single_char;
 	///
 	int cursor_width;
+	/// One of: yes, no, ask
+	std::string close_buffer_with_last_view;
 };
 
 
