@@ -31,7 +31,7 @@ OutputParams::OutputParams(Encoding const * enc)
 	  dryrun(false), silent(false), pass_thru(false),
 	  html_disable_captions(false), html_in_par(false),
 	  html_make_pars(true), for_toc(false), for_tooltip(false),
-    for_search(false), includeall(false)
+	  for_search(false), for_preview(false), includeall(false)
 {
 	// Note: in PreviewLoader::Impl::dumpPreamble
 	// OutputParams runparams(0);
