@@ -1007,7 +1007,10 @@ void Text::insertChar(Cursor & cur, char_type c)
 
 	// Prevent to insert uncodable characters in verbatim and ERT
 	// (workaround for bug 9012)
-	if (cur.paragraph().isPassThru() && cur.current_font.language()) {
+	// Don't do it for listings inset, since InsetListings::latex() tries
+	// to switch to a usable encoding which works in many cases (bug 9102).
+	if (cur.paragraph().isPassThru() && owner_->lyxCode() != LISTINGS_CODE &&
+	    cur.current_font.language()) {
 		Encoding const * e = cur.current_font.language()->encoding();
 		if (!e->encodable(c)) {
 			cur.message(_("Character is uncodable in verbatim paragraphs."));
@@ -1272,8 +1275,7 @@ void Text::acceptOrRejectChanges(Cursor & cur, ChangeOp op)
 	LBUFERR(this == cur.text());
 
 	if (!cur.selection()) {
-		bool const changed = cur.paragraph().isChanged(cur.pos());
-		if (!(changed && findNextChange(&cur.bv())))
+		if (!selectChange(cur))
 			return;
 	}
 
@@ -1289,7 +1291,6 @@ void Text::acceptOrRejectChanges(Cursor & cur, ChangeOp op)
 	bool endsBeforeEndOfPar = (endPos < pars_[endPit].size());
 
 	// first, accept/reject changes within each individual paragraph (do not consider end-of-par)
-
 	for (pit_type pit = begPit; pit <= endPit; ++pit) {
 		pos_type parSize = pars_[pit].size();
 
@@ -1365,11 +1366,8 @@ void Text::acceptOrRejectChanges(Cursor & cur, ChangeOp op)
 	}
 
 	// finally, invoke the DEPM
-
 	deleteEmptyParagraphMechanism(begPit, endPit, cur.buffer()->params().trackChanges);
 
-	//
-
 	cur.finishUndo();
 	cur.clearSelection();
 	setCursorIntern(cur, begPit, begPos);
@@ -2120,7 +2118,7 @@ bool Text::completionSupported(Cursor const & cur) const
 
 CompletionList const * Text::createCompletionList(Cursor const & cur) const
 {
-	WordList const * list = theWordList(*cur.getFont().language());
+	WordList const * list = theWordList(cur.getFont().language()->lang());
 	return new TextCompletionList(cur, list);
 }
 
