@@ -200,6 +200,7 @@ void Text::setLayout(Cursor & cur, docstring const & layout)
 	pit_type end = cur.selEnd().pit() + 1;
 	cur.recordUndoSelection();
 	setLayout(start, end, layout);
+	cur.setCurrentFont();
 	cur.forceBufferUpdate();
 }
 
@@ -264,6 +265,11 @@ void Text::changeDepth(Cursor & cur, DEPTH_CHANGE type)
 void Text::setFont(Cursor & cur, Font const & font, bool toggleall)
 {
 	LASSERT(this == cur.text(), return);
+
+	// If there is a selection, record undo before the cursor font is changed.
+	if (cur.selection())
+		cur.recordUndoSelection();
+
 	// Set the current_font
 	// Determine basis font
 	FontInfo layoutfont;
@@ -289,7 +295,6 @@ void Text::setFont(Cursor & cur, Font const & font, bool toggleall)
 		return;
 
 	// Ok, we have a selection.
-	cur.recordUndoSelection();
 	Font newfont = font;
 
 	if (toggleall) {	
@@ -498,7 +503,7 @@ void Text::setParagraphs(Cursor & cur, docstring arg, bool merge)
 		// with same layout in a sequence.
 		// Do this only once for a selected range of paragraphs
 		// of the same layout and depth.
-		cur.recordUndo();
+		c.recordUndo();
 		par.params().apply(params, par.layout());
 		if (par.getDepth() != priordepth || par.layout() != priorlayout)
 			setLabelWidthStringToSequence(c, params.labelWidthString());
@@ -592,8 +597,6 @@ void Text::setCursorIntern(Cursor & cur,
 
 bool Text::checkAndActivateInset(Cursor & cur, bool front)
 {
-	if (cur.selection())
-		return false;
 	if (front && cur.pos() == cur.lastpos())
 		return false;
 	if (!front && cur.pos() == 0)
@@ -601,6 +604,8 @@ bool Text::checkAndActivateInset(Cursor & cur, bool front)
 	Inset * inset = front ? cur.nextInset() : cur.prevInset();
 	if (!inset || !inset->editable())
 		return false;
+	if (cur.selection() && cur.realAnchor().find(inset) == -1)
+		return false;
 	/*
 	 * Apparently, when entering an inset we are expected to be positioned
 	 * *before* it in the containing paragraph, regardless of the direction
@@ -611,14 +616,13 @@ bool Text::checkAndActivateInset(Cursor & cur, bool front)
 	if (!front)
 		--cur.pos();
 	inset->edit(cur, front);
+	cur.setCurrentFont();
 	return true;
 }
 
 
 bool Text::checkAndActivateInsetVisual(Cursor & cur, bool movingForward, bool movingLeft)
 {
-	if (cur.selection())
-		return false;
 	if (cur.pos() == -1)
 		return false;
 	if (cur.pos() == cur.lastpos())
@@ -627,8 +631,11 @@ bool Text::checkAndActivateInsetVisual(Cursor & cur, bool movingForward, bool mo
 	Inset * inset = par.isInset(cur.pos()) ? par.getInset(cur.pos()) : 0;
 	if (!inset || !inset->editable())
 		return false;
+	if (cur.selection() && cur.realAnchor().find(inset) == -1)
+		return false;
 	inset->edit(cur, movingForward, 
 		movingLeft ? Inset::ENTRY_DIRECTION_RIGHT : Inset::ENTRY_DIRECTION_LEFT);
+	cur.setCurrentFont();
 	return true;
 }
 
