@@ -616,26 +616,15 @@ void Text::dispatch(Cursor & cur, FuncRequest & cmd)
 		cur.screenUpdateFlags(Update::FitCursor);
 		break;
 
-	case LFUN_INSET_SELECT_ALL:
-		if (cur.depth() == 1 || !cur.selection() || !cur.selBegin().at_begin()
-			  || !cur.selEnd().at_end()) {
-			needsUpdate |= cur.selHandle(false);
-			needsUpdate |= cursorTop(cur);
-			needsUpdate |= cur.selHandle(true);
-			needsUpdate |= cursorBottom(cur);
-		} else
-			cur.undispatched();
-		cur.screenUpdateFlags(Update::FitCursor);
-		break;
-
 	case LFUN_CHAR_FORWARD:
-	case LFUN_CHAR_FORWARD_SELECT:
+	case LFUN_CHAR_FORWARD_SELECT: {
 		//LYXERR0(" LFUN_CHAR_FORWARD[SEL]:\n" << cur);
 		needsUpdate |= cur.selHandle(act == LFUN_CHAR_FORWARD_SELECT);
-		needsUpdate |= cursorForward(cur);
+		bool const cur_moved = cursorForward(cur);
+		needsUpdate |= cur_moved;
 
-		if (!needsUpdate && oldTopSlice == cur.top()
-				&& cur.boundary() == oldBoundary) {
+		if (!cur_moved && oldTopSlice == cur.top()
+			       && cur.boundary() == oldBoundary) {
 			cur.undispatched();
 			cmd = FuncRequest(LFUN_FINISHED_FORWARD);
 
@@ -654,15 +643,17 @@ void Text::dispatch(Cursor & cur, FuncRequest & cmd)
 			}
 		}
 		break;
+	}
 
 	case LFUN_CHAR_BACKWARD:
-	case LFUN_CHAR_BACKWARD_SELECT:
+	case LFUN_CHAR_BACKWARD_SELECT: {
 		//lyxerr << "handle LFUN_CHAR_BACKWARD[_SELECT]:\n" << cur << endl;
 		needsUpdate |= cur.selHandle(act == LFUN_CHAR_BACKWARD_SELECT);
-		needsUpdate |= cursorBackward(cur);
+		bool const cur_moved = cursorBackward(cur);
+		needsUpdate |= cur_moved;
 
-		if (!needsUpdate && oldTopSlice == cur.top()
-			&& cur.boundary() == oldBoundary) {
+		if (!cur_moved && oldTopSlice == cur.top()
+			       && cur.boundary() == oldBoundary) {
 			cur.undispatched();
 			cmd = FuncRequest(LFUN_FINISHED_BACKWARD);
 
@@ -681,14 +672,16 @@ void Text::dispatch(Cursor & cur, FuncRequest & cmd)
 			}
 		}
 		break;
+	}
 
 	case LFUN_CHAR_LEFT:
 	case LFUN_CHAR_LEFT_SELECT:
 		if (lyxrc.visual_cursor) {
 			needsUpdate |= cur.selHandle(act == LFUN_CHAR_LEFT_SELECT);
-			needsUpdate |= cursorVisLeft(cur);
-			if (!needsUpdate && oldTopSlice == cur.top()
-					&& cur.boundary() == oldBoundary) {
+			bool const cur_moved = cursorVisLeft(cur);
+			needsUpdate |= cur_moved;
+			if (!cur_moved && oldTopSlice == cur.top()
+				       && cur.boundary() == oldBoundary) {
 				cur.undispatched();
 				cmd = FuncRequest(LFUN_FINISHED_LEFT);
 			}
@@ -709,9 +702,10 @@ void Text::dispatch(Cursor & cur, FuncRequest & cmd)
 	case LFUN_CHAR_RIGHT_SELECT:
 		if (lyxrc.visual_cursor) {
 			needsUpdate |= cur.selHandle(cmd.action() == LFUN_CHAR_RIGHT_SELECT);
-			needsUpdate |= cursorVisRight(cur);
-			if (!needsUpdate && oldTopSlice == cur.top()
-					&& cur.boundary() == oldBoundary) {
+			bool const cur_moved = cursorVisRight(cur);
+			needsUpdate |= cur_moved;
+			if (!cur_moved && oldTopSlice == cur.top()
+				       && cur.boundary() == oldBoundary) {
 				cur.undispatched();
 				cmd = FuncRequest(LFUN_FINISHED_RIGHT);
 			}
@@ -820,9 +814,10 @@ void Text::dispatch(Cursor & cur, FuncRequest & cmd)
 	case LFUN_WORD_RIGHT_SELECT:
 		if (lyxrc.visual_cursor) {
 			needsUpdate |= cur.selHandle(cmd.action() == LFUN_WORD_RIGHT_SELECT);
-			needsUpdate |= cursorVisRightOneWord(cur);
-			if (!needsUpdate && oldTopSlice == cur.top()
-					&& cur.boundary() == oldBoundary) {
+			bool const cur_moved = cursorVisRightOneWord(cur);
+			needsUpdate |= cur_moved;
+			if (!cur_moved && oldTopSlice == cur.top()
+				       && cur.boundary() == oldBoundary) {
 				cur.undispatched();
 				cmd = FuncRequest(LFUN_FINISHED_RIGHT);
 			}
@@ -840,12 +835,13 @@ void Text::dispatch(Cursor & cur, FuncRequest & cmd)
 		break;
 
 	case LFUN_WORD_FORWARD:
-	case LFUN_WORD_FORWARD_SELECT:
+	case LFUN_WORD_FORWARD_SELECT: {
 		needsUpdate |= cur.selHandle(cmd.action() == LFUN_WORD_FORWARD_SELECT);
-		needsUpdate |= cursorForwardOneWord(cur);
+		bool const cur_moved = cursorForwardOneWord(cur);
+		needsUpdate |= cur_moved;
 
-		if (!needsUpdate && oldTopSlice == cur.top()
-				&& cur.boundary() == oldBoundary) {
+		if (!cur_moved && oldTopSlice == cur.top()
+			       && cur.boundary() == oldBoundary) {
 			cur.undispatched();
 			cmd = FuncRequest(LFUN_FINISHED_FORWARD);
 
@@ -864,14 +860,16 @@ void Text::dispatch(Cursor & cur, FuncRequest & cmd)
 			}
 		}
 		break;
+	}
 
 	case LFUN_WORD_LEFT:
 	case LFUN_WORD_LEFT_SELECT:
 		if (lyxrc.visual_cursor) {
 			needsUpdate |= cur.selHandle(cmd.action() == LFUN_WORD_LEFT_SELECT);
-			needsUpdate |= cursorVisLeftOneWord(cur);
-			if (!needsUpdate && oldTopSlice == cur.top()
-					&& cur.boundary() == oldBoundary) {
+			bool const cur_moved = cursorVisLeftOneWord(cur);
+			needsUpdate |= cur_moved;
+			if (!cur_moved && oldTopSlice == cur.top()
+				       && cur.boundary() == oldBoundary) {
 				cur.undispatched();
 				cmd = FuncRequest(LFUN_FINISHED_LEFT);
 			}
@@ -889,12 +887,13 @@ void Text::dispatch(Cursor & cur, FuncRequest & cmd)
 		break;
 
 	case LFUN_WORD_BACKWARD:
-	case LFUN_WORD_BACKWARD_SELECT:
+	case LFUN_WORD_BACKWARD_SELECT: {
 		needsUpdate |= cur.selHandle(cmd.action() == LFUN_WORD_BACKWARD_SELECT);
-		needsUpdate |= cursorBackwardOneWord(cur);
+		bool const cur_moved = cursorBackwardOneWord(cur);
+		needsUpdate |= cur_moved;
 
-		if (!needsUpdate && oldTopSlice == cur.top()
-				&& cur.boundary() == oldBoundary) {
+		if (!cur_moved && oldTopSlice == cur.top()
+			       && cur.boundary() == oldBoundary) {
 			cur.undispatched();
 			cmd = FuncRequest(LFUN_FINISHED_BACKWARD);
 
@@ -914,6 +913,7 @@ void Text::dispatch(Cursor & cur, FuncRequest & cmd)
 			}
 		}
 		break;
+	}
 
 	case LFUN_WORD_SELECT: {
 		selectWord(cur, WHOLE_WORD);
@@ -1514,7 +1514,7 @@ void Text::dispatch(Cursor & cur, FuncRequest & cmd)
 
 	case LFUN_MOUSE_DOUBLE:
 		if (cmd.button() == mouse_button::button1) {
-			selectWord(cur, WHOLE_WORD_STRICT);
+			selectWord(cur, WHOLE_WORD);
 			bv->cursor() = cur;
 		}
 		break;
@@ -2253,8 +2253,13 @@ void Text::dispatch(Cursor & cur, FuncRequest & cmd)
 				word = cur.selectionAsString(false);
 			}
 			lang = const_cast<Language *>(cur.getFont().language());
-		} else
+		} else if (cmd.getArg(1).empty()) {
+			// optional language argument is missing
+			// use the language at cursor position
+			lang = const_cast<Language *>(cur.getFont().language());
+		} else {
 			lang = const_cast<Language *>(languages.getLanguage(cmd.getArg(1)));
+		}
 		WordLangTuple wl(word, lang);
 		theSpellChecker()->insert(wl);
 		break;
@@ -2272,8 +2277,11 @@ void Text::dispatch(Cursor & cur, FuncRequest & cmd)
 				word = cur.selectionAsString(false);
 			}
 			lang = const_cast<Language *>(cur.getFont().language());
-		} else
+		} else if (cmd.getArg(1).empty()) {
+			lang = const_cast<Language *>(cur.getFont().language());
+		} else {
 			lang = const_cast<Language *>(languages.getLanguage(cmd.getArg(1)));
+		}
 		WordLangTuple wl(word, lang);
 		theSpellChecker()->accept(wl);
 		break;
@@ -2291,8 +2299,11 @@ void Text::dispatch(Cursor & cur, FuncRequest & cmd)
 				word = cur.selectionAsString(false);
 			}
 			lang = const_cast<Language *>(cur.getFont().language());
-		} else
+		} else if (cmd.getArg(1).empty()) {
+			lang = const_cast<Language *>(cur.getFont().language());
+		} else {
 			lang = const_cast<Language *>(languages.getLanguage(cmd.getArg(1)));
+		}
 		WordLangTuple wl(word, lang);
 		theSpellChecker()->remove(wl);
 		break;
@@ -2945,12 +2956,25 @@ bool Text::getStatus(Cursor & cur, FuncRequest const & cmd,
 	case LFUN_SPELLING_ADD:
 	case LFUN_SPELLING_IGNORE:
 	case LFUN_SPELLING_REMOVE:
-		enable = theSpellChecker();
+		enable = theSpellChecker() != NULL;
+		if (enable && !cmd.getArg(1).empty()) {
+			// validate explicitly given language
+			Language const * const lang = const_cast<Language *>(languages.getLanguage(cmd.getArg(1)));
+			enable &= lang != NULL;
+		}
 		break;
 
-	case LFUN_LAYOUT:
+	case LFUN_LAYOUT: {
 		enable = !cur.inset().forcePlainLayout();
+
+		docstring layout = cmd.argument();
+		if (layout.empty()) {
+			DocumentClass const & tclass = cur.buffer()->params().documentClass();
+			layout = tclass.defaultLayoutName();
+		}
+		flag.setOnOff(layout == cur.paragraph().layout().name());
 		break;
+	}
 
 	case LFUN_ENVIRONMENT_SPLIT: {
 		// FIXME This hardcoding is bad
@@ -3058,7 +3082,6 @@ bool Text::getStatus(Cursor & cur, FuncRequest const & cmd,
 	case LFUN_INSET_END:
 	case LFUN_INSET_BEGIN_SELECT:
 	case LFUN_INSET_END_SELECT:
-	case LFUN_INSET_SELECT_ALL:
 	case LFUN_PARAGRAPH_UP:
 	case LFUN_PARAGRAPH_DOWN:
 	case LFUN_LINE_BEGIN:
