@@ -1492,37 +1492,30 @@ Inset * TextMetrics::editXY(Cursor & cur, int x, int y,
 	int yy = y; // is modified by getPitAndRowNearY
 	Row const & row = getPitAndRowNearY(yy, pit, assert_in_view, up);
 
-	bool bound = false; // is modified by getColumnNearX
-	int xx = x; // is modified by getColumnNearX
-	pos_type const pos = row.pos()
-		+ getColumnNearX(pit, row, xx, bound);
 	cur.pit() = pit;
-	cur.pos() = pos;
-	cur.boundary(bound);
-	cur.setTargetX(x);
 
-	// try to descend into nested insets
-	Inset * inset = checkInsetHit(x, yy);
-	//lyxerr << "inset " << inset << " hit at x: " << x << " y: " << y << endl;
-	if (!inset) {
+	// Do we cover an inset?
+	InsetList::InsetTable * it = checkInsetHit(pit, x, yy);
+
+	if (!it) {
+		// No inset, set position in the text
+		bool bound = false; // is modified by getColumnNearX
+		int xx = x; // is modified by getColumnNearX
+		cur.pos() = row.pos()
+			+ getColumnNearX(pit, row, xx, bound);
+		cur.boundary(bound);
 		cur.setCurrentFont();
+		cur.setTargetX(xx);
 		return 0;
 	}
 
-	ParagraphList const & pars = text_->paragraphs();
-	Inset const * inset_before = pos ? pars[pit].getInset(pos - 1) : 0;
+	Inset * inset = it->inset;
+	//lyxerr << "inset " << inset << " hit at x: " << x << " y: " << y << endl;
 
-	// This should be just before or just behind the
-	// cursor position set above.
-	LASSERT(inset == inset_before
-		|| inset == pars[pit].getInset(pos), return 0);
-
-	// Make sure the cursor points to the position before
-	// this inset.
-	if (inset == inset_before) {
-		--cur.pos();
-		cur.boundary(false);
-	}
+	// Set position in front of inset
+	cur.pos() = it->pos;
+	cur.boundary(false);
+	cur.setTargetX(x);
 
 	// Try to descend recursively inside the inset.
 	inset = inset->editXY(cur, x, yy);
@@ -1571,11 +1564,8 @@ void TextMetrics::setCursorFromCoordinates(Cursor & cur, int const x, int const
 
 
 //takes screen x,y coordinates
-Inset * TextMetrics::checkInsetHit(int x, int y)
+InsetList::InsetTable * TextMetrics::checkInsetHit(pit_type pit, int x, int y)
 {
-	pit_type pit = getPitNearY(y);
-	LASSERT(pit != -1, return 0);
-
 	Paragraph const & par = text_->paragraphs()[pit];
 	ParagraphMetrics const & pm = par_metrics_[pit];
 
@@ -1604,7 +1594,7 @@ Inset * TextMetrics::checkInsetHit(int x, int y)
 			&& y >= p.y_ - dim.asc
 			&& y <= p.y_ + dim.des) {
 			LYXERR(Debug::DEBUG, "Hit inset: " << inset);
-			return inset;
+			return const_cast<InsetList::InsetTable *>(&(*iit));
 		}
 	}
 
@@ -1613,6 +1603,20 @@ Inset * TextMetrics::checkInsetHit(int x, int y)
 }
 
 
+//takes screen x,y coordinates
+Inset * TextMetrics::checkInsetHit(int x, int y)
+{
+	pit_type const pit = getPitNearY(y);
+	LASSERT(pit != -1, return 0);
+	InsetList::InsetTable * it = checkInsetHit(pit, x, y);
+
+	if (!it)
+		return 0;
+
+	return it->inset;
+}
+
+
 int TextMetrics::cursorX(CursorSlice const & sl,
 		bool boundary) const
 {
@@ -1892,6 +1896,15 @@ int TextMetrics::leftMargin(int max_width,
 		if (newpar != pit_type(pars.size())) {
 			if (pars[newpar].layout().isEnvironment()) {
 				l_margin = leftMargin(max_width, newpar);
+				// Remove the parindent that has been added
+				// if the paragraph was empty.
+				if (pars[newpar].empty() &&
+				    buffer.params().paragraph_separation ==
+				    BufferParams::ParagraphIndentSeparation) {
+					docstring pi = pars[newpar].layout().parindent;
+					l_margin -= theFontMetrics(
+						buffer.params().getFont()).signedWidth(pi);
+				}
 			}
 			if (tclass.isDefaultLayout(par.layout())
 			    || tclass.isPlainLayout(par.layout())) {
@@ -1903,12 +1916,20 @@ int TextMetrics::leftMargin(int max_width,
 		}
 	}
 
-	// This happens after sections in standard classes. The 1.3.x
-	// code compared depths too, but it does not seem necessary
-	// (JMarc)
-	if (tclass.isDefaultLayout(par.layout())
-	    && pit > 0 && pars[pit - 1].layout().nextnoindent)
+	// This happens after sections or environments in standard classes.
+	// We have to check the previous layout at same depth.
+	if (buffer.params().paragraph_separation ==
+			BufferParams::ParagraphSkipSeparation)
 		parindent.erase();
+	else if (pit > 0 && pars[pit - 1].getDepth() >= par.getDepth()) {
+		pit_type prev = text_->depthHook(pit, par.getDepth());
+		if (par.layout() == pars[prev].layout()) {
+			if (prev != pit - 1
+			    && pars[pit - 1].layout().nextnoindent)
+				parindent.erase();
+		} else if (pars[prev].layout().nextnoindent)
+			parindent.erase();
+	}
 
 	FontInfo const labelfont = text_->labelFont(par);
 	FontMetrics const & labelfont_metrics = theFontMetrics(labelfont);
