@@ -14,9 +14,9 @@
 #define THESAURUS_H
 
 #include "support/docstring.h"
+#include "WordLangTuple.h"
 
 #include <map>
-#include <string>
 #include <vector>
 
 namespace lyx {
@@ -38,7 +38,7 @@ public:
 	/**
 	 * look up some text in the thesaurus
 	 */
-	Meanings lookup(docstring const & text, docstring const & lang);
+	Meanings lookup(WordLangTuple const & wl);
 	/** check if a thesaurus for a given language \p lang is available
 	 *  (installed and loaded)
 	 */
