@@ -160,13 +160,17 @@ public:
 	/// Push an item on to the stack, deleting the bottom group on
 	/// overflow.
 	void push(UndoElement const & v) {
-		c_.push_front(v);
-		if (c_.size() > limit_) {
+		// Remove some entries if the limit has been reached.
+		// However, if the only group on the stack is the one
+		// we are currently populating, do nothing.
+		if (c_.size() >= limit_
+		    && c_.front().group_id != v.group_id) {
 			// remove a whole group at once.
 			const size_t gid = c_.back().group_id;
 			while (!c_.empty() && c_.back().group_id == gid)
 				c_.pop_back();
 		}
+		c_.push_front(v);
 	}
 
 	/// Mark all the elements of the stack as dirty
@@ -392,6 +396,7 @@ void Undo::Private::doTextUndoOrRedo(CursorData & cur, UndoElementStack & stack,
 	// We will store in otherstack the part of the document under 'undo'
 	DocIterator cell_dit = undo.cell.asDocIterator(&buffer_);
 
+	LATTEST(undo.end <= cell_dit.lastpit());
 	doRecordUndo(ATOMIC_UNDO, cell_dit,
 		undo.from, cell_dit.lastpit() - undo.end, undo.cur_after,
 		undo.isFullBuffer, otherstack);
@@ -583,5 +588,18 @@ void Undo::recordUndoFullDocument(CursorData const & cur)
 	endUndoGroup();
 }
 
+/// UndoGroupHelper class stuff
+
+void UndoGroupHelper::resetBuffer(Buffer * buf)
+{
+	if (buf == buffer_)
+		return;
+	if (buffer_)
+		buffer_->undo().endUndoGroup();
+	buffer_ = buf;
+	if (buffer_)
+		buffer_->undo().beginUndoGroup();
+}
+
 
 } // namespace lyx
