@@ -47,6 +47,7 @@
 #include "support/filetools.h"
 #include "support/gettext.h"
 #include "support/lstrings.h"
+#include "support/mutex.h"
 #include "support/textutils.h"
 
 using namespace std;
@@ -173,7 +174,9 @@ Buffer * newUnnamedFile(FileName const & path, string const & prefix,
 						string const & templatename)
 {
 	static map<string, int> file_number;
+	static Mutex mutex;
 
+	Mutex::Locker locker(&mutex);
 	FileName filename;
 
 	do {
