@@ -55,8 +55,8 @@ public:
 };
 
 
-FileDialog::FileDialog(QString const & t, FuncCode s)
-	: private_(new FileDialog::Private), title_(t), success_(s)
+FileDialog::FileDialog(QString const & t)
+	: private_(new FileDialog::Private), title_(t)
 {}
 
 
@@ -81,7 +81,8 @@ void FileDialog::setButton2(QString const & label, QString const & dir)
 
 
 FileDialog::Result FileDialog::save(QString const & path,
-	QStringList const & filters, QString const & suggested)
+	QStringList const & filters, QString const & suggested,
+	QString * selectedFilter)
 {
 	LYXERR(Debug::GUI, "Select with path \"" << path
 			   << "\", mask \"" << filters.join(";;")
@@ -94,19 +95,19 @@ FileDialog::Result FileDialog::save(QString const & path,
 	QString const startsWith = makeAbsPath(suggested, path);
 	QString const name = 
 		QFileDialog::getSaveFileName(qApp->focusWidget(),
-	     title_, startsWith, filters.join(";;"),
-			 0, QFileDialog::DontConfirmOverwrite);
+		title_, startsWith, filters.join(";;"),
+		selectedFilter, QFileDialog::DontConfirmOverwrite);
 	if (name.isNull())
 		result.first = FileDialog::Later;
 	else
 		result.second = toqstr(os::internal_path(fromqstr(name)));
 #else
 	LyXFileDialog dlg(title_, path, filters, private_->b1, private_->b2);
-#if QT_VERSION != 0x040203
 	dlg.setFileMode(QFileDialog::AnyFile);
-#endif
 	dlg.setAcceptMode(QFileDialog::AcceptSave);
 	dlg.setConfirmOverwrite(false);
+	if (selectedFilter != 0 && !selectedFilter->isEmpty())
+		dlg.selectNameFilter(*selectedFilter);
 
 	if (!suggested.isEmpty())
 		dlg.selectFile(suggested);
@@ -118,12 +119,21 @@ FileDialog::Result FileDialog::save(QString const & path,
 		result.second = internalPath(dlg.selectedFiles()[0]);
 	else
 		result.first = FileDialog::Later;
+	if (selectedFilter != 0)
+		*selectedFilter = dlg.selectedNameFilter();
 	dlg.hide();
 #endif
 	return result;
 }
 
 
+FileDialog::Result FileDialog::save(QString const & path,
+	QStringList const & filters, QString const & suggested)
+{
+	return save(path, filters, suggested, 0);
+}
+
+
 FileDialog::Result FileDialog::open(QString const & path,
 	QStringList const & filters, QString const & suggested)
 {
