@@ -185,14 +185,15 @@ frontend::Application * createApplication(int & argc, char * argv[])
 void setLocale()
 {
 	QLocale theLocale;
+	string code;
 	if (lyxrc.gui_language == "auto") {
 		theLocale = QLocale::system();
+		code = fromqstr(theLocale.name());
 	} else {
 		Language const * l = languages.getLanguage(lyxrc.gui_language);
-		string const code = l ? l->code() : string();
+		code = l ? l->code() : "C";
 		theLocale = QLocale(toqstr(code));
 	}
-	string const code = fromqstr(theLocale.name());
 	// Qt tries to outsmart us and transforms en_US to C.
 	Messages::guiLanguage((code == "C") ? "en_US" : code);
 	QLocale::setDefault(theLocale);
@@ -333,12 +334,16 @@ PngMap sorted_png_map[] = {
 	{ "XBox", "xbox3" },
 	{ "Xbox", "xbox2" },
 	{ "Xi", "xi2" },
+	{ "lVert", "vert2" },
+	{ "lvert", "vert" }, 
 	{ "nLeftarrow", "nleftarrow2" },
 	{ "nLeftrightarrow", "nleftrightarrow2" },
 	{ "nRightarrow", "nrightarrow2" },
 	{ "nVDash", "nvdash3" },
 	{ "nVdash", "nvdash4" },
 	{ "nvDash", "nvdash2" },
+	{ "rVert", "vert2" },
+	{ "rvert", "vert" },
 	{ "textrm \\AA", "textrm_AA"},
 	{ "textrm \\O", "textrm_O"},
 	{ "vDash", "vdash2" },
@@ -1067,6 +1072,12 @@ docstring Application::iconName(FuncRequest const & f, bool unknown)
 }
 
 
+docstring Application::mathIcon(docstring const & c)
+{
+	return qstring_to_ucs4(findPng(toqstr(c)));
+}
+
+
 FuncStatus GuiApplication::getStatus(FuncRequest const & cmd) const
 {
 	FuncStatus status;
@@ -2649,10 +2660,11 @@ void GuiApplication::commitData(QSessionManager & sm)
 
 void GuiApplication::unregisterView(GuiView * gv)
 {
-	LAPPERR(d->views_[gv->id()] == gv);
-	d->views_.remove(gv->id());
-	if (current_view_ == gv)
-		current_view_ = 0;
+	if(d->views_.contains(gv->id()) && d->views_.value(gv->id()) == gv) {
+		d->views_.remove(gv->id());
+		if (current_view_ == gv)
+			current_view_ = 0;
+	}
 }
 
 
