@@ -137,6 +137,8 @@ public:
 		// get icon from cache
 		QPixmap scaled;
 		QString const name = ":" + toqstr(list_->icon(index.row()));
+		if (name == ":")
+			return scaled;
 		if (!QPixmapCache::find("completion" + name, scaled)) {
 			// load icon from disk
 			QPixmap p = QPixmap(name);
