@@ -156,7 +156,7 @@ void GuiPainter::enterMonochromeMode(Color const & min, Color const & max)
 
 void GuiPainter::leaveMonochromeMode()
 {
-	LASSERT(!monochrome_min_.empty(), /**/);
+	LASSERT(!monochrome_min_.empty(), return);
 	monochrome_min_.pop();
 	monochrome_max_.pop();
 }
@@ -352,6 +352,12 @@ int GuiPainter::text(int x, int y, docstring const & s,
 	// same as that of a soft-hyphen (0x00ad), unless it
 	// occurs at a line-break. As a kludge, we force Qt to
 	// render this glyph using a one-column line.
+	// This is needed for some math glyphs.
+	// FIXME In texted, this behaves differently depending
+	// on lyxrc.force_paint_single_char status.
+	// Should the soft hyphen char be displayed at all?
+	// I don't think so (i.e., Qt is correct as far as
+	// texted is concerned). /spitz
 	if (s.size() == 1 && str[0].unicode() == 0x00ad) {
 		setQPainterPen(computeColor(f.realColor()));
 		QTextLayout adsymbol(str);
