@@ -16,10 +16,8 @@
 
 #include "GuiDialog.h"
 
-#include "ColorCode.h"
 #include "Converter.h"
 #include "Format.h"
-#include "FuncCode.h"
 #include "KeyMap.h"
 #include "LyXRC.h"
 #include "Mover.h"
@@ -104,11 +102,6 @@ public:
 	/// update the screen fonts after change
 	void updateScreenFonts();
 
-	/// adjust the prefs paper sizes
-	PAPER_SIZE toPaperSize(int i) const;
-	/// adjust the prefs paper sizes
-	int fromPaperSize(PAPER_SIZE papersize) const;
-
 	LyXRC & rc() { return rc_; }
 	Converters & converters() { return converters_; }
 	Formats & formats() { return formats_; }
@@ -130,7 +123,6 @@ private:
 	/// A list of colors to be dispatched
 	std::vector<std::string> colors_;
 
-	bool redraw_gui_;
 	bool update_screen_font_;
 };
 
@@ -360,9 +352,10 @@ Q_SIGNALS:
 
 private Q_SLOTS:
 	void on_copierED_textEdited(const QString & s);
-	void on_extensionED_textEdited(const QString &);
+	void on_extensionsED_textEdited(const QString &);
 	void on_viewerED_textEdited(const QString &);
 	void on_editorED_textEdited(const QString &);
+	void on_mimeED_textEdited(const QString &);
 	void on_shortcutED_textEdited(const QString &);
 	void on_formatED_editingFinished();
 	void on_formatED_textChanged(const QString &);
