@@ -198,7 +198,7 @@ class GuiSymbols::Model : public QAbstractItemModel
 {
 public:
 	Model(GuiSymbols * parent)
-		: QAbstractItemModel(parent), parent_(parent)
+		: QAbstractItemModel(parent)
 	{}
 
 	QModelIndex index(int row, int column, QModelIndex const &) const
@@ -255,8 +255,7 @@ public:
 
 private:
 	friend class GuiSymbols;
-	GuiSymbols * parent_;
-	
+
 	QList<char_type> symbols_;
 };
 
