@@ -17,14 +17,6 @@
 
 #include "DockView.h"
 
-#include "insets/InsetCommandParams.h"
-
-#include "TocBackend.h"
-
-#include <QStandardItemModel>
-
-#include <vector>
-
 namespace lyx {
 namespace frontend {
 
