@@ -77,8 +77,6 @@ public:
 
 	int id() const { return id_; }
 
-	///
-	void setBusy(bool);
 	/// are we busy ?
 	bool busy() const;
 
@@ -137,13 +135,6 @@ public:
 	GuiToolbar * makeToolbar(ToolbarInfo const & tbinfo, bool newline);
 	void updateStatusBar();
 
-#if defined(Q_WS_MACX) && defined(QT_MAC_USE_COCOA)
-	// We have to share one menu bar, so we must create a menu bar that does not have a parent.
-	// To enforce this we override the QMainWindow::menuBar() implementation.
-	// See https://bugreports.qt-project.org/browse/QTBUG-25399
-	QMenuBar *menuBar() const;
-#endif
-
 	/// updates the possible layouts selectable
 	void updateLayoutList();
 	void updateToolbars();
@@ -154,7 +145,7 @@ public:
 
 	/// hides the workarea and makes sure it is clean
 	bool hideWorkArea(GuiWorkArea * wa);
-	/// closes the workarea
+	/// closes workarea; close buffer only if no other workareas point to it
 	bool closeWorkArea(GuiWorkArea * wa);
 	/// closes the buffer
 	bool closeBuffer(Buffer & buf);
@@ -214,6 +205,8 @@ Q_SIGNALS:
 	void triggerShowDialog(QString const & qname, QString const & qdata, Inset * inset);
 
 public Q_SLOTS:
+	///
+	void setBusy(bool);
 	/// idle timeout.
 	/// clear any temporary message and replace with current status.
 	void clearMessage();
@@ -236,7 +229,6 @@ private Q_SLOTS:
 
 	/// For completion of autosave or export threads.
 	void processingThreadStarted();
-	void processingThreadFinished(bool show_errors);
 	void processingThreadFinished();
 	void autoSaveThreadFinished();
 
@@ -352,6 +344,8 @@ private:
 	///
 	void initToolbars();
 	///
+	void initToolbar(std::string const & name);
+	///
 	bool lfunUiToggle(std::string const & ui_component);
 	///
 	void toggleFullScreen();
@@ -360,7 +354,12 @@ private:
 	///
 	void insertPlaintextFile(docstring const & fname,
 		bool asParagraph);
+	/// Open Export As ... dialog. \p iformat is the format the
+	/// filter is initially set to.
+	bool exportBufferAs(Buffer & b, docstring const & iformat);
 
+	///
+	enum RenameKind { LV_WRITE_AS, LV_VC_RENAME, LV_VC_COPY };
 	/// Save a buffer as a new file. 
 	/**
 	Write a buffer to a new file name and rename the buffer
@@ -375,8 +374,14 @@ private:
     If 'newname' is non-empty and has an absolute path, that is used.
     Otherwise the base directory of the buffer is used as the base
     for any relative path in 'newname'.
-	*/
-	bool renameBuffer(Buffer & b, docstring const & newname);
+
+	 \p kind controls what is done besides the pure renaming:
+         * LV_WRITE_AS  => The buffer is written without version control actions.
+         * LV_VC_RENAME => The file is renamed in version control.
+         * LV_VC_COPY   => The file is copied in version control.
+	 */
+	bool renameBuffer(Buffer & b, docstring const & newname,
+	                  RenameKind kind = LV_WRITE_AS);
 	///
 	bool saveBuffer(Buffer & b);
 	/// save and rename buffer to fn. If fn is empty, the buffer
