@@ -32,6 +32,7 @@
 
 #include <boost/crc.hpp>
 
+#include <QBoxLayout>
 #include <QSettings>
 #include <QTextCursor>
 #include <QTextDocument>
@@ -55,6 +56,8 @@ ViewSourceWidget::ViewSourceWidget()
 		updatePB, SLOT(setDisabled(bool)));
 	connect(autoUpdateCB, SIGNAL(toggled(bool)),
 		this, SLOT(updateView()));
+	connect(masterPerspectiveCB, SIGNAL(toggled(bool)),
+		this, SLOT(updateView()));
 	connect(updatePB, SIGNAL(clicked()),
 		this, SLOT(updateView()));
 	connect(outputFormatCO, SIGNAL(activated(int)),
@@ -91,7 +94,8 @@ static size_t crcCheck(docstring const & s)
 	\return true if the content has changed since last call.
  */
 static bool getContent(BufferView const * view, Buffer::OutputWhat output,
-		       QString & qstr, string const format, bool force_getcontent)
+		       QString & qstr, string const format, bool force_getcontent,
+		       bool master)
 {
 	// get the *top* level paragraphs that contain the cursor,
 	// or the selected text
@@ -108,7 +112,8 @@ static bool getContent(BufferView const * view, Buffer::OutputWhat output,
 	if (par_begin > par_end)
 		swap(par_begin, par_end);
 	odocstringstream ostr;
-	view->buffer().getSourceCode(ostr, format, par_begin, par_end + 1, output);
+	view->buffer().getSourceCode(ostr, format, par_begin, par_end + 1,
+				     output, master);
 	docstring s = ostr.str();
 	static size_t crc = 0;
 	size_t newcrc = crcCheck(s);
@@ -143,6 +148,7 @@ void ViewSourceWidget::setViewFormat()
 	updateView();
 }
 
+
 void ViewSourceWidget::updateView()
 {
 	if (!bv_) {
@@ -164,7 +170,8 @@ void ViewSourceWidget::updateView()
 	else if (contentsCO->currentIndex() == 3)
 		output = Buffer::OnlyBody;
 
-	if (getContent(bv_, output, content, format, force_getcontent_))
+	if (getContent(bv_, output, content, format,
+		      force_getcontent_, masterPerspectiveCB->isChecked()))
 		document_->setPlainText(content);
 
 	CursorSlice beg = bv_->cursor().selectionBegin().bottom();
@@ -221,11 +228,24 @@ void ViewSourceWidget::updateDefaultFormat()
 }
 
 
+void ViewSourceWidget::resizeEvent (QResizeEvent * event)
+{
+	QSize const & formSize = formLayout->sizeHint();
+	// minimize the size of the part that contains the buttons
+	if (width() * formSize.height() < height() * formSize.width()) {
+		layout_->setDirection(QBoxLayout::TopToBottom);
+	} else {
+		layout_->setDirection(QBoxLayout::LeftToRight);
+	}
+	QWidget::resizeEvent(event);
+}
+
+
 GuiViewSource::GuiViewSource(GuiView & parent,
 		Qt::DockWidgetArea area, Qt::WindowFlags flags)
 	: DockView(parent, "view-source", qt_("LaTeX Source"), area, flags)
 {
-	widget_ = new ViewSourceWidget();
+	widget_ = new ViewSourceWidget;
 	setWidget(widget_);
 }
 
@@ -242,6 +262,7 @@ void GuiViewSource::updateView()
 		widget_->setBufferView(bufferview());
 		widget_->updateView();
 	}
+	widget_->masterPerspectiveCB->setEnabled(buffer().parent());
 }
 
 
@@ -272,7 +293,7 @@ QString GuiViewSource::title() const
 		case LITERATE:
 			return qt_("Literate Source");
 	}
-	LASSERT(false, /**/);
+	LATTEST(false);
 	return QString();
 }
 
