@@ -44,6 +44,8 @@
 #include <fstream>
 #include <iomanip>
 
+#include <QTimer>
+
 using namespace std;
 using namespace lyx::support;
 
@@ -207,6 +209,8 @@ public:
 	/// \p wait whether to wait for the process to complete or, instead,
 	/// to do it in the background.
 	void startLoading(bool wait = false);
+	///
+	void refreshPreviews();
 
 	/// Emit this signal when an image is ready for display.
 	boost::signal<void(PreviewImage const &)> imageReady;
@@ -245,6 +249,12 @@ private:
 	PreviewLoader & parent_;
 	///
 	Buffer const & buffer_;
+	///
+	mutable int font_scaling_factor_;
+	///
+	QTimer * delay_refresh_;
+	///
+	bool finished_generating_;
 
 	/// We don't own this
 	static lyx::Converter const * pconverter_;
@@ -299,6 +309,12 @@ void PreviewLoader::startLoading(bool wait) const
 }
 
 
+void PreviewLoader::refreshPreviews()
+{
+	pimpl_->refreshPreviews();
+}
+
+
 boost::signals::connection PreviewLoader::connect(slot_type const & slot) const
 {
 	return pimpl_->imageReady.connect(slot);
@@ -387,15 +403,25 @@ namespace lyx {
 namespace graphics {
 
 PreviewLoader::Impl::Impl(PreviewLoader & p, Buffer const & b)
-	: parent_(p), buffer_(b)
+	: parent_(p), buffer_(b), finished_generating_(true)
 {
+	font_scaling_factor_ = buffer_.isExporting()
+		? int(75.0 * buffer_.params().html_math_img_scale)
+		: int(0.01 * lyxrc.dpi * lyxrc.zoom * lyxrc.preview_scale_factor);
 	if (!pconverter_)
 		pconverter_ = setConverter("lyxpreview");
+
+	delay_refresh_ = new QTimer(&parent_);
+	delay_refresh_->setSingleShot(true);
+	QObject::connect(delay_refresh_, SIGNAL(timeout()),
+	                 &parent_, SLOT(refreshPreviews()));
 }
 
 
 PreviewLoader::Impl::~Impl()
 {
+	delete delay_refresh_;
+
 	InProgressProcesses::iterator ipit  = in_progress_.begin();
 	InProgressProcesses::iterator ipend = in_progress_.end();
 
@@ -407,11 +433,42 @@ PreviewLoader::Impl::~Impl()
 PreviewImage const *
 PreviewLoader::Impl::preview(string const & latex_snippet) const
 {
+	int fs = buffer_.isExporting()
+		? int(75.0 * buffer_.params().html_math_img_scale)
+		: int(0.01 * lyxrc.dpi * lyxrc.zoom * lyxrc.preview_scale_factor);
+	if (font_scaling_factor_ != fs) {
+		// Schedule refresh of all previews on zoom changes.
+		// The previews are regenerated only after the zoom factor
+		// has not been changed for about 1 second.
+		delay_refresh_->start(1000);
+	}
+	// Don't try to access the cache until we are done.
+	if (delay_refresh_->isActive() || !finished_generating_)
+		return 0;
 	Cache::const_iterator it = cache_.find(latex_snippet);
 	return (it == cache_.end()) ? 0 : it->second.get();
 }
 
 
+void PreviewLoader::Impl::refreshPreviews()
+{
+	font_scaling_factor_ = buffer_.isExporting()
+		? int(75.0 * buffer_.params().html_math_img_scale)
+		: int(0.01 * lyxrc.dpi * lyxrc.zoom * lyxrc.preview_scale_factor);
+	// Reschedule refresh until the previous process completed.
+	if (!finished_generating_) {
+		delay_refresh_->start(1000);
+		return;
+	}
+	Cache::const_iterator cit = cache_.begin();
+	Cache::const_iterator cend = cache_.end();
+	while (cit != cend)
+		parent_.remove((cit++)->first);
+	finished_generating_ = false;
+	buffer_.updatePreviews();
+}
+
+
 namespace {
 
 class FindSnippet {
@@ -568,8 +625,6 @@ void PreviewLoader::Impl::startLoading(bool wait)
 	}
 	of << "\\batchmode\n";
 	dumpPreamble(os);
-	// handle inputenc etc.
-	buffer_.params().writeEncodingPreamble(os, features);
 	of << "\n\\begin{document}\n";
 	dumpData(of, inprogress.snippets);
 	of << "\n\\end{document}\n";
@@ -579,16 +634,15 @@ void PreviewLoader::Impl::startLoading(bool wait)
 					 << "File was not closed properly.");
 		return;
 	}
-
-	double const font_scaling_factor = 
-		buffer_.isExporting() ? 75.0 * buffer_.params().html_math_img_scale 
-			: 0.01 * lyxrc.dpi * lyxrc.zoom * lyxrc.preview_scale_factor;
 	
 	// The conversion command.
 	ostringstream cs;
 	cs << pconverter_->command
 	   << " " << quoteName(latexfile.toFilesystemEncoding())
-	   << " --dpi " << int(font_scaling_factor);
+	   << " --dpi " << font_scaling_factor_;
+	if (lyxerr.debugging(Debug::GRAPHICS)) {
+		cs << " --verbose";
+	}
 
 	// FIXME XHTML 
 	// The colors should be customizable.
@@ -655,6 +709,7 @@ void PreviewLoader::Impl::finishedGenerating(pid_t pid, int retval)
 	if (git == in_progress_.end()) {
 		lyxerr << "PreviewLoader::finishedGenerating(): unable to find "
 			"data for PID " << pid << endl;
+		finished_generating_ = true;
 		return;
 	}
 
@@ -663,8 +718,11 @@ void PreviewLoader::Impl::finishedGenerating(pid_t pid, int retval)
 	LYXERR(Debug::GRAPHICS, "PreviewLoader::finishedInProgress("
 				<< retval << "): processing " << status
 				<< " for " << command);
-	if (retval > 0)
+	if (retval > 0) {
+		in_progress_.erase(git);
+		finished_generating_ = true;
 		return;
+	}
 
 	// Read the metrics file, if it exists
 	vector<double> ascent_fractions(git->second.snippets.size());
@@ -684,7 +742,8 @@ void PreviewLoader::Impl::finishedGenerating(pid_t pid, int retval)
 		double af = ascent_fractions[metrics_counter];
 
 		// Add the image to the cache only if it's actually present
-		if (file.isReadableFile()) {
+		// and not empty (an empty image is signaled by af < 0)
+		if (af >= 0 && file.isReadableFile()) {
 			PreviewImagePtr ptr(new PreviewImage(parent_, snip, file, af));
 			cache_[snip] = ptr;
 
@@ -704,6 +763,7 @@ void PreviewLoader::Impl::finishedGenerating(pid_t pid, int retval)
 	for (; nit != nend; ++nit) {
 		imageReady(*nit->get());
 	}
+	finished_generating_ = true;
 }
 
 
@@ -768,3 +828,5 @@ void PreviewLoader::Impl::dumpData(odocstream & os,
 
 } // namespace graphics
 } // namespace lyx
+
+#include "moc_PreviewLoader.cpp"
