@@ -94,6 +94,9 @@ static void build_translator()
 	insetnames[FLOAT_CODE] = InsetName("float", _("Float"));
 	insetnames[WRAP_CODE] = InsetName("wrap");
 	insetnames[SPECIALCHAR_CODE] = InsetName("specialchar");
+	insetnames[IPA_CODE] = InsetName("ipa");
+	insetnames[IPACHAR_CODE] = InsetName("ipachar");
+	insetnames[IPADECO_CODE] = InsetName("ipadeco");
 	insetnames[TABULAR_CODE] = InsetName("tabular", _("Table"));
 	insetnames[EXTERNAL_CODE] = InsetName("external");
 	insetnames[CAPTION_CODE] = InsetName("caption");
@@ -123,6 +126,8 @@ static void build_translator()
 	insetnames[MATH_BOLDSYMBOL_CODE] = InsetName("mathboldsymbol");
 	insetnames[MATH_BOX_CODE] = InsetName("mathbox");
 	insetnames[MATH_BRACE_CODE] = InsetName("mathbrace");
+	insetnames[MATH_CANCEL_CODE] = InsetName("mathcancel");
+	insetnames[MATH_CANCELTO_CODE] = InsetName("mathcancelto");
 	insetnames[MATH_CASES_CODE] = InsetName("mathcases");
 	insetnames[MATH_CHAR_CODE] = InsetName("mathchar");
 	insetnames[MATH_COLOR_CODE] = InsetName("mathcolor");
@@ -190,7 +195,7 @@ Buffer & Inset::buffer()
 		LYXERR0("Inset: " << this << " LyX Code: " << lyxCode()
 					<< " name: " << iname);
 		s << "LyX Code: " << lyxCode() << " name: " << iname;
-		LASSERT(false, /**/);
+		LATTEST(false);
 		throw ExceptionMessage(BufferException, 
 			from_ascii("Inset::buffer_ member not initialized!"), s.str());
 	}
@@ -258,15 +263,15 @@ void Inset::forToc(docstring &, size_t) const
 }
 
 
-docstring Inset::contextMenu(BufferView const &, int, int) const
+string Inset::contextMenu(BufferView const &, int, int) const
 {
 	return contextMenuName();
 }
 
 
-docstring Inset::contextMenuName() const
+string Inset::contextMenuName() const
 {
-	return docstring();
+	return string();
 }
 
 
@@ -345,7 +350,7 @@ void Inset::doDispatch(Cursor & cur, FuncRequest &cmd)
 		// if the derived inset did not explicitly handle mouse_release,
 		// we assume we request the settings dialog
 		if (!cur.selection() && cmd.button() == mouse_button::button1
-			  && hasSettings()) {
+		    && clickable(cmd.x(), cmd.y()) && hasSettings()) {
 			FuncRequest tmpcmd(LFUN_INSET_SETTINGS);
 			dispatch(cur, tmpcmd);
 		}
@@ -406,12 +411,17 @@ bool Inset::getStatus(Cursor &, FuncRequest const & cmd,
 		} else {
 			return false;
 		}
-	
+
 	case LFUN_IN_MATHMACROTEMPLATE:
 		// By default we're not in a MathMacroTemplate inset
 		flag.setEnabled(false);
 		return true;
 
+	case LFUN_IN_IPA:
+		// By default we're not in an IPA inset
+		flag.setEnabled(false);
+		return true;
+
 	default:
 		break;
 	}
@@ -632,6 +642,12 @@ Buffer const * Inset::updateFrontend() const
 }
 
 
+bool Inset::resetFontEdit() const
+{
+	return getLayout().resetsFont() || !inheritFont();
+}
+
+
 docstring Inset::completionPrefix(Cursor const &) const 
 {
 	return docstring();
