@@ -263,10 +263,6 @@ public:
 	virtual row_type row(idx_type) const { return 0; }
 	/// cell index corresponding to row and column;
 	virtual idx_type index(row_type row, col_type col) const;
-	/// any additional x-offset when drawing a cell?
-	virtual int cellXOffset(idx_type) const { return 0; }
-	/// any additional y-offset when drawing a cell?
-	virtual int cellYOffset(idx_type) const { return 0; }
 	/// number of embedded cells
 	virtual size_t nargs() const { return 0; }
 	/// number of rows in gridlike structures
@@ -404,6 +400,9 @@ public:
 		{ return true; }
 	/// Is the width forced to some value?
 	virtual bool hasFixedWidth() const { return false; }
+	/// if this inset has paragraphs should they be forced to use a
+	/// local font language switch?
+	virtual bool forceLocalFontSwitch() const { return false; }
 
 	/// Is the content of this inset part of the output document?
 	virtual bool producesOutput() const { return true; }
