@@ -67,7 +67,8 @@ private:
 	///
 	void latex(otexstream &, OutputParams const &) const;
 	///
-	int plaintext(odocstream &, OutputParams const &) const;
+	int plaintext(odocstringstream & ods, OutputParams const & op,
+	              size_t max_length = INT_MAX) const;
 	///
 	int docbook(odocstream &, OutputParams const &) const;
 	///
@@ -79,18 +80,18 @@ private:
 	///
 	void validate(LaTeXFeatures &) const;
 	///
-	docstring contextMenuName() const;
+	std::string contextMenuName() const;
 	///
-	void addToToc(DocIterator const &) const;
+	void addToToc(DocIterator const & di, bool output_active) const;
 	///
 	InsetBranchParams const & params() const { return params_; }
 	///
 	void setParams(InsetBranchParams const & params) { params_ = params; }
 
 	/** \returns true if params_.branch is listed as 'selected' in
-	    \c buffer. This handles the case of child documents.
+		\c buffer. \p child only checks within child documents.
 	 */
-	bool isBranchSelected() const;
+	bool isBranchSelected(bool const child = false) const;
 	/*!
 	 * Is the content of this inset part of the output document?
 	 *
@@ -109,7 +110,7 @@ private:
 	///
 	void doDispatch(Cursor & cur, FuncRequest & cmd);
 	///
-	docstring layoutName() const { return from_ascii("Branch"); }
+	docstring layoutName() const { return from_ascii("Branch:") + branch(); }
 	///
 	Inset * clone() const { return new InsetBranch(*this); }
 
