@@ -15,8 +15,6 @@
 
 #include "InsetCommand.h"
 
-#include "Citation.h"
-
 namespace lyx {
 
 /////////////////////////////////////////////////////////////////////////
@@ -34,6 +32,9 @@ public:
 	///
 	~InsetCitation();
 
+	///
+	bool addKey(std::string const & key);
+
 	/// \name Public functions inherited from Inset class
 	//@{
 	///
@@ -49,7 +50,8 @@ public:
 	///
 	void latex(otexstream &, OutputParams const &) const;
 	///
-	int plaintext(odocstream &, OutputParams const &) const;
+	int plaintext(odocstringstream & ods, OutputParams const & op,
+	              size_t max_length = INT_MAX) const;
 	///
 	int docbook(odocstream &, OutputParams const &) const;
 	///
@@ -59,23 +61,20 @@ public:
 	///
 	void forToc(docstring &, size_t) const;
 	///
-	void validate(LaTeXFeatures &) const;
+	void validate(LaTeXFeatures &) const {}
 	///
 	void updateBuffer(ParIterator const & it, UpdateType);
 	///
-	void addToToc(DocIterator const &) const;
+	void addToToc(DocIterator const & di, bool output_active) const;
 	///
-	docstring contextMenuName() const;
+	std::string contextMenuName() const;
 	//@}
 
 	/// \name Static public methods obligated for InsetCommand derived classes
 	//@{
 	///
 	static ParamInfo const & findInfo(std::string const &);
-	// FIXME This is the locus of the design problem we have.
-	// It really ought to do what default_cite_command() does,
-	// but to do that it needs to know what CiteEngine we are
-	// using.
+	///
 	static std::string defaultCommand() { return "cite"; }
 	///
 	static bool isCompatibleCommand(std::string const & cmd);
@@ -101,13 +100,13 @@ private:
 	docstring screenLabel() const;
 	//@}
 
-	/// we'll eventually want to be able to get info on this from the 
+	/// we'll eventually want to be able to get info on this from the
 	/// various CiteEngines
 	static ParamInfo param_info_;
 
 	///
 	struct Cache {
-		Cache() : recalculate(true) {};
+		Cache() : recalculate(true) {}
 		///
 		bool recalculate;
 		///
