@@ -19,7 +19,9 @@
 #include "Cursor.h"
 #include "FuncRequest.h"
 #include "FuncStatus.h"
+#include "Language.h"
 #include "Lexer.h"
+#include "ParIterator.h"
 #include "TextClass.h"
 
 #include "support/gettext.h"
@@ -33,9 +35,7 @@ namespace lyx {
 
 InsetFlex::InsetFlex(Buffer * buf, string const & layoutName)
 	: InsetCollapsable(buf), name_(layoutName)
-{
-	status_= Collapsed;
-}
+{}
 
 
 InsetFlex::InsetFlex(InsetFlex const & in)
@@ -43,6 +43,7 @@ InsetFlex::InsetFlex(InsetFlex const & in)
 {}
 
 
+// special code for InsetFlex when there is not the explicit Flex:: prefix
 InsetLayout const & InsetFlex::getLayout() const
 {
 	if (!buffer_)
@@ -56,14 +57,6 @@ InsetLayout const & InsetFlex::getLayout() const
 }
 
 
-bool InsetFlex::resetFontEdit() const
-{
-	if (getLayout().resetsFont())
-		return true;
-	return InsetCollapsable::resetFontEdit();
-}
-
-
 InsetLayout::InsetDecoration InsetFlex::decoration() const
 {
 	InsetLayout::InsetDecoration const dec = getLayout().decoration();
@@ -125,4 +118,33 @@ void InsetFlex::doDispatch(Cursor & cur, FuncRequest & cmd)
 }
 
 
+void InsetFlex::updateBuffer(ParIterator const & it, UpdateType utype)
+{
+	BufferParams const & bp = buffer().masterBuffer()->params();
+	InsetLayout const & il = getLayout();
+	docstring custom_label = translateIfPossible(il.labelstring());
+
+	Counters & cnts = bp.documentClass().counters();
+	docstring const & count = il.counter();
+	bool const have_counter = cnts.hasCounter(count);
+	if (have_counter) {
+		cnts.step(count, utype);
+		custom_label += ' ' +
+			cnts.theCounter(count, it.paragraph().getParLanguage(bp)->code());
+	}
+	setLabel(custom_label);
+	
+	bool const save_counter = have_counter && utype == OutputUpdate;
+	if (save_counter) {
+		// we assume the counter is local to this inset
+		// if this turns out to be wrong in some case, we will
+		// need a layout flag
+		cnts.saveLastCounter();
+	}
+	InsetCollapsable::updateBuffer(it, utype);
+	if (save_counter)
+		cnts.restoreLastCounter();
+}
+
+
 } // namespace lyx
