@@ -30,8 +30,6 @@ public:
 	InsetLayout const & getLayout() const;
 	///
 	InsetCode lyxCode() const { return FLEX_CODE; }
-	///
-	bool resetFontEdit() const;
 	/// Default looks
 	InsetLayout::InsetDecoration decoration() const;
 	///
@@ -45,6 +43,8 @@ public:
 		FuncStatus &) const;
 	///
 	void doDispatch(Cursor & cur, FuncRequest & cmd);
+	///
+	void updateBuffer(ParIterator const & it, UpdateType utype);
 
 protected:
 	///
