@@ -18,12 +18,15 @@
 #include "Font.h"
 #include "MetricsInfo.h"
 
+#include "support/lstrings.h"
+
 #include <iostream>
 
 using namespace std;
 
 namespace lyx {
 
+using support::token;
 
 InsetFootlike::InsetFootlike(Buffer * buf)
 	: InsetCollapsable(buf)
@@ -50,14 +53,16 @@ void InsetFootlike::draw(PainterInfo & pi, int x, int y) const
 
 void InsetFootlike::write(ostream & os) const
 {
-	os << to_utf8(layoutName()) << "\n";
+	// The layoutName may contain a "InTitle" qualifier
+	os << to_utf8(token(layoutName(), char_type(':'), 0)) << "\n";
 	InsetCollapsable::write(os);
 }
 
 
 bool InsetFootlike::insetAllowed(InsetCode code) const
 {
-	if (code == FOOT_CODE || code == MARGIN_CODE || code == FLOAT_CODE)
+	if (code == FOOT_CODE || code == MARGIN_CODE || code == FLOAT_CODE
+		|| code == LISTINGS_CODE)
 		return false;
 	return InsetCollapsable::insetAllowed(code);
 }
