@@ -110,7 +110,7 @@ string findTargetFormat(string const & format, OutputParams const & runparams)
 		Format const * const f = formats.getFormat(format);
 		// Convert vector graphics to pdf
 		if (f && f->vectorFormat())
-			return "pdf";
+			return "pdf6";
 		// pdflatex can use jpeg, png and pdf directly
 		if (format == "jpg")
 			return format;
@@ -487,10 +487,10 @@ copyToDirIfNeeded(DocFileName const & file, string const & dir)
 	string const file_in = file.absFileName();
 	string const only_path = onlyPath(file_in);
 	if (rtrim(onlyPath(file_in) , "/") == rtrim(dir, "/"))
-		return make_pair(IDENTICAL_PATHS, file_in);
+		return make_pair(IDENTICAL_PATHS, FileName(file_in));
 
 	string mangled = file.mangledFileName();
-	if (file.isZipped()) {
+	if (formats.isZippedFile(file)) {
 		// We need to change _eps.gz to .eps.gz. The mangled name is
 		// still unique because of the counter in mangledFileName().
 		// We can't just call mangledFileName() with the zip
@@ -616,7 +616,7 @@ string InsetGraphics::prepareFile(OutputParams const & runparams) const
 	// If the file is compressed and we have specified that it
 	// should not be uncompressed, then just return its name and
 	// let LaTeX do the rest!
-	if (params().filename.isZipped()) {
+	if (formats.isZippedFile(params().filename)) {
 		if (params().noUnzip) {
 			// We don't know whether latex can actually handle
 			// this file, but we can't check, because that would
@@ -755,10 +755,20 @@ void InsetGraphics::latex(otexstream & os,
 
 	bool const file_exists = !params().filename.empty()
 			&& params().filename.isReadableFile();
-	string const message = file_exists ?
-		string() : string("bb = 0 0 200 100, draft, type=eps");
-	// if !message.empty() then there was no existing file
-	// "filename" found. In this case LaTeX
+	string message;
+	if (!file_exists) {
+		if (params().bb.empty())
+		    message = "bb = 0 0 200 100";
+		if (!params().draft) {
+			if (!message.empty())
+				message += ", ";
+			message += "draft";
+		}
+		if (!message.empty())
+			message += ", ";
+		message += "type=eps";
+	}
+	// If no existing file "filename" was found LaTeX
 	// draws only a rectangle with the above bb and the
 	// not found filename in it.
 	LYXERR(Debug::GRAPHICS, "\tMessage = \"" << message << '\"');
@@ -789,7 +799,8 @@ void InsetGraphics::latex(otexstream & os,
 	// Convert the file if necessary.
 	// Remove the extension so LaTeX will use whatever is appropriate
 	// (when there are several versions in different formats)
-	latex_str += prepareFile(runparams);
+	string file_path = prepareFile(runparams);
+	latex_str += file_path;
 	latex_str += '}' + after;
 	// FIXME UNICODE
 	os << from_utf8(latex_str);
@@ -798,7 +809,8 @@ void InsetGraphics::latex(otexstream & os,
 }
 
 
-int InsetGraphics::plaintext(odocstream & os, OutputParams const &) const
+int InsetGraphics::plaintext(odocstringstream & os,
+        OutputParams const &, size_t) const
 {
 	// No graphics in ascii output. Possible to use gifscii to convert
 	// images to ascii approximation.
@@ -998,9 +1010,7 @@ void InsetGraphics::validate(LaTeXFeatures & features) const
 	features.require("graphicx");
 
 	if (features.runparams().nice) {
-		Buffer const * masterBuffer = features.buffer().masterBuffer();
-		string const rel_file = removeExtension(
-			params().filename.relFileName(masterBuffer->filePath()));
+		string const rel_file = params().filename.onlyFileNameWithoutExt();
 		if (contains(rel_file, "."))
 			features.require("lyxdot");
 	}
@@ -1037,19 +1047,17 @@ void InsetGraphics::editGraphics(InsetGraphicsParams const & p) const
 }
 
 
-void InsetGraphics::addToToc(DocIterator const & cpit) const
+void InsetGraphics::addToToc(DocIterator const & cpit, bool output_active) const
 {
-	TocBackend & backend = buffer().tocBackend();
-
 	//FIXME UNICODE
 	docstring const str = from_utf8(params_.filename.onlyFileName());
-	backend.toc("graphics").push_back(TocItem(cpit, 0, str));
+	buffer().tocBackend().toc("graphics").push_back(TocItem(cpit, 0, str, output_active));
 }
 
 
-docstring InsetGraphics::contextMenuName() const
+string InsetGraphics::contextMenuName() const
 {
-	return from_ascii("context-graphics");
+	return "context-graphics";
 }
 
 
@@ -1155,7 +1163,7 @@ void unifyGraphicsGroups(Buffer & b, string const & argument)
 			InsetGraphics & ins = static_cast<InsetGraphics &>(*it);
 			InsetGraphicsParams inspar = ins.getParams();
 			if (params.groupId == inspar.groupId) {
-				b.undo().recordUndo(it);
+				b.undo().recordUndo(CursorData(it));
 				params.filename = inspar.filename;
 				ins.setParams(params);
 			}
