@@ -23,7 +23,6 @@
 #include "OutputParams.h"
 #include "output_xhtml.h"
 
-#include "support/debug.h"
 #include "support/docstream.h"
 #include "support/FileName.h"
 #include "support/filetools.h"
@@ -189,40 +188,17 @@ void InsetHyperlink::latex(otexstream & os,
 			(pos = name.find('~', i)) != string::npos;
 			i = pos + 1)
 			name.replace(pos, 1, sim);
-		docstring name_latex;
-		docstring uncodable;
-		// validate encoding
-		for (size_t n = 0; n < name.size(); ++n) {
-			try {
-				char_type const c = name[n];
-				docstring const latex = runparams.encoding->latexChar(c);
-				name_latex += latex;
-				if (latex.length() > 1 && latex[latex.length() - 1] != '}') {
-					// Prevent eating of a following
-					// space or command corruption by
-					// following characters
-					name_latex += "{}";
-				}
-			} catch (EncodingException & /* e */) {
-				LYXERR0("Uncodable character in href inset!");
-				if (runparams.dryrun) {
-					name_latex += "<" + _("LyX Warning: ")
-						   + _("uncodable character") + " '";
-					name_latex += docstring(1, name[n]);
-					name_latex += "'>";
-				} else
-					uncodable += name[n];
-			}
-		}
-		if (!uncodable.empty()) {
+		pair<docstring, docstring> name_latexed =
+			runparams.encoding->latexString(name, runparams.dryrun);
+		name = name_latexed.first;
+		if (!name_latexed.second.empty() && !runparams.silent) {
 			// issue a warning about omitted characters
 			// FIXME: should be passed to the error dialog
 			frontend::Alert::warning(_("Uncodable characters"),
 				bformat(_("The following characters that are used in the href inset are not\n"
 					  "representable in the current encoding and therefore have been omitted:\n%1$s."),
-					uncodable));
+					name_latexed.second));
 		}
-		name = name_latex;
 	}  // end if (!name.empty())
 	
 	if (runparams.moving_arg)
@@ -233,7 +209,8 @@ void InsetHyperlink::latex(otexstream & os,
 }
 
 
-int InsetHyperlink::plaintext(odocstream & os, OutputParams const &) const
+int InsetHyperlink::plaintext(odocstringstream & os,
+        OutputParams const &, size_t) const
 {
 	odocstringstream oss;
 
@@ -274,7 +251,9 @@ docstring InsetHyperlink::xhtml(XHTMLStream & xs, OutputParams const &) const
 
 void InsetHyperlink::toString(odocstream & os) const
 {
-	plaintext(os, OutputParams(0));
+	odocstringstream ods;
+	plaintext(ods, OutputParams(0), INT_MAX);
+	os << ods.str();
 }
 
 
@@ -308,9 +287,9 @@ void InsetHyperlink::validate(LaTeXFeatures & features) const
 }
 
 
-docstring InsetHyperlink::contextMenuName() const
+string InsetHyperlink::contextMenuName() const
 {
-	return from_ascii("context-hyperlink");
+	return "context-hyperlink";
 }
 
 
