@@ -61,6 +61,7 @@
 #include "support/lassert.h"
 #include "support/lstrings.h" // contains
 #include "support/lyxalgo.h"
+#include "support/mutex.h"
 
 #include "support/bind.h"
 
@@ -77,6 +78,8 @@ namespace {
 docstring const uniqueID()
 {
 	static unsigned int seed = 1000;
+	static Mutex mutex;
+	Mutex::Locker lock(&mutex);
 	return "file" + convert<docstring>(++seed);
 }
 
@@ -553,7 +556,8 @@ void InsetInclude::latex(otexstream & os, OutputParams const & runparams) const
 			from_utf8(incfile), true);
 	}
 
-	FileName const writefile(makeAbsPath(mangled, masterBuffer->temppath()));
+	FileName const writefile(makeAbsPath(mangled, runparams.for_preview ?
+						 buffer().temppath() : masterBuffer->temppath()));
 
 	LYXERR(Debug::LATEX, "incfile:" << incfile);
 	LYXERR(Debug::LATEX, "exportfile:" << exportfile);
@@ -1027,6 +1031,12 @@ void InsetInclude::draw(PainterInfo & pi, int x, int y) const
 }
 
 
+void InsetInclude::write(ostream & os) const
+{
+	params().Write(os, &buffer());
+}
+
+
 string InsetInclude::contextMenuName() const
 {
 	return "context-include";
@@ -1083,6 +1093,7 @@ docstring latexString(InsetInclude const & inset)
 	// by latex() anyway.
 	OutputParams runparams(0);
 	runparams.flavor = OutputParams::LATEX;
+	runparams.for_preview = true;
 	inset.latex(os, runparams);
 
 	return ods.str();
