@@ -22,6 +22,7 @@
 #include "FuncRequest.h"
 #include "FuncStatus.h"
 #include "IndicesList.h"
+#include "Language.h"
 #include "LaTeXFeatures.h"
 #include "Lexer.h"
 #include "output_latex.h"
@@ -37,8 +38,8 @@
 
 #include "frontends/alert.h"
 
-#include <ostream>
 #include <algorithm>
+#include <ostream>
 
 using namespace std;
 using namespace lyx::support;
@@ -65,7 +66,7 @@ void InsetIndex::latex(otexstream & os, OutputParams const & runparams_in) const
 	if (buffer().masterBuffer()->params().use_indices && !params_.index.empty()
 	    && params_.index != "idx") {
 		os << "\\sindex[";
-		os << params_.index;
+		os << escape(params_.index);
 		os << "]{";
 	} else {
 		os << "\\index";
@@ -134,15 +135,11 @@ void InsetIndex::latex(otexstream & os, OutputParams const & runparams_in) const
 			// the sorting part are representable in the current
 			// encoding. If not try the LaTeX macro which might
 			// or might not be a good choice, and issue a warning.
-			docstring spart2;
-			for (size_t n = 0; n < spart.size(); ++n) {
-				try {
-					spart2 += runparams.encoding->latexChar(spart[n]);
-				} catch (EncodingException & /* e */) {
+			pair<docstring, docstring> spart_latexed =
+				runparams.encoding->latexString(spart, runparams.dryrun);
+			if (!spart_latexed.second.empty())
 					LYXERR0("Uncodable character in index entry. Sorting might be wrong!");
-				}
-			}
-			if (spart != spart2 && !runparams.dryrun) {
+			if (spart != spart_latexed.first && !runparams.dryrun) {
 				// FIXME: warning should be passed to the error dialog
 				frontend::Alert::warning(_("Index sorting failed"),
 				bformat(_("LyX's automatic index sorting algorithm faced\n"
@@ -152,7 +149,7 @@ void InsetIndex::latex(otexstream & os, OutputParams const & runparams_in) const
 			}
 			// remove remaining \'s for the sorting part
 			docstring const ppart =
-				subst(spart2, from_ascii("\\"), docstring());
+				subst(spart_latexed.first, from_ascii("\\"), docstring());
 			os << ppart;
 			os << '@';
 		}
@@ -351,15 +348,18 @@ void InsetIndex::string2params(string const & in, InsetIndexParams & params)
 }
 
 
-void InsetIndex::addToToc(DocIterator const & cpit) const
+void InsetIndex::addToToc(DocIterator const & cpit, bool output_active) const
 {
 	DocIterator pit = cpit;
 	pit.push_back(CursorSlice(const_cast<InsetIndex &>(*this)));
 	docstring str;
+	string type = "index";
+	if (buffer().masterBuffer()->params().use_indices)
+		type += ":" + to_utf8(params_.index);
 	text().forToc(str, 0);
-	buffer().tocBackend().toc("index").push_back(TocItem(pit, 0, str));
+	buffer().tocBackend().toc(type).push_back(TocItem(pit, 0, str, output_active));
 	// Proceed with the rest of the inset.
-	InsetCollapsable::addToToc(cpit);
+	InsetCollapsable::addToToc(cpit, output_active);
 }
 
 
@@ -373,9 +373,9 @@ void InsetIndex::validate(LaTeXFeatures & features) const
 }
 
 
-docstring InsetIndex::contextMenuName() const
+string InsetIndex::contextMenuName() const
 {
-	return from_ascii("context-index");
+	return "context-index";
 }
 
 
@@ -429,7 +429,8 @@ ParamInfo const & InsetPrintIndex::findInfo(string const & /* cmdName */)
 {
 	static ParamInfo param_info_;
 	if (param_info_.empty()) {
-		param_info_.add("type", ParamInfo::LATEX_OPTIONAL);
+		param_info_.add("type", ParamInfo::LATEX_OPTIONAL,
+                        ParamInfo::HANDLING_ESCAPE);
 		param_info_.add("name", ParamInfo::LATEX_REQUIRED);
 	}
 	return param_info_;
@@ -573,10 +574,10 @@ void InsetPrintIndex::validate(LaTeXFeatures & features) const
 }
 
 
-docstring InsetPrintIndex::contextMenuName() const
+string InsetPrintIndex::contextMenuName() const
 {
 	return buffer().masterBuffer()->params().use_indices ?
-		from_ascii("context-indexprint") : docstring();
+		"context-indexprint" : string();
 }
 
 
@@ -691,17 +692,24 @@ docstring InsetPrintIndex::xhtml(XHTMLStream &, OutputParams const & op) const
 	if (toc.empty())
 		return docstring();
 
-	// Collection the index entries in a form we can use them.
+	// Collect the index entries in a form we can use them.
 	Toc::const_iterator it = toc.begin();
 	Toc::const_iterator const en = toc.end();
 	vector<IndexEntry> entries;
 	for (; it != en; ++it)
-		entries.push_back(IndexEntry(it->str(), it->dit()));
+		if (it->isOutput())
+			entries.push_back(IndexEntry(it->str(), it->dit()));
+
+	if (entries.empty())
+		// not very likely that all the index entries are in notes or
+		// whatever, but....
+		return docstring();
+
 	stable_sort(entries.begin(), entries.end());
 
 	Layout const & lay = bp.documentClass().htmlTOCLayout();
 	string const & tocclass = lay.defaultCSSClass();
-	string const tocattr = "class='tochead " + tocclass + "'";
+	string const tocattr = "class='index " + tocclass + "'";
 
 	// we'll use our own stream, because we are going to defer everything.
 	// that's how we deal with the fact that we're probably inside a standard
@@ -709,9 +717,10 @@ docstring InsetPrintIndex::xhtml(XHTMLStream &, OutputParams const & op) const
 	odocstringstream ods;
 	XHTMLStream xs(ods);
 
-	xs << html::StartTag("div", "class='index'");
+	xs << html::StartTag("div", tocattr);
 	xs << html::StartTag(lay.htmltag(), lay.htmlattr()) 
-		 << _("Index") 
+		 << translateIfPossible(from_ascii("Index"),
+	                          op.local_font->language()->lang())
 		 << html::EndTag(lay.htmltag());
 	xs << html::StartTag("ul", "class='main'");
 	Font const dummy;
