@@ -39,6 +39,7 @@
 #include "support/filetools.h"
 #include "support/gettext.h"
 #include "support/lstrings.h"
+#include "support/Translator.h"
 
 #include <sstream>
 
@@ -380,7 +381,7 @@ void InsetInfo::updateInfo()
 			// do not insert > for the top level menu item
 			if (it != beg)
 				par.insertInset(par.size(), new InsetSpecialChar(InsetSpecialChar::MENU_SEPARATOR),
-						Change(Change::UNCHANGED));
+						f, Change(Change::UNCHANGED));
 			//FIXME: add proper underlines here. This
 			// involves rewriting searchMenu used above to
 			// return a vector of menus. If we do not do
@@ -404,6 +405,12 @@ void InsetInfo::updateInfo()
 		// installed one is preferred anyway, and all icons that are
 		// embedded in the resources are installed as well.
 		FileName file(to_utf8(icon_name));
+		if (file.onlyFileNameWithoutExt() == "unknown") {
+			string dir = "images";
+			FileName file2(imageLibFileSearch(dir, name_, "png"));
+			if (!file2.empty())
+				file = file2;
+		}
 		if (!file.exists())
 			break;
 		InsetGraphics * inset = new InsetGraphics(buffer_);
@@ -411,7 +418,8 @@ void InsetInfo::updateInfo()
 		igp.filename = file;
 		inset->setParams(igp);
 		clear();
-		paragraphs().front().insertInset(0, inset, 
+		Font const f(inherit_font, buffer().params().language);
+		paragraphs().front().insertInset(0, inset, f,
 						 Change(Change::UNCHANGED));
 		break;
 	}
@@ -461,7 +469,7 @@ void InsetInfo::updateInfo()
 }
 
 
-docstring InsetInfo::contextMenu(BufferView const &, int, int) const
+string InsetInfo::contextMenu(BufferView const &, int, int) const
 {
 	//FIXME: We override the implementation of InsetCollapsable,
 	//because this inset is not a collapsable inset.
@@ -469,9 +477,9 @@ docstring InsetInfo::contextMenu(BufferView const &, int, int) const
 }
 
 
-docstring InsetInfo::contextMenuName() const
+string InsetInfo::contextMenuName() const
 {
-	return from_ascii("context-info");
+	return "context-info";
 }
 
 
