@@ -15,6 +15,7 @@
 
 #include "ColorCode.h"
 #include "FontInfo.h"
+#include "Layout.h"
 
 #include "support/docstring.h"
 
@@ -75,13 +76,32 @@ public:
 	///
 	std::string latexparam() const { return latexparam_; }
 	///
+	docstring leftdelim() const { return leftdelim_; }
+	///
+	docstring rightdelim() const { return rightdelim_; }
+	///
 	FontInfo font() const { return font_; }
 	///
 	FontInfo labelfont() const { return labelfont_; }
 	///
 	ColorCode bgcolor() const { return bgcolor_; }
 	///
+	Layout::LaTeXArgMap args() const;
+	///
+	Layout::LaTeXArgMap latexargs() const { return latexargs_; }
+	///
+	Layout::LaTeXArgMap postcommandargs() const { return postcommandargs_; }
+	///
+	unsigned int optArgs() const;
+	///
+	unsigned int requiredArgs() const;
+	///
 	docstring preamble() const { return preamble_; }
+	/// Get language dependent macro definitions needed for this inset
+	docstring const langpreamble() const { return langpreamble_; }
+	/// Get language and babel dependent macro definitions needed for
+	/// this inset
+	docstring const babelpreamble() const { return babelpreamble_; }
 	///
 	docstring counter() const { return counter_; }
 	///
@@ -146,6 +166,8 @@ public:
 	bool resetsFont() const { return resetsfont_; }
 	///
 	bool isDisplay() const { return display_; }
+	///
+	bool forcelocalfontswitch() const { return forcelocalfontswitch_; }
 private:
 	///
 	void makeDefaultCSS() const;
@@ -154,6 +176,8 @@ private:
 	///
 	std::string defaultCSSLabelClass() const { return defaultCSSClass() + "_label"; }
 	///
+	void readArgument(Lexer &);
+	///
 	docstring name_;
 	/**
 		* This is only used (at present) to decide where to put them on the menus.
@@ -174,6 +198,10 @@ private:
 	///
 	std::string latexparam_;
 	///
+	docstring leftdelim_;
+	///
+	docstring rightdelim_;
+	///
 	FontInfo font_;
 	///
 	FontInfo labelfont_;
@@ -183,6 +211,10 @@ private:
 	docstring counter_;
 	///
 	docstring preamble_;
+	/// Language dependent macro definitions needed for this inset
+	docstring langpreamble_;
+	/// Language and babel dependent macro definitions needed for this inset
+	docstring babelpreamble_;
 	///
 	docstring refprefix_;
 	///
@@ -236,10 +268,17 @@ private:
 	bool resetsfont_;
 	///
 	bool display_;
+	///
+	bool forcelocalfontswitch_;
+	///
+	Layout::LaTeXArgMap latexargs_;
+	///
+	Layout::LaTeXArgMap postcommandargs_;
 };
 
 ///
 InsetLayout::InsetLyXType translateLyXType(std::string const & str);
+InsetLayout::InsetDecoration translateDecoration(std::string const & str);
 
 } // namespace lyx
 
