@@ -31,11 +31,12 @@ public:
 	///
 	docstring layoutName() const { return from_ascii("Marginal"); }
 	///
-	int plaintext(odocstream &, OutputParams const & runparams) const;
+	int plaintext(odocstringstream & ods, OutputParams const & op,
+	              size_t max_length = INT_MAX) const;
 	///
 	int docbook(odocstream &, OutputParams const & runparams) const;
 	///
-	void addToToc(DocIterator const &) const;
+	void addToToc(DocIterator const & di, bool output_active) const;
 private:
 	///
 	Inset * clone() const { return new InsetMarginal(*this); }
