@@ -261,7 +261,7 @@ bool InsetScript::getStatus(Cursor & cur, FuncRequest const & cmd,
 		FuncStatus & flag) const
 {
 	switch (cmd.action()) {
-	case LFUN_BREAK_PARAGRAPH:
+	case LFUN_PARAGRAPH_BREAK:
 	case LFUN_LAYOUT:
 	case LFUN_LAYOUT_PARAGRAPH:
 	case LFUN_MATH_DISPLAY:
@@ -272,7 +272,6 @@ bool InsetScript::getStatus(Cursor & cur, FuncRequest const & cmd,
 	case LFUN_FLOAT_LIST_INSERT:
 	case LFUN_FLOAT_WIDE_INSERT:
 	case LFUN_FOOTNOTE_INSERT:
-	case LFUN_HFILL_INSERT:
 	case LFUN_INDEX_PRINT:
 	case LFUN_LISTING_INSERT:
 	case LFUN_MARGINALNOTE_INSERT:
@@ -314,11 +313,11 @@ docstring InsetScript::toolTip(BufferView const &, int, int) const
 {
 	OutputParams rp(&buffer().params().encoding());
 	odocstringstream ods;
-	InsetText::plaintext(ods, rp);
+	InsetText::plaintext(ods, rp, 200);
 	docstring content_tip = ods.str();
 	// shorten it if necessary
-	if (content_tip.size() > 200)
-		content_tip = content_tip.substr(0, 200) + "...";
+	if (content_tip.size() >= 200)
+		content_tip = content_tip.substr(0, 197) + "...";
 	docstring res = scripttranslator_loc().find(params_.type);
 	if (!content_tip.empty())
 		res += from_ascii(": ") + content_tip;
@@ -326,33 +325,11 @@ docstring InsetScript::toolTip(BufferView const &, int, int) const
 }
 
 
-void InsetScript::validate(LaTeXFeatures & features) const
-{
-	if (params_.type == InsetScriptParams::Subscript)
-		features.require("subscript");
-	InsetText::validate(features);
-}
-
-
-void InsetScript::latex(otexstream & os, OutputParams const & runparams) const
-{
-	switch (params_.type) {
-	case InsetScriptParams::Subscript:
-		os << "\\textsubscript{";
-		break;
-	case InsetScriptParams::Superscript:
-		os << "\\textsuperscript{";
-		break;
-	}
-	InsetText::latex(os, runparams);
-	os << "}";
-}
-
-
-int InsetScript::plaintext(odocstream & os, OutputParams const & runparams) const
+int InsetScript::plaintext(odocstringstream & os,
+        OutputParams const & runparams, size_t max_length) const
 {
 	odocstringstream oss;
-	InsetText::plaintext(oss, runparams);
+	InsetText::plaintext(oss, runparams, max_length);
 	docstring const text = oss.str();
 	switch (params_.type) {
 	case InsetScriptParams::Subscript:
@@ -376,7 +353,7 @@ int InsetScript::plaintext(odocstream & os, OutputParams const & runparams) cons
 		os << '[' << buffer().B_("superscript") << ':';
 		break;
 	}
-	InsetText::plaintext(os, runparams);
+	InsetText::plaintext(os, runparams, max_length);
 	os << ']';
 
 	return PLAINTEXT_NEWLINE;
@@ -402,9 +379,9 @@ int InsetScript::docbook(odocstream & os, OutputParams const & runparams) const
 }
 
 
-docstring InsetScript::contextMenuName() const
+string InsetScript::contextMenuName() const
 {
-	return from_ascii("context-script");
+	return "context-script";
 }
 
 
