@@ -61,7 +61,8 @@ public:
 	///
 	void latex(otexstream &, OutputParams const &) const;
 	///
-	int plaintext(odocstream &, OutputParams const &) const;
+	int plaintext(odocstringstream & ods, OutputParams const & op,
+	              size_t max_length = INT_MAX) const;
 	///
 	int docbook(odocstream &, OutputParams const &) const;
 	///
@@ -84,7 +85,7 @@ public:
 	/// should we break lines after this inset?
 	bool isLineSeparator() const;
 private:
-	Inset * clone() const { return new InsetSpecialChar(*this); };
+	Inset * clone() const { return new InsetSpecialChar(*this); }
 
 	/// And which kind is this?
 	Kind kind_;
