@@ -115,6 +115,8 @@ private:
 	virtual bool forcePlainLayout(idx_type = 0) const;
 	/// 
 	virtual bool allowParagraphCustomization(idx_type = 0) const;
+	///
+	virtual bool forceLocalFontSwitch() const;
 	/// Is the width forced to some value?
 	bool hasFixedWidth() const { return isFixedWidth; }
 };
@@ -525,7 +527,8 @@ public:
 	bool hasMultiRow(row_type r) const;
 	///
 	idx_type setMultiRow(idx_type cell, idx_type number,
-			     bool const bottom_border);
+			     bool const bottom_border,
+			     LyXAlignment const halign);
 	///
 	void unsetMultiRow(idx_type cell);
 	///
