@@ -13,6 +13,7 @@
 #include "InsetText.h"
 
 #include "insets/InsetArgument.h"
+#include "insets/InsetLayout.h"
 
 #include "buffer_funcs.h"
 #include "Buffer.h"
@@ -30,6 +31,7 @@
 #include "InsetList.h"
 #include "Intl.h"
 #include "Language.h"
+#include "Layout.h"
 #include "LaTeXFeatures.h"
 #include "Lexer.h"
 #include "lyxfind.h"
@@ -54,6 +56,7 @@
 #include "frontends/alert.h"
 #include "frontends/Painter.h"
 
+#include "support/convert.h"
 #include "support/debug.h"
 #include "support/gettext.h"
 #include "support/lstrings.h"
@@ -122,7 +125,7 @@ void InsetText::setMacrocontextPositionRecursive(DocIterator const & pos)
 void InsetText::clear()
 {
 	ParagraphList & pars = paragraphs();
-	LASSERT(!pars.empty(), /**/);
+	LBUFERR(!pars.empty());
 
 	// This is a gross hack...
 	Layout const & old_layout = pars.begin()->layout();
@@ -248,7 +251,6 @@ void InsetText::edit(Cursor & cur, bool front, EntryDirection entry_from)
 	}
 
 	text_.setCursor(cur.top(), pit, pos);
-	cur.clearSelection();
 	cur.finishUndo();
 }
 
@@ -263,7 +265,8 @@ void InsetText::doDispatch(Cursor & cur, FuncRequest & cmd)
 {
 	LYXERR(Debug::ACTION, "InsetText::doDispatch(): cmd: " << cmd);
 
-	if (getLayout().isPassThru()) {
+	// See bug #9042, for instance.
+	if (isPassThru() && lyxCode() != ARG_CODE) {
 		// Force any new text to latex_language FIXME: This
 		// should only be necessary in constructor, but new
 		// paragraphs that are created by pressing enter at
@@ -329,6 +332,40 @@ bool InsetText::getStatus(Cursor & cur, FuncRequest const & cmd,
 		return target_inset;
 	}
 
+	case LFUN_ARGUMENT_INSERT: {
+		string const arg = cmd.getArg(0);
+		if (arg.empty()) {
+			status.setEnabled(false);
+			return true;
+		}
+		if (&buffer().inset() == this || !cur.paragraph().layout().args().empty())
+			return text_.getStatus(cur, cmd, status);
+
+		Layout::LaTeXArgMap args = getLayout().args();
+		Layout::LaTeXArgMap::const_iterator const lait = args.find(arg);
+		if (lait != args.end()) {
+			status.setEnabled(true);
+			ParagraphList::const_iterator pit = paragraphs().begin();
+			for (; pit != paragraphs().end(); ++pit) {
+				InsetList::const_iterator it = pit->insetList().begin();
+				InsetList::const_iterator end = pit->insetList().end();
+				for (; it != end; ++it) {
+					if (it->inset->lyxCode() == ARG_CODE) {
+						InsetArgument const * ins =
+							static_cast<InsetArgument const *>(it->inset);
+						if (ins->name() == arg) {
+							// we have this already
+							status.setEnabled(false);
+							return true;
+						}
+					}
+				}
+			}
+		} else
+			status.setEnabled(false);
+		return true;
+	}
+
 	default:
 		// Dispatch only to text_ if the cursor is inside
 		// the text_. It is not for context menus (bug 5797).
@@ -399,28 +436,43 @@ void InsetText::latex(otexstream & os, OutputParams const & runparams) const
 	if (!il.latexname().empty()) {
 		if (il.latextype() == InsetLayout::COMMAND) {
 			// FIXME UNICODE
+			// FIXME \protect should only be used for fragile
+			//    commands, but we do not provide this information yet.
 			if (runparams.moving_arg)
 				os << "\\protect";
 			os << '\\' << from_utf8(il.latexname());
+			if (!il.latexargs().empty())
+				getArgs(os, runparams);
 			if (!il.latexparam().empty())
 				os << from_utf8(il.latexparam());
 			os << '{';
 		} else if (il.latextype() == InsetLayout::ENVIRONMENT) {
 			if (il.isDisplay())
-			    os << breakln;
+				os << breakln;
 			else
-			    os << safebreakln;
+				os << safebreakln;
 			if (runparams.lastid != -1)
 				os.texrow().start(runparams.lastid,
 						  runparams.lastpos);
 			os << "\\begin{" << from_utf8(il.latexname()) << "}";
+			if (!il.latexargs().empty())
+				getArgs(os, runparams);
 			if (!il.latexparam().empty())
 				os << from_utf8(il.latexparam());
-			os << "\n";
+			os << '\n';
 		}
+	} else {
+		if (!il.latexargs().empty())
+			getArgs(os, runparams);
+		if (!il.latexparam().empty())
+			os << from_utf8(il.latexparam());
 	}
+
+	if (!il.leftdelim().empty())
+		os << il.leftdelim();
+
 	OutputParams rp = runparams;
-	if (il.isPassThru())
+	if (isPassThru())
 		rp.pass_thru = true;
 	if (il.isNeedProtect())
 		rp.moving_arg = true;
@@ -431,15 +483,20 @@ void InsetText::latex(otexstream & os, OutputParams const & runparams) const
 	latexParagraphs(buffer(), text_, os, rp);
 	runparams.encoding = rp.encoding;
 
+	if (!il.rightdelim().empty())
+		os << il.rightdelim();
+
 	if (!il.latexname().empty()) {
 		if (il.latextype() == InsetLayout::COMMAND) {
 			os << "}";
+			if (!il.postcommandargs().empty())
+				getArgs(os, runparams, true);
 		} else if (il.latextype() == InsetLayout::ENVIRONMENT) {
 			// A comment environment doesn't need a % before \n\end
 			if (il.isDisplay() || runparams.inComment)
-			    os << breakln;
+				os << breakln;
 			else
-			    os << safebreakln;
+				os << safebreakln;
 			os << "\\end{" << from_utf8(il.latexname()) << "}\n";
 			if (!il.isDisplay())
 				os.protectSpace(true);
@@ -448,7 +505,8 @@ void InsetText::latex(otexstream & os, OutputParams const & runparams) const
 }
 
 
-int InsetText::plaintext(odocstream & os, OutputParams const & runparams) const
+int InsetText::plaintext(odocstringstream & os,
+        OutputParams const & runparams, size_t max_length) const
 {
 	ParagraphList::const_iterator beg = paragraphs().begin();
 	ParagraphList::const_iterator end = paragraphs().end();
@@ -462,12 +520,14 @@ int InsetText::plaintext(odocstream & os, OutputParams const & runparams) const
 				os << '\n';
 		}
 		odocstringstream oss;
-		writePlaintextParagraph(buffer(), *it, oss, runparams, ref_printed);
+		writePlaintextParagraph(buffer(), *it, oss, runparams, ref_printed, max_length);
 		docstring const str = oss.str();
 		os << str;
 		// FIXME: len is not computed fully correctly; in principle,
 		// we have to count the characters after the last '\n'
 		len = str.size();
+		if (os.str().size() >= max_length)
+			break;
 	}
 
 	return len;
@@ -565,6 +625,20 @@ docstring InsetText::insetAsXHTML(XHTMLStream & xs, OutputParams const & rp,
 	return docstring();
 }
 
+void InsetText::getArgs(otexstream & os, OutputParams const & runparams_in,
+			bool const post) const
+{
+	OutputParams runparams = runparams_in;
+	runparams.local_font =
+		&paragraphs()[0].getFirstFontSettings(buffer().masterBuffer()->params());
+	if (isPassThru())
+		runparams.pass_thru = true;
+	if (post)
+		latexArgInsets(paragraphs(), paragraphs().begin(), os, runparams, getLayout().postcommandargs(), "post:");
+	else
+		latexArgInsets(paragraphs(), paragraphs().begin(), os, runparams, getLayout().latexargs());
+}
+
 
 void InsetText::cursorPos(BufferView const & bv,
 		CursorSlice const & sl, bool boundary, int & x, int & y) const
@@ -676,6 +750,18 @@ ParagraphList & InsetText::paragraphs()
 }
 
 
+bool InsetText::insetAllowed(InsetCode code) const
+{
+	switch (code) {
+	// Arguments are also allowed in PassThru insets
+	case ARG_CODE:
+		return true;
+	default:
+		return !isPassThru();
+	}
+}
+
+
 void InsetText::updateBuffer(ParIterator const & it, UpdateType utype)
 {
 	ParIterator it2 = it;
@@ -723,21 +809,24 @@ void InsetText::forToc(docstring & os, size_t maxlen) const
 }
 
 
-void InsetText::addToToc(DocIterator const & cdit) const
+void InsetText::addToToc(DocIterator const & cdit, bool output_active) const
 {
 	DocIterator dit = cdit;
 	dit.push_back(CursorSlice(const_cast<InsetText &>(*this)));
-	iterateForToc(dit);
+	iterateForToc(dit, output_active);
 }
 
 
-void InsetText::iterateForToc(DocIterator const & cdit) const
+void InsetText::iterateForToc(DocIterator const & cdit, bool output_active) const
 {
 	DocIterator dit = cdit;
 	Toc & toc = buffer().tocBackend().toc("tableofcontents");
 
 	BufferParams const & bufparams = buffer_->params();
 	int const min_toclevel = bufparams.documentClass().min_toclevel();
+	// we really should have done this before we got here, but it
+	// can't hurt too much to do it again
+	bool const doing_output = output_active && producesOutput();
 
 	// For each paragraph, traverse its insets and let them add
 	// their toc items
@@ -754,29 +843,30 @@ void InsetText::iterateForToc(DocIterator const & cdit) const
 			Inset & inset = *it->inset;
 			dit.pos() = it->pos;
 			//lyxerr << (void*)&inset << " code: " << inset.lyxCode() << std::endl;
-			inset.addToToc(dit);
+			inset.addToToc(dit, doing_output);
 			if (inset.lyxCode() == ARG_CODE)
 				arginset = inset.asInsetText();
 		}
 		// now the toc entry for the paragraph
-		int const toclevel = par.layout().toclevel;
+		int const toclevel = text().getTocLevel(pit);
 		if (toclevel != Layout::NOT_IN_TOC && toclevel >= min_toclevel) {
 			// insert this into the table of contents
 			docstring tocstring;
+			int const length = doing_output ? INT_MAX : TOC_ENTRY_LENGTH;
 			if (arginset) {
 				tocstring = par.labelString();
 				if (!tocstring.empty())
 					tocstring += ' ';
-				arginset->text().forToc(tocstring, TOC_ENTRY_LENGTH);
+				arginset->text().forToc(tocstring, length);
 			} else
-				par.forToc(tocstring, TOC_ENTRY_LENGTH);
+				par.forToc(tocstring, length);
 			dit.pos() = 0;
 			toc.push_back(TocItem(dit, toclevel - min_toclevel,
-				tocstring, tocstring));
+				tocstring, doing_output, tocstring));
 		}
 		
 		// And now the list of changes.
-		par.addChangesToToc(dit, buffer());
+		par.addChangesToToc(dit, buffer(), doing_output);
 	}
 }
 
@@ -789,9 +879,11 @@ bool InsetText::notifyCursorLeaves(Cursor const & old, Cursor & cur)
 	// find text inset in old cursor
 	Cursor insetCur = old;
 	int scriptSlice	= insetCur.find(this);
-	LASSERT(scriptSlice != -1, /**/);
+	// we can try to continue here. returning true means
+	// the cursor is "now" invalid. which it was.
+	LASSERT(scriptSlice != -1, return true);
 	insetCur.cutOff(scriptSlice);
-	LASSERT(&insetCur.inset() == this, /**/);
+	LASSERT(&insetCur.inset() == this, return true);
 	
 	// update the old paragraph's words
 	insetCur.paragraph().updateWords();
@@ -802,7 +894,7 @@ bool InsetText::notifyCursorLeaves(Cursor const & old, Cursor & cur)
 
 bool InsetText::completionSupported(Cursor const & cur) const
 {
-	//LASSERT(&cur.bv().cursor().inset() != this, return false);
+	//LASSERT(&cur.bv().cursor().inset() == this, return false);
 	return text_.completionSupported(cur);
 }
 
@@ -863,18 +955,18 @@ void InsetText::completionPosAndDim(Cursor const & cur, int & x, int & y,
 }
 
 
-docstring InsetText::contextMenu(BufferView const &, int, int) const
+string InsetText::contextMenu(BufferView const &, int, int) const
 {
-	docstring context_menu = contextMenuName();
+	string context_menu = contextMenuName();
 	if (context_menu != InsetText::contextMenuName())
 		context_menu += ";" + InsetText::contextMenuName(); 
 	return context_menu;
 }
 
 
-docstring InsetText::contextMenuName() const
+string InsetText::contextMenuName() const
 {
-	return from_ascii("context-edit");
+	return "context-edit";
 }
 
 
@@ -883,6 +975,7 @@ docstring InsetText::toolTipText(docstring prefix,
 {
 	size_t const max_length = numlines * len;
 	OutputParams rp(&buffer().params().encoding());
+	rp.for_tooltip = true;
 	odocstringstream oss;
 	oss << prefix;
 
@@ -895,9 +988,9 @@ docstring InsetText::toolTipText(docstring prefix,
 	for (; it != end; ++it) {
 		if (it != beg)
 			oss << '\n';
-		writePlaintextParagraph(buffer(), *it, oss, rp, ref_printed);
+		writePlaintextParagraph(buffer(), *it, oss, rp, ref_printed, max_length);
 		str = oss.str();
-		if (str.length() > max_length)
+		if (str.length() >= max_length)
 			break;
 	}
 	return support::wrapParas(str, 4, len, numlines);
