@@ -17,8 +17,6 @@
 #include "ColorCode.h"
 #include "Text.h"
 
-#include "insets/InsetLayout.h"
-
 namespace lyx {
 
 class CompletionList;
@@ -75,7 +73,8 @@ public:
 	///
 	void latex(otexstream &, OutputParams const &) const;
 	///
-	int plaintext(odocstream &, OutputParams const &) const;
+	int plaintext(odocstringstream & ods, OutputParams const & op,
+	              size_t max_length = INT_MAX) const;
 	///
 	int docbook(odocstream &, OutputParams const &) const;
 	///
@@ -93,6 +92,9 @@ public:
 	                       XHTMLOptions) const;
 	///
 	void validate(LaTeXFeatures & features) const;
+	
+	/// return the argument(s) only
+	void getArgs(otexstream & os, OutputParams const &, bool const post = false) const;
 
 	/// return x,y of given position relative to the inset's baseline
 	void cursorPos(BufferView const & bv, CursorSlice const & sl,
@@ -146,7 +148,7 @@ public:
 	///
 	ParagraphList const & paragraphs() const;
 	///
-	bool insetAllowed(InsetCode) const { return !getLayout().isPassThru(); }
+	bool insetAllowed(InsetCode) const;
 	///
 	bool allowSpellCheck() const { return getLayout().spellcheck() && !getLayout().isPassThru(); }
 	///
@@ -155,11 +157,14 @@ public:
 	virtual bool allowMultiPar() const { return getLayout().isMultiPar(); }
 	///
 	/// should paragraphs be forced to use the empty layout?
-	virtual bool forcePlainLayout(idx_type = 0) const 
+	virtual bool forcePlainLayout(idx_type = 0) const
 		{ return getLayout().forcePlainLayout(); }
 	/// should the user be allowed to customize alignment, etc.?
-	virtual bool allowParagraphCustomization(idx_type = 0) const 
+	virtual bool allowParagraphCustomization(idx_type = 0) const
 		{ return getLayout().allowParagraphCustomization(); }
+	/// should paragraphs be forced to use a local font language switch?
+	virtual bool forceLocalFontSwitch() const
+		{ return getLayout().forcelocalfontswitch(); }
 
 	/// Update the counters of this inset and of its contents
 	virtual void updateBuffer(ParIterator const &, UpdateType);
@@ -170,7 +175,7 @@ public:
 	///
 	void forToc(docstring &, size_t) const;
 	///
-	void addToToc(DocIterator const &) const;
+	void addToToc(DocIterator const & di, bool output_active) const;
 	///
 	Inset * clone() const { return new InsetText(*this); }
 	///
@@ -203,9 +208,9 @@ public:
 			size_t numlines = 5, size_t len = 80) const;
 
 	///
-	docstring contextMenu(BufferView const &, int, int) const;
+	std::string contextMenu(BufferView const &, int, int) const;
 	///
-	docstring contextMenuName() const;
+	std::string contextMenuName() const;
 	///
 	void doDispatch(Cursor & cur, FuncRequest & cmd);
 protected:
@@ -216,15 +221,13 @@ protected:
 	///
 	docstring getCaptionHTML(OutputParams const &) const;
 	///
-	void iterateForToc(DocIterator const & cdit) const;
+	void iterateForToc(DocIterator const & cdit, bool output_active) const;
 private:
 	///
 	bool drawFrame_;
 	///
 	ColorCode frame_color_;
 	///
-	mutable pit_type old_pit;
-	///
 	mutable Text text_;
 };
 
