@@ -18,8 +18,12 @@
 #include "MathParser.h"
 #include "MetricsInfo.h"
 
+#include "support/gettext.h"
+#include "support/lstrings.h"
+
 #include <ostream>
 
+using namespace lyx::support;
 
 namespace lyx {
 
@@ -200,7 +204,7 @@ void InsetMathFont::mathmlize(MathStream & os) const
 
 void InsetMathFont::infoize(odocstream & os) const
 {
-	os << "Font: " << key_->name;
+	os << bformat(_("Font: %1$s"), key_->name);
 }
 
 
