@@ -185,8 +185,6 @@ public:
 	/// swap two columns
 	virtual void swapCol(col_type c);
 	///
-	virtual void appendRow();
-	///
 	idx_type index(row_type r, col_type c) const;
 	///
 	bool idxBetween(idx_type idx, idx_type from, idx_type to) const;
@@ -196,6 +194,8 @@ public:
 	virtual char defaultColAlign(col_type) { return 'c'; }
 	///
 	void setDefaults();
+	///
+	virtual bool interpretString(Cursor & cur, docstring const & str);
 
 	///
 	virtual int colsep() const;
@@ -244,6 +244,10 @@ protected:
 	virtual docstring eocString(col_type col, col_type lastcol) const;
 	/// splits cells and shifts right part to the next cell
 	void splitCell(Cursor & cur);
+	/// Column aligmment for display of cell at (\p row, \p col).
+	/// Must not be written to file!
+	virtual char displayColAlign(col_type col, row_type) const { return colinfo_[col].align_; }
+
 
 	/// row info.
 	/// rowinfo_[nrows()] is a dummy row used only for hlines.
