@@ -23,6 +23,7 @@
 namespace lyx {
 
 class InsetLabel;
+class MacroNameSet;
 class ParConstIterator;
 class RenderPreview;
 
@@ -152,6 +153,9 @@ public:
 	/// Recreates the preview if preview is enabled.
 	void reloadPreview(DocIterator const & pos) const;
 	///
+	void usedMacros(MathData const & md, DocIterator const & pos,
+	                MacroNameSet & macros, MacroNameSet & defs) const;
+	///
 	void initUnicodeMath() const;
 
 	///
@@ -271,7 +275,9 @@ protected:
 	///
 	void handleFont2(Cursor & cur, docstring const & arg);
 	///
-	bool previewState(BufferView * bv) const;
+	bool previewState(BufferView const * const bv) const;
+	///
+	bool previewTooSmall(Dimension const & dim) const;
 };
 
 
