@@ -54,6 +54,7 @@
 #include "OutputParams.h"
 #include "Text.h"
 
+#include "frontends/Application.h"
 #include "frontends/Clipboard.h"
 #include "frontends/Painter.h"
 #include "frontends/Selection.h"
@@ -1770,7 +1771,7 @@ bool InsetMathNest::interpretChar(Cursor & cur, char_type const c)
 			// but suppress direct insertion of two spaces in a row
 			// the still allows typing  '<space>a<space>' and deleting the 'a', but
 			// it is better than nothing...
-			if (!cur.pos() != 0 || cur.prevAtom()->getChar() != ' ') {
+			if (cur.pos() == 0 || cur.prevAtom()->getChar() != ' ') {
 				cur.insert(c);
 				// FIXME: we have to enable full redraw here because of the
 				// visual box corners that define the inset. If we know for
@@ -2110,7 +2111,7 @@ MathCompletionList::MathCompletionList(Cursor const & cur)
 
 	// fill in global macros
 	macros.clear();
-	MacroTable::globalMacros().getMacroNames(macros);
+	MacroTable::globalMacros().getMacroNames(macros, false);
 	//lyxerr << "Globals completion macros: ";
 	for (it = macros.begin(); it != macros.end(); ++it) {
 		//lyxerr << "\\" + *it << " ";
@@ -2182,7 +2183,7 @@ MathCompletionList::MathCompletionList(Cursor const & cur)
 	MathWordList::const_iterator it2;
 	//lyxerr << "Globals completion commands: ";
 	for (it2 = words.begin(); it2 != words.end(); ++it2) {
-		if (it2->second.inset != "macro") {
+		if (it2->second.inset != "macro" && !it2->second.hidden) {
 			// macros are already read from MacroTable::globalMacros()
 			globals.push_back('\\' + it2->first);
 			//lyxerr << '\\' + it2->first << ' ';
@@ -2225,7 +2226,10 @@ std::string MathCompletionList::icon(size_t idx) const
 		cmd = locals[idx];
 
 	// get the icon resource name by stripping the backslash
-	return "images/math/" + to_utf8(cmd.substr(1)) + ".png";
+	docstring icon_name = frontend::Application::mathIcon(cmd.substr(1));
+	if (icon_name.empty())
+		return std::string();
+	return "images/math/" + to_utf8(icon_name);
 }
 
 std::vector<docstring> MathCompletionList::globals;
