@@ -86,7 +86,8 @@ void InsetMathSymbol::metrics(MetricsInfo & mi, Dimension & dim) const
 	scriptable_ = false;
 	if (mi.base.style == LM_ST_DISPLAY)
 		if (sym_->inset == "cmex" || sym_->inset == "esint" ||
-		    sym_->extra == "funclim")
+		    sym_->extra == "funclim" ||
+		    (sym_->inset == "stmry" && sym_->extra == "mathop"))
 			scriptable_ = true;
 }
 
@@ -143,7 +144,8 @@ bool InsetMathSymbol::takesLimits() const
 		sym_->inset == "cmex" ||
 		sym_->inset == "lyxboldsymb" ||
 		sym_->inset == "esint" ||
-		sym_->extra == "funclim";
+		sym_->extra == "funclim" ||
+		(sym_->inset == "stmry" && sym_->extra == "mathop");
 }
 
 
@@ -185,22 +187,13 @@ void InsetMathSymbol::mathematica(MathematicaStream & os) const
 }
 
 
-// FIXME This will likely need some work.
-char const * MathMLtype(docstring const & s)
-{
-	if (s == "mathord")
-		return "mi";
-	return "mo";
-}
-
-
 void InsetMathSymbol::mathmlize(MathStream & os) const
 {
-	// FIXME We may need to do more interesting things 
+	// FIXME We may need to do more interesting things
 	// with MathMLtype.
-	char const * type = MathMLtype(sym_->extra);
+	char const * type = sym_->MathMLtype();
 	os << '<' << type << "> ";
-	if (sym_->xmlname == "x") 
+	if (sym_->xmlname == "x")
 		// unknown so far
 		os << name();
 	else
@@ -211,15 +204,15 @@ void InsetMathSymbol::mathmlize(MathStream & os) const
 
 void InsetMathSymbol::htmlize(HtmlStream & os, bool spacing) const
 {
-	// FIXME We may need to do more interesting things 
+	// FIXME We may need to do more interesting things
 	// with MathMLtype.
-	char const * type = MathMLtype(sym_->extra);
+	char const * type = sym_->MathMLtype();
 	bool op = (std::string(type) == "mo");
-	
-	if (sym_->xmlname == "x") 
+
+	if (sym_->xmlname == "x")
 		// unknown so far
 		os << ' ' << name() << ' ';
-	else if (op && spacing) 
+	else if (op && spacing)
 		os << ' ' << sym_->xmlname << ' ';
 	else
 		os << sym_->xmlname;
