@@ -68,8 +68,7 @@ bool MacroData::expand(vector<MathData> const & args, MathData & to) const
 	updateData();
 
 	// Hack. Any inset with a cell would do.
-	static InsetMathSqrt inset(0);
-	inset.setBuffer(const_cast<Buffer &>(*buffer_));
+	InsetMathSqrt inset(const_cast<Buffer *>(buffer_));
 
 	docstring const & definition(display_.empty() ? definition_ : display_);
 	asArray(definition, inset.cell(0));
@@ -119,6 +118,14 @@ string const MacroData::requires() const
 }
 
 
+bool MacroData::hidden() const
+{
+	if (sym_)
+		return sym_->hidden;
+	return false;
+}
+
+
 docstring const MacroData::xmlname() const
 {
 	if (sym_)
@@ -242,10 +249,11 @@ MacroTable::insert(Buffer * buf, docstring const & def)
 }
 
 
-void MacroTable::getMacroNames(std::set<docstring> & names) const
+void MacroTable::getMacroNames(std::set<docstring> & names, bool gethidden) const
 {
 	for (const_iterator it = begin(); it != end(); ++it)
-		names.insert(it->first);
+		if (gethidden || !it->second.hidden())
+			names.insert(it->first);
 }
 
 
