@@ -36,6 +36,7 @@
 #include "frontends/FontMetrics.h"
 #include "frontends/Painter.h"
 
+#include "support/gettext.h"
 #include "support/lassert.h"
 #include <boost/next_prior.hpp>
 
@@ -53,14 +54,14 @@ MathData::MathData(Buffer * buf, const_iterator from, const_iterator to)
 
 MathAtom & MathData::operator[](pos_type pos)
 {
-	LASSERT(pos < size(), /**/);
+	LBUFERR(pos < size());
 	return base_type::operator[](pos);
 }
 
 
 MathAtom const & MathData::operator[](pos_type pos) const
 {
-	LASSERT(pos < size(), /**/);
+	LBUFERR(pos < size());
 	return base_type::operator[](pos);
 }
 
@@ -73,7 +74,7 @@ void MathData::insert(size_type pos, MathAtom const & t)
 
 void MathData::insert(size_type pos, MathData const & ar)
 {
-	LASSERT(pos <= size(), /**/);
+	LBUFERR(pos <= size());
 	base_type::insert(begin() + pos, ar.begin(), ar.end());
 }
 
@@ -215,6 +216,7 @@ void MathData::touch() const
 }
 
 
+#if 0
 namespace {
 
 bool isInside(DocIterator const & it, MathData const & ar,
@@ -229,7 +231,7 @@ bool isInside(DocIterator const & it, MathData const & ar,
 }
 
 }
-
+#endif
 
 
 void MathData::metrics(MetricsInfo & mi, Dimension & dim) const
@@ -339,13 +341,13 @@ void MathData::draw(PainterInfo & pi, int x, int y) const
 		docstring s1 = completion.substr(0, uniqueTo);
 		docstring s2 = completion.substr(uniqueTo);
 		
-		if (s1.size() > 0) {
+		if (!s1.empty()) {
 			f.setColor(Color_inlinecompletion);
 			pi.pain.text(x, y, s1, f);
 			x += mathed_string_width(f, s1);
 		}
 		
-		if (s2.size() > 0) {
+		if (!s2.empty()) {
 			f.setColor(Color_nonunique_inlinecompletion);
 			pi.pain.text(x, y, s2, f);
 			x += mathed_string_width(f, s2);
@@ -487,7 +489,7 @@ void MathData::updateMacros(Cursor * cur, MacroContext const & mc,
 		InsetMath * inset = operator[](i).nucleus();
 		if (inset->asScriptInset())
 			inset = inset->asScriptInset()->nuc()[0].nucleus();
-		LASSERT(inset->asMacro(), /**/);
+		LASSERT(inset->asMacro(), continue);
 		inset->asMacro()->updateRepresentation(cur, mc, utype);
 	}
 }
