@@ -352,7 +352,7 @@ void splitScripts(MathData & ar)
 			continue;
 
 		// we must have a nucleus if we only have a superscript
-		if (!script->hasDown() && script->nuc().size() == 0)
+		if (!script->hasDown() && script->nuc().empty())
 			continue;
 
 		if (script->nuc().size() == 1) {
@@ -537,6 +537,9 @@ void extractDelims(MathData & ar)
 // assume 'extractDelims' ran before
 void extractFunctions(MathData & ar, ExternalMath kind)
 {
+	// FIXME From what I can see, this is quite broken right now, for reasons
+	// I will note below. (RGH)
+
 	// we need at least two items...
 	if (ar.size() < 2)
 		return;
@@ -551,9 +554,26 @@ void extractFunctions(MathData & ar, ExternalMath kind)
 		docstring name;
 		// is it a function?
 		// it certainly is if it is well known...
+
+		// FIXME This will never give us anything. When we get here, *it will
+		// never point at a string, but only at a character. I.e., if we are
+		// working on "sin(x)", then we are seeing:
+		// [char s mathalpha][char i mathalpha][char n mathalpha][delim ( ) [char x mathalpha]]
+		// and of course we will not find the function name "sin" in there, but
+		// rather "n(x)".
+		//
+		// It appears that we original ran extractStrings() before we ran
+		// extractFunctions(), but Andre changed this at f200be55, I think
+		// because this messed up what he was trying to do with "dx" in the
+		// context of integrals.
+		//
+		// This could be fixed by looking at a charSequence instead of just at
+		// the various characters, one by one. But I am not sure I understand
+		// exactly what we are trying to do here. And it involves a lot of
+		// guessing.
 		if (!extractFunctionName(*it, name)) {
 			// is this a user defined function?
-			// it it probably not, if it doesn't have a name.
+			// probably not, if it doesn't have a name.
 			if (!extractString(*it, name))
 				continue;
 			// it is not if it has no argument
@@ -564,7 +584,7 @@ void extractFunctions(MathData & ar, ExternalMath kind)
 			InsetMathDelim const * del = (*jt)->asDelimInset();
 			if (!del || del->cell(0).size() != 1)
 				continue;
-			// fall trough into main branch
+			// fall through into main branch
 		}
 
 		// do we have an exponent like in
@@ -619,7 +639,7 @@ bool testIntegral(MathAtom const & at)
 	return
 	 testIntSymbol(at) ||
 		( at->asScriptInset()
-		  && at->asScriptInset()->nuc().size()
+		  && !at->asScriptInset()->nuc().empty()
 			&& testIntSymbol(at->asScriptInset()->nuc().back()) );
 }
 
@@ -719,7 +739,7 @@ bool testSum(MathAtom const & at)
 	return
 	 testSumSymbol(at) ||
 		( at->asScriptInset()
-		  && at->asScriptInset()->nuc().size()
+		  && !at->asScriptInset()->nuc().empty()
 			&& testSumSymbol(at->asScriptInset()->nuc().back()) );
 }
 
@@ -800,7 +820,7 @@ bool testDiffItem(MathAtom const & at)
 
 bool testDiffArray(MathData const & ar)
 {
-	return ar.size() && testDiffItem(ar.front());
+	return !ar.empty() && testDiffItem(ar.front());
 }
 
 
@@ -1419,7 +1439,7 @@ void mathmlize(MathData const & dat, MathStream & os)
 {
 	MathData ar = dat;
 	extractStructure(ar, MATHML);
-	if (ar.size() == 0)
+	if (ar.empty())
 		os << "<mrow/>";
 	else if (ar.size() == 1)
 		os << ar.front();
@@ -1436,7 +1456,7 @@ void htmlize(MathData const & dat, HtmlStream & os)
 {
 	MathData ar = dat;
 	extractStructure(ar, HTML);
-	if (ar.size() == 0) 
+	if (ar.empty())
 		return;
 	if (ar.size() == 1) {
 		os << ar.front();
@@ -1452,7 +1472,8 @@ bool extractNumber(MathData const & ar, int & i)
 {
 	idocstringstream is(charSequence(ar.begin(), ar.end()));
 	is >> i;
-	return is;
+	// Do not convert is implicitly to bool, since that is forbidden in C++11.
+	return !is.fail();
 }
 
 
@@ -1460,7 +1481,8 @@ bool extractNumber(MathData const & ar, double & d)
 {
 	idocstringstream is(charSequence(ar.begin(), ar.end()));
 	is >> d;
-	return is;
+	// Do not convert is implicitly to bool, since that is forbidden in C++11.
+	return !is.fail();
 }
 
 
