@@ -186,6 +186,7 @@ void initSymbols()
 			string requires;
 			string extra;
 			string xmlname;
+			bool hidden = false;
 			is >> macro >> requires;
 			if ((is >> xmlname)) {
 				extra = requires;
@@ -207,6 +208,11 @@ void initSymbols()
 					tmp.extra = from_utf8(extra);
 					tmp.xmlname = from_utf8(xmlname);
 					tmp.requires = from_utf8(requires);
+					if (requires == "hiddensymbol") {
+						requires = "";
+						tmp.hidden = hidden = true;
+					} else
+						tmp.requires = from_utf8(requires);
 					theMathWordList[it->first] = tmp;
 					wit = theMathWordList.find(it->first);
 					it->second.setSymbol(&(wit->second));
@@ -219,7 +225,8 @@ void initSymbols()
 				<< "  draw: 0"
 				<< "  extra: " << extra
 				<< "  xml: " << xmlname
-				<< "  requires: " << requires << '\'');
+				<< "  requires: " << requires
+				<< "  hidden: " << hidden << '\'');
 			continue;
 		}
 
@@ -290,6 +297,12 @@ void initSymbols()
 					      << " used for " << to_utf8(tmp.name));
 		}
 
+		if (tmp.requires == "hiddensymbol")
+		{
+			tmp.requires.clear();
+			tmp.hidden = true;
+		}
+
 		if (theMathWordList.find(tmp.name) != theMathWordList.end())
 			LYXERR(Debug::MATHED, "readSymbols: inset " << to_utf8(tmp.name)
 				<< " already exists.");
@@ -303,7 +316,8 @@ void initSymbols()
 			<< "  draw: " << int(tmp.draw.empty() ? 0 : tmp.draw[0])
 			<< "  extra: " << to_utf8(tmp.extra)
 			<< "  xml: " << to_utf8(tmp.xmlname)
-			<< "  requires: " << to_utf8(tmp.requires) << '\'');
+			<< "  requires: " << to_utf8(tmp.requires)
+			<< "  hidden: " << tmp.hidden << '\'');
 	}
 	docstring tmp = from_ascii("cmm");
 	docstring tmp2 = from_ascii("cmsy");
