@@ -52,35 +52,37 @@ namespace lyx {
 class ArgumentProxy : public InsetMath {
 public:
 	///
-	ArgumentProxy(MathMacro & mathMacro, size_t idx)
+	ArgumentProxy(MathMacro * mathMacro, size_t idx)
 		: mathMacro_(mathMacro), idx_(idx) {}
 	///
-	ArgumentProxy(MathMacro & mathMacro, size_t idx, docstring const & def)
+	ArgumentProxy(MathMacro * mathMacro, size_t idx, docstring const & def)
 		: mathMacro_(mathMacro), idx_(idx)
 	{
 			asArray(def, def_);
 	}
 	///
+	void setOwner(MathMacro * mathMacro) { mathMacro_ = mathMacro; }
+	///
 	InsetCode lyxCode() const { return ARGUMENT_PROXY_CODE; }
 	///
 	void metrics(MetricsInfo & mi, Dimension & dim) const {
-		mathMacro_.macro()->unlock();
-		mathMacro_.cell(idx_).metrics(mi, dim);
+		mathMacro_->macro()->unlock();
+		mathMacro_->cell(idx_).metrics(mi, dim);
 
-		if (!mathMacro_.editMetrics(mi.base.bv)
-		    && mathMacro_.cell(idx_).empty())
+		if (!mathMacro_->editMetrics(mi.base.bv)
+		    && mathMacro_->cell(idx_).empty())
 			def_.metrics(mi, dim);
 
-		mathMacro_.macro()->lock();
+		mathMacro_->macro()->lock();
 	}
 	// FIXME Other external things need similar treatment.
 	///
-	void mathmlize(MathStream & ms) const { ms << mathMacro_.cell(idx_); }
+	void mathmlize(MathStream & ms) const { ms << mathMacro_->cell(idx_); }
 	///
-	void htmlize(HtmlStream & ms) const { ms << mathMacro_.cell(idx_); }
+	void htmlize(HtmlStream & ms) const { ms << mathMacro_->cell(idx_); }
 	///
 	void draw(PainterInfo & pi, int x, int y) const {
-		if (mathMacro_.editMetrics(pi.base.bv)) {
+		if (mathMacro_->editMetrics(pi.base.bv)) {
 			// The only way a ArgumentProxy can appear is in a cell of the
 			// MathMacro. Moreover the cells are only drawn in the DISPLAY_FOLDED
 			// mode and then, if the macro is edited the monochrome
@@ -90,22 +92,22 @@ public:
 			// here (and the assert triggers in pain.leaveMonochromeMode())
 			// it's a bug.
 			pi.pain.leaveMonochromeMode();
-			mathMacro_.cell(idx_).draw(pi, x, y);
+			mathMacro_->cell(idx_).draw(pi, x, y);
 			pi.pain.enterMonochromeMode(Color_mathbg, Color_mathmacroblend);
-		} else if (mathMacro_.cell(idx_).empty()) {
-			mathMacro_.cell(idx_).setXY(*pi.base.bv, x, y);
+		} else if (mathMacro_->cell(idx_).empty()) {
+			mathMacro_->cell(idx_).setXY(*pi.base.bv, x, y);
 			def_.draw(pi, x, y);
 		} else
-			mathMacro_.cell(idx_).draw(pi, x, y);
+			mathMacro_->cell(idx_).draw(pi, x, y);
 	}
 	///
 	size_t idx() const { return idx_; }
 	///
 	int kerning(BufferView const * bv) const
 	{
-		if (mathMacro_.editMetrics(bv)
-		    || !mathMacro_.cell(idx_).empty())
-			return mathMacro_.cell(idx_).kerning(bv);
+		if (mathMacro_->editMetrics(bv)
+		    || !mathMacro_->cell(idx_).empty())
+			return mathMacro_->cell(idx_).kerning(bv);
 		else
 			return def_.kerning(bv);
 	}
@@ -117,7 +119,7 @@ private:
 		return new ArgumentProxy(*this);
 	}
 	///
-	MathMacro & mathMacro_;
+	MathMacro * mathMacro_;
 	///
 	size_t idx_;
 	///
@@ -133,6 +135,56 @@ MathMacro::MathMacro(Buffer * buf, docstring const & name)
 {}
 
 
+MathMacro::MathMacro(MathMacro const & that)
+	: InsetMathNest(that), expanded_(that.buffer_)
+{
+	assign(that);
+}
+
+
+MathMacro & MathMacro::operator=(MathMacro const & that)
+{
+	if (&that == this)
+		return *this;
+	InsetMathNest::operator=(that);
+	assign(that);
+	return *this;
+}
+
+
+void MathMacro::assign(MathMacro const & that)
+{
+	name_ = that.name_;
+	displayMode_ = that.displayMode_;
+	expanded_ = that.expanded_;
+	definition_ = that.definition_;
+	attachedArgsNum_ = that.attachedArgsNum_;
+	optionals_ = that.optionals_;
+	nextFoldMode_ = that.nextFoldMode_;
+	macroBackup_ = that.macroBackup_;
+	macro_ = that.macro_;
+	editing_ = that.editing_;
+	requires_ = that.requires_;
+	needsUpdate_ = that.needsUpdate_;
+	isUpdating_ = that.isUpdating_;
+	appetite_ = that.appetite_;
+	// Update the pointers to our owner of all expanded macros.
+	// This needs to be called every time a copy of the owner is created
+	// (bug 9418).
+	for (size_t i = 0; i < expanded_.cell(0).size(); ++i) {
+		ArgumentProxy * p = dynamic_cast<ArgumentProxy *>(expanded_.cell(0)[i].nucleus());
+		if (p)
+			p->setOwner(this);
+	}
+	if (macro_ && lyxrc.preview == LyXRC::PREVIEW_ON) {
+		// As MathData::metrics() is not called when instant preview is
+		// on, we have to update macro_ by ourselves. In this case, we
+		// simply let it point to the last known good copy of MacroData.
+		macro_ = &macroBackup_;
+	}
+}
+
+
 Inset * MathMacro::clone() const
 {
 	MathMacro * copy = new MathMacro(*this);
@@ -358,9 +410,9 @@ void MathMacro::updateRepresentation(Cursor * cur, MacroContext const & mc,
 	for (size_t i = 0; i < nargs(); ++i) {
 		ArgumentProxy * proxy;
 		if (i < defaults.size())
-			proxy = new ArgumentProxy(*this, i, defaults[i]);
+			proxy = new ArgumentProxy(this, i, defaults[i]);
 		else
-			proxy = new ArgumentProxy(*this, i);
+			proxy = new ArgumentProxy(this, i);
 		values[i].insert(0, MathAtom(proxy));
 	}
 	// expanding macro with the values
@@ -558,8 +610,22 @@ bool MathMacro::validName() const
 
 void MathMacro::validate(LaTeXFeatures & features) const
 {
+	// Immediately after a document is loaded, in some cases the MacroData
+	// of the global macros defined in the lib/symbols file may still not
+	// be known to the macro machinery because it will be set only after
+	// the first call to updateMacros(). This is not a problem unless
+	// instant preview is on for math, in which case we will be missing
+	// the corresponding requirements.
+	// In this case, we get the required info from the global macro table.
 	if (!requires_.empty())
 		features.require(requires_);
+	else if (!macro_) {
+		// Update requires for known global macros.
+		MacroData const * data = MacroTable::globalMacros().get(name());
+		if (data && !data->requires().empty())
+			features.require(data->requires());
+	}
+
 
 	if (name() == "binom")
 		features.require("binom");
@@ -786,7 +852,8 @@ void MathMacro::maple(MapleStream & os) const
 
 void MathMacro::mathmlize(MathStream & os) const
 {
-	LATTEST(macro_);
+	// macro_ is 0 if this is an unknown macro
+	LATTEST(macro_ || displayMode_ != DISPLAY_NORMAL);
 	if (macro_) {
 		docstring const xmlname = macro_->xmlname();
 		if (!xmlname.empty()) {
@@ -807,7 +874,8 @@ void MathMacro::mathmlize(MathStream & os) const
 
 void MathMacro::htmlize(HtmlStream & os) const
 {
-	LATTEST(macro_);
+	// macro_ is 0 if this is an unknown macro
+	LATTEST(macro_ || displayMode_ != DISPLAY_NORMAL);
 	if (macro_) {
 		docstring const xmlname = macro_->xmlname();
 		if (!xmlname.empty()) {
