@@ -400,6 +400,12 @@ public:
 	int lineno() const { return lineno_; }
 	///
 	void putback();
+	/// store current position
+	void pushPosition();
+	/// restore previous position
+	void popPosition();
+	/// forget last saved position
+	void dropPosition();
 
 private:
 	///
@@ -447,6 +453,8 @@ private:
 	vector<Token> tokens_;
 	///
 	unsigned pos_;
+	///
+	std::vector<unsigned> positions_;
 	/// Stack of active environments
 	vector<docstring> environments_;
 	///
@@ -528,6 +536,25 @@ void Parser::putback()
 }
 
 
+void Parser::pushPosition()
+{
+	positions_.push_back(pos_);
+}
+
+
+void Parser::popPosition()
+{
+	pos_ = positions_.back();
+	positions_.pop_back();
+}
+
+
+void Parser::dropPosition()
+{
+	positions_.pop_back();
+}
+
+
 bool Parser::good() const
 {
 	return pos_ < tokens_.size();
@@ -1039,10 +1066,10 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 			 t.cs() == "def") {
 			if (t.cs() == "global")
 				getToken();
-			
+
 			// get name
 			docstring name = getToken().cs();
-			
+
 			// read parameters
 			int nargs = 0;
 			docstring pars;
@@ -1051,17 +1078,17 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 				++nargs;
 			}
 			nargs /= 2;
-			
+
 			// read definition
 			MathData def;
 			parse(def, FLAG_ITEM, InsetMath::UNDECIDED_MODE);
-			
+
 			// is a version for display attached?
 			skipSpaces();
 			MathData display;
 			if (nextToken().cat() == catBegin)
 				parse(display, FLAG_ITEM, InsetMath::MATH_MODE);
-			
+
 			cell->push_back(MathAtom(new MathMacroTemplate(buf,
 				name, nargs, 0, MacroTypeDef,
 				vector<MathData>(), def, display)));
@@ -1069,7 +1096,7 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 			if (buf && (mode_ & Parse::TRACKMACRO))
 				buf->usermacros.insert(name);
 		}
-		
+
 		else if (t.cs() == "newcommand" ||
 			 t.cs() == "renewcommand" ||
 			 t.cs() == "newlyxcommand") {
@@ -1083,13 +1110,13 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 				error("'}' in \\newcommand expected");
 				return success_;
 			}
-				
+
 			// get arity
 			docstring const arg = getArg('[', ']');
 			int nargs = 0;
 			if (!arg.empty())
 				nargs = convert<int>(arg);
-				
+
 			// optional argument given?
 			skipSpaces();
 			int optionals = 0;
@@ -1100,16 +1127,16 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 				parse(optionalValues[optionals], FLAG_BRACK_LAST, mode);
 				++optionals;
 			}
-			
+
 			MathData def;
 			parse(def, FLAG_ITEM, InsetMath::UNDECIDED_MODE);
-			
+
 			// is a version for display attached?
 			skipSpaces();
 			MathData display;
 			if (nextToken().cat() == catBegin)
 				parse(display, FLAG_ITEM, InsetMath::MATH_MODE);
-			
+
 			cell->push_back(MathAtom(new MathMacroTemplate(buf,
 				name, nargs, optionals, MacroTypeNewcommand,
 				optionalValues, def, display)));
@@ -1117,7 +1144,7 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 			if (buf && (mode_ & Parse::TRACKMACRO))
 				buf->usermacros.insert(name);
 		}
-		
+
 		else if (t.cs() == "newcommandx" ||
 			 t.cs() == "renewcommandx") {
 			// \newcommandx{\foo}[2][usedefault, addprefix=\global,1=default]{#1,#2}
@@ -1132,7 +1159,7 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 				}
 			} else
 				name = getToken().cs();
-				
+
 			// get arity
 			docstring const arg = getArg('[', ']');
 			if (arg.empty()) {
@@ -1140,14 +1167,14 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 				return success_;
 			}
 			int nargs = convert<int>(arg);
-			
+
 			// get options
 			int optionals = 0;
 			vector<MathData> optionalValues;
 			if (nextToken().character() == '[') {
 				// skip '['
 				getToken();
-					
+
 				// handle 'opt=value' options, separated by ','.
 				skipSpaces();
 				while (nextToken().character() != ']' && good()) {
@@ -1160,14 +1187,14 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 							      "for given optional parameter.");
 							return success_;
 						}
-						
+
 						// skip '='
 						if (getToken().character() != '=') {
 							error("'=' and optional parameter value "
 							      "expected for \\newcommandx");
 							return success_;
 						}
-						
+
 						// get value
 						int optNum = max(size_t(n), optionalValues.size());
 						optionalValues.resize(optNum);
@@ -1182,12 +1209,12 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 					} else if (nextToken().cat() == catLetter) {
 						// we in fact ignore every non-optional
 						// parameter
-						
+
 						// get option name
 						docstring opt;
 						while (nextToken().cat() == catLetter)
 							opt += getChar();
-					
+
 						// value?
 						skipSpaces();
 						MathData value;
@@ -1195,14 +1222,14 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 							getToken();
 							while (nextToken().character() != ']'
 								&& nextToken().character() != ',')
-								parse(value, FLAG_ITEM, 
+								parse(value, FLAG_ITEM,
 								      InsetMath::UNDECIDED_MODE);
 						}
 					} else {
 						error("option for \\newcommandx expected");
 						return success_;
 					}
-					
+
 					// skip komma
 					skipSpaces();
 					if (nextToken().character() == ',') {
@@ -1214,7 +1241,7 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 						return success_;
 					}
 				}
-				
+
 				// skip ']'
 				if (!good())
 					return success_;
@@ -1307,14 +1334,36 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 		else if (t.cs() == "\\") {
 			if (flags & FLAG_ALIGN)
 				return success_;
-			bool added = false;
+			bool starred = false;
+			docstring arg;
 			if (nextToken().asInput() == "*") {
 				getToken();
-				added = addRow(grid, cellrow, docstring(), false);
-			} else if (good())
-				added = addRow(grid, cellrow, getArg('[', ']'));
-			else
+				starred = true;
+			} else if (nextToken().asInput() == "[")
+				arg = getArg('[', ']');
+			else if (!good())
 				error("missing token after \\\\");
+			// skip "{}" added in front of "[" (the
+			// counterpart is in InsetMathGrid::eolString())
+			// skip spaces because formula could come from tex2lyx
+			bool skipBraces = false;
+			pushPosition();
+			if (nextToken().cat() == catBegin) {
+				getToken();
+				if (nextToken().cat() == catEnd) {
+					getToken();
+					pushPosition();
+					skipSpaces();
+					if (nextToken().asInput() == "[")
+						skipBraces = true;
+					popPosition();
+				}
+			}
+			if (skipBraces)
+				dropPosition();
+			else
+				popPosition();
+			bool const added = addRow(grid, cellrow, arg, !starred);
 			if (added) {
 				cellcol = 0;
 				if (grid.asHullInset())
@@ -1434,7 +1483,7 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 		else if (t.cs() == "cfrac") {
 			// allowed formats are \cfrac[pos]{num}{denom}
 			docstring const arg = getArg('[', ']');
-			//lyxerr << "got so far: '" << arg << "'" << endl;				
+			//lyxerr << "got so far: '" << arg << "'" << endl;
 				if (arg == "l")
 					cell->push_back(MathAtom(new InsetMathFrac(buf, InsetMathFrac::CFRACLEFT)));
 				else if (arg == "r")
@@ -1534,13 +1583,13 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 
 		else if (t.cs() == "begin") {
 			docstring const name = getArg('{', '}');
-			
+
 			if (name.empty()) {
 				success_ = false;
 				error("found invalid environment");
 				return success_;
 			}
-			
+
 			environments_.push_back(name);
 
 			if (name == "array" || name == "subarray") {
@@ -2068,7 +2117,8 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 					//}
 					for (InsetMath::idx_type i = start; i < at->nargs(); ++i) {
 						parse(at.nucleus()->cell(i), FLAG_ITEM, m);
-						skipSpaces();
+						if (mode == InsetMath::MATH_MODE)
+							skipSpaces();
 					}
 					cell->push_back(at);
 				}
@@ -2089,6 +2139,15 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 } // anonymous namespace
 
 
+// FIXME This will likely need some work.
+char const * latexkeys::MathMLtype() const
+{
+	if (extra == "mathord")
+		return "mi";
+	return "mo";
+}
+
+
 bool mathed_parse_cell(MathData & ar, docstring const & str, Parse::flags f)
 {
 	return Parser(str, f, ar.buffer()).parse(ar, 0, f & Parse::TEXTMODE ?
