@@ -30,6 +30,10 @@ class Lexer;
 ///
 class latexkeys {
 public:
+	///
+	latexkeys() : hidden(false) {}
+	///
+	char const * MathMLtype() const;
 	/// name of the macro or primitive
 	docstring name;
 	/// name of a inset that handles that macro
@@ -54,6 +58,9 @@ public:
 	docstring xmlname;
 	/// required LaTeXFeatures
 	docstring requires;
+	/// Should this macro be hidden from autocompletion (since it requires
+	/// user preamble code)?
+	bool hidden;
 };
 
 
