@@ -15,10 +15,10 @@
 
 #include "MathParser_flags.h"
 #include "support/strfwd.h"
-#include <vector>
 
 namespace lyx {
 
+class MetricsInfo;
 class PainterInfo;
 class FontInfo;
 class Dimension;
@@ -40,6 +40,8 @@ void mathed_string_dim(FontInfo const & font,
 
 int mathed_string_width(FontInfo const &, docstring const & s);
 
+void metricsStrRedBlack(MetricsInfo & mi, Dimension & dim, docstring const & s);
+
 void drawStrRed(PainterInfo & pi, int x, int y, docstring const & s);
 void drawStrBlack(PainterInfo & pi, int x, int y, docstring const & s);
 
@@ -53,6 +55,8 @@ bool isMathFont(docstring const & name);
 
 bool isTextFont(docstring const & name);
 
+bool isAlphaSymbol(MathAtom const & at);
+
 // converts single cell to string
 docstring asString(MathData const & ar);
 // converts single inset to string
