@@ -42,7 +42,7 @@ namespace {
 
 ParagraphList::const_iterator searchParagraph(
 	ParagraphList::const_iterator p,
-  ParagraphList::const_iterator const & pend)
+	ParagraphList::const_iterator const & pend)
 {
 	for (++p; p != pend && p->layout().latextype == LATEX_PARAGRAPH; ++p)
 		;
@@ -52,8 +52,8 @@ ParagraphList::const_iterator searchParagraph(
 
 
 ParagraphList::const_iterator searchCommand(
-		ParagraphList::const_iterator p,
-		ParagraphList::const_iterator const & pend)
+	ParagraphList::const_iterator p,
+	ParagraphList::const_iterator const & pend)
 {
 	Layout const & bstyle = p->layout();
 
@@ -68,8 +68,8 @@ ParagraphList::const_iterator searchCommand(
 
 
 ParagraphList::const_iterator searchEnvironment(
-		ParagraphList::const_iterator p,
-		ParagraphList::const_iterator const & pend)
+	ParagraphList::const_iterator p,
+	ParagraphList::const_iterator const & pend)
 {
 	Layout const & bstyle = p->layout();
 	size_t const depth = p->params().depth();
@@ -95,12 +95,13 @@ ParagraphList::const_iterator searchEnvironment(
 }
 
 
-ParagraphList::const_iterator makeParagraph(Buffer const & buf,
-					    odocstream & os,
-					    OutputParams const & runparams,
-					    Text const & text,
-					    ParagraphList::const_iterator const & pbegin,
-					    ParagraphList::const_iterator const & pend)
+ParagraphList::const_iterator makeParagraph(
+	Buffer const & buf,
+	odocstream & os,
+	OutputParams const & runparams,
+	Text const & text,
+	ParagraphList::const_iterator const & pbegin,
+	ParagraphList::const_iterator const & pend)
 {
 	ParagraphList const & paragraphs = text.paragraphs();
 	for (ParagraphList::const_iterator par = pbegin; par != pend; ++par) {
@@ -123,12 +124,13 @@ ParagraphList::const_iterator makeParagraph(Buffer const & buf,
 }
 
 
-ParagraphList::const_iterator makeEnvironment(Buffer const & buf,
-					      odocstream & os,
-					      OutputParams const & runparams,
-					      Text const & text,
-					      ParagraphList::const_iterator const & pbegin,
-					      ParagraphList::const_iterator const & pend)
+ParagraphList::const_iterator makeEnvironment(
+	Buffer const & buf,
+	odocstream & os,
+	OutputParams const & runparams,
+	Text const & text,
+	ParagraphList::const_iterator const & pbegin,
+	ParagraphList::const_iterator const & pend)
 {
 	ParagraphList const & paragraphs = text.paragraphs();
 	ParagraphList::const_iterator par = pbegin;
@@ -244,12 +246,13 @@ ParagraphList::const_iterator makeEnvironment(Buffer const & buf,
 }
 
 
-ParagraphList::const_iterator makeCommand(Buffer const & buf,
-					  odocstream & os,
-					  OutputParams const & runparams,
-					  Text const & text,
-					  ParagraphList::const_iterator const & pbegin,
-					  ParagraphList::const_iterator const & pend)
+ParagraphList::const_iterator makeCommand(
+	Buffer const & buf,
+	odocstream & os,
+	OutputParams const & runparams,
+	Text const & text,
+	ParagraphList::const_iterator const & pbegin,
+	ParagraphList::const_iterator const & pend)
 {
 	ParagraphList const & paragraphs = text.paragraphs();
 	ParagraphList::const_iterator par = pbegin;
@@ -317,11 +320,13 @@ void docbookParagraphs(Text const & text,
 		       odocstream & os,
 		       OutputParams const & runparams)
 {
+	LASSERT(runparams.par_begin <= runparams.par_end,
+		{ os << "<!-- Docbook Output Error -->\n"; return; });
+
 	ParagraphList const & paragraphs = text.paragraphs();
 	ParagraphList::const_iterator par = paragraphs.begin();
 	ParagraphList::const_iterator pend = paragraphs.end();
 
-	LASSERT(runparams.par_begin <= runparams.par_end, /**/);
 	// if only part of the paragraphs will be outputed
 	if (runparams.par_begin !=  runparams.par_end) {
 		par = boost::next(paragraphs.begin(), runparams.par_begin);
@@ -340,18 +345,18 @@ void docbookParagraphs(Text const & text,
 		switch (style.latextype) {
 		case LATEX_COMMAND: {
 			send = searchCommand(par, pend);
-			par = makeCommand(buf, os, runparams, text, par,send);
+			par = makeCommand(buf, os, runparams, text, par, send);
 			break;
 		}
 		case LATEX_ENVIRONMENT:
 		case LATEX_ITEM_ENVIRONMENT: {
 			send = searchEnvironment(par, pend);
-			par = makeEnvironment(buf, os, runparams, text, par,send);
+			par = makeEnvironment(buf, os, runparams, text, par, send);
 			break;
 		}
 		case LATEX_PARAGRAPH:
 			send = searchParagraph(par, pend);
-			par = makeParagraph(buf, os, runparams, text, par,send);
+			par = makeParagraph(buf, os, runparams, text, par, send);
 			break;
 		case LATEX_BIB_ENVIRONMENT:
 		case LATEX_LIST_ENVIRONMENT:
