@@ -686,7 +686,7 @@ void TeXOnePar(Buffer const & buf,
 
 	// In some insets (such as Arguments), we cannot use \selectlanguage
 	bool const localswitch = !use_polyglossia
-		&& text.inset().getLayout().forcelocalfontswitch();
+		&& text.inset().forceLocalFontSwitch();
 	if (localswitch) {
 		lang_begin_command = lyxrc.language_command_local;
 		lang_end_command = "}";
@@ -875,7 +875,7 @@ void TeXOnePar(Buffer const & buf,
 
 	if (style.resfont.size() != font.fontInfo().size()
 	    && (nextpar || maintext
-	        || (text.inset().getLayout().isMultiPar()
+	        || (text.inset().paragraphs().size() > 1
 	            && text.inset().lyxCode() != CELL_CODE))
 	    && !is_command) {
 		os << '{';
