@@ -15,8 +15,7 @@
 #include <utility>
 
 #include "support/docstream.h"
-#include "Paragraph.h"
-#include "ParIterator.h"
+#include "Layout.h"
 #include "ParagraphList.h"
 
 
@@ -31,14 +30,18 @@ class OutputParams;
 class TexRow;
 class Text;
 
-/// Export up to \p reqargs required arguments and
-/// \p optargs optional ones. If not enough required
-/// ones are given, we'll output: {}. The optional ones
-/// must all come first.
+/** Export optional and required arguments of the paragraph \p par.
+    Non-existing required arguments are output empty: {}.
+ */
 void latexArgInsets(Paragraph const & par,
 		otexstream & os, OutputParams const & runparams,
-		unsigned int reqargs, unsigned int optargs);
-
+		Layout::LaTeXArgMap const & latexargs,
+		std::string const & prefix = std::string());
+/// Same for multi-par sequences (e.g. merged environments or InsetLayouts)
+void latexArgInsets(ParagraphList const & pars, ParagraphList::const_iterator pit,
+		otexstream & os, OutputParams const & runparams,
+		Layout::LaTeXArgMap const & latexargs,
+		std::string const & prefix = std::string());
 /** Export \p paragraphs of buffer \p buf to LaTeX.
     Don't use a temporary stringstream for \p os if the final output is
     supposed to go to a file.
