@@ -13,7 +13,9 @@
 #define OUTPUT_XHTML_H
 
 #include "LayoutEnums.h"
+
 #include "support/docstream.h"
+#include "support/shared_ptr.h"
 #include "support/strfwd.h"
 
 #include <deque>
@@ -28,19 +30,36 @@ class Text;
 // came from MathStream and its cousins.
 
 namespace html {
+
+struct FontTag;
+struct EndFontTag;
+
 /// Attributes will be escaped automatically and so should NOT
 /// be escaped before being passed to the constructor.
-struct StartTag {
+struct StartTag
+{
 	///
 	explicit StartTag(std::string const & tag) : tag_(tag), keepempty_(false) {}
 	///
 	explicit StartTag(std::string const & tag, std::string const & attr, 
 		bool keepempty = false) 
 		: tag_(tag), attr_(attr), keepempty_(keepempty) {}
+	///
+	~StartTag() {}
 	/// <tag_ attr_>
-	docstring asTag() const;
+	virtual docstring writeTag() const;
 	/// </tag_>
-	docstring asEndTag() const;
+	virtual docstring writeEndTag() const;
+	///
+	virtual FontTag const * asFontTag() const { return 0; }
+	///
+	virtual bool operator==(StartTag const & rhs) const
+		{ return tag_ == rhs.tag_; }
+	///
+	virtual bool operator!=(StartTag const & rhs) const
+		{ return !(*this == rhs); }
+	///
+	virtual bool operator==(FontTag const & rhs) const;
 	///
 	std::string tag_;
 	///
@@ -51,24 +70,31 @@ struct StartTag {
 };
 
 
-struct EndTag {
+///
+struct EndTag
+{
 	///
 	explicit EndTag(std::string tag) : tag_(tag) {}
 	/// </tag_>
-	docstring asEndTag() const;
+	virtual docstring writeEndTag() const;
+	///
+	bool operator==(StartTag const & rhs) const
+		{ return tag_ == rhs.tag_; }
+	///
+	bool operator!=(StartTag const & rhs) const
+		{ return !(*this == rhs); }
+	///
+	virtual EndFontTag const * asFontTag() const { return 0; }
 	///
 	std::string tag_;
 };
 
 
-// FIXME XHTML
-// We need to allow these to be deferrable, which means it should
-// inherit from StartTag. This is probably better, anyway, but we'll
-// need to re-work a bit of code....
 /// Tags like <img />
 /// Attributes will be escaped automatically and so should NOT
 /// be escaped before being passed to the constructor.
-struct CompTag {
+struct CompTag
+{
 	///
 	explicit CompTag(std::string const & tag)
 		: tag_(tag) {}
@@ -76,13 +102,93 @@ struct CompTag {
 	explicit CompTag(std::string const & tag, std::string const & attr)
 		: tag_(tag), attr_(attr) {}
 	/// <tag_ attr_ />
-	docstring asTag() const;
+	docstring writeTag() const;
 	///
 	std::string tag_;
 	///
 	std::string attr_;
 };
 
+
+/// A special case of StartTag, used exclusively for tags that wrap paragraphs.
+struct ParTag : public StartTag
+{
+	///
+	explicit ParTag(std::string const & tag, std::string const & attr,
+	       std::string const & parid)
+	  : StartTag(tag, attr), parid_(parid)
+	{}
+	///
+	~ParTag() {}
+	///
+	docstring writeTag() const;
+	/// the "magic par label" for this paragraph
+	std::string parid_;
+};
+
+
+///
+enum FontTypes {
+	// ranges
+	FT_EMPH,
+	FT_NOUN,
+	FT_UBAR,
+	FT_DBAR,
+	FT_WAVE,
+	FT_SOUT,
+	// bold
+	FT_BOLD,
+	// shapes
+	FT_UPRIGHT,
+	FT_ITALIC,
+	FT_SLANTED,
+	FT_SMALLCAPS,
+	// families
+	FT_ROMAN,
+	FT_SANS,
+	FT_TYPE,
+	// sizes
+	FT_SIZE_TINY,
+	FT_SIZE_SCRIPT,
+	FT_SIZE_FOOTNOTE,
+	FT_SIZE_SMALL,
+	FT_SIZE_NORMAL,
+	FT_SIZE_LARGE,
+	FT_SIZE_LARGER,
+	FT_SIZE_LARGEST,
+	FT_SIZE_HUGE,
+	FT_SIZE_HUGER,
+	FT_SIZE_INCREASE,
+	FT_SIZE_DECREASE
+};
+
+
+///
+struct FontTag : public StartTag
+{
+	///
+	explicit FontTag(FontTypes type);
+	///
+	FontTag const * asFontTag() const { return this; }
+	///
+	bool operator==(StartTag const &) const;
+	///
+	FontTypes font_type_;
+};
+
+
+///
+struct EndFontTag : public EndTag
+{
+	///
+	explicit EndFontTag(FontTypes type);
+	///
+	EndFontTag const * asFontTag() const { return this; }
+	///
+	FontTypes font_type_;
+};
+
+
 // trivial struct for output of newlines
 struct CR{};
 
@@ -124,6 +230,10 @@ public:
 	///
 	XHTMLStream & operator<<(html::CompTag const &);
 	///
+	XHTMLStream & operator<<(html::ParTag const &);
+	///
+	XHTMLStream & operator<<(html::FontTag const &);
+	///
 	XHTMLStream & operator<<(html::CR const &);
 	///
 	enum EscapeSettings {
@@ -134,25 +244,45 @@ public:
 	/// Sets what we are going to escape on the NEXT write.
 	/// Everything is reset for the next time.
 	XHTMLStream & operator<<(EscapeSettings);
+#if 0
+	/// This routine is for debugging the tag stack, etc. Code
+	/// for it is disabled by default, however, so you will need
+	/// to enable it if you want to use it.
+	void dumpTagStack(std::string const & msg) const;
+#endif
 private:
 	///
 	void clearTagDeque();
 	///
-	bool isTagOpen(std::string const &) const;
+	bool isTagOpen(html::StartTag const &) const;
 	///
-	bool isTagPending(std::string const &) const;
+	bool isTagOpen(html::EndTag const &) const;
+	///
+	bool isTagPending(html::StartTag const &) const;
 	///
 	void writeError(std::string const &) const;
 	///
 	odocstream & os_;
-	///
-	typedef std::deque<html::StartTag> TagStack;
-	/// holds start tags until we know there is content in them.
-	TagStack pending_tags_;
-	/// remembers the history, so we can make sure we nest properly.
-	TagStack tag_stack_;
 	/// 
 	EscapeSettings escape_;
+	// What we would really like to do here is simply use a
+	// deque<StartTag>. But we want to store both StartTags and
+	// sub-classes thereof on this stack, which means we run into the 
+	// so-called polymorphic class problem with the STL. We therefore have
+	// to use a deque<StartTag *>, which leads to the question who will
+	// own these pointers and how they will be deleted, so we use shared
+	// pointers.
+	///
+	typedef shared_ptr<html::StartTag> TagPtr;
+	typedef std::deque<TagPtr> TagDeque;
+	///
+	template <typename T> 
+	shared_ptr<T> makeTagPtr(T const & tag) 
+		{ return shared_ptr<T>(new T(tag)); }
+	///
+	TagDeque pending_tags_;
+	///
+	TagDeque tag_stack_;
 };
 
 ///
