@@ -55,8 +55,8 @@ public:
 	 * \param filename the file in question. Must have an absolute path.
 	 * Encoding is always UTF-8.
 	 */
-	virtual void set(std::string const & filename);
-	virtual void set(FileName const & fn, std::string const & suffix);
+	void set(std::string const & filename);
+	void set(FileName const & fn, std::string const & suffix);
 	virtual void erase();
 	/// Is this filename empty?
 	bool empty() const;
@@ -169,20 +169,13 @@ public:
 	*/
 	void changeExtension(std::string const & extension);
 
-	/** Guess the file format name (as in Format::name()) from contents.
-	 Normally you don't want to use this directly, but rather
-	 Formats::getFormatFromFile().
-	 */
-	std::string guessFormatFromContents() const;
-
-	/// check for zipped file
-	bool isZippedFile() const;
-
 	static FileName fromFilesystemEncoding(std::string const & name);
-	/// (securely) create a temporary file with the given mask.
+	/// Create a temporary file with the given mask.
 	/// \p mask must be in filesystem encoding, if it contains a
 	/// relative path, the template file will be created in the global
 	/// temporary directory as given by 'package().temp_dir()'.
+	/// CAUTION: This method may create race conditions.
+	///          Do not use, use the TempFile class instead.
 	static FileName tempName(std::string const & mask);
 	static FileName tempName(FileName const & temp_dir,
 		std::string const & mask);
@@ -203,6 +196,9 @@ public:
 	bool hasExtension(const std::string & ext);
 	/// path without file name
 	FileName onlyPath() const;
+	/// path of parent directory
+	/// returns empty path for root directory
+	FileName parentPath() const;
 	/// used for display in the Gui
 	docstring displayName(int threshold = 1000) const;
 
@@ -257,7 +253,7 @@ public:
 	 *  \param buffer_path if \c filename has a relative path, generate
 	 *  the absolute path using this.
 	 */
-	virtual void set(std::string const & filename, std::string const & buffer_path);
+	void set(std::string const & filename, std::string const & buffer_path);
 	///
 	void erase();
 	///
@@ -290,8 +286,6 @@ public:
 	std::string
 	mangledFileName(std::string const & dir = empty_string()) const;
 
-	/// \return true if the file is compressed.
-	bool isZipped() const;
 	/// \return the absolute file name without its .gz, .z, .Z extension
 	std::string unzippedFileName() const;
 
@@ -299,10 +293,6 @@ private:
 	/// Records whether we should save (or export) the filename as a relative
 	/// or absolute path.
 	bool save_abs_path_;
-	/// Cache for isZipped() because zippedFile() is expensive
-	mutable bool zipped_;
-	/// Is zipped_ valid?
-	mutable bool zipped_valid_;
 };
 
 
