@@ -178,6 +178,8 @@ string const quoteName(string const & name, quote_style style)
 		// simple parser in Systemcall.cpp do the substitution.
 		return '"' + subst(name, "\"", "\\\"") + '"';
 #endif
+	case quote_shell_filename:
+		return quoteName(os::external_path(name), quote_shell);
 	case quote_python:
 		return "\"" + subst(subst(name, "\\", "\\\\"), "\"", "\\\"")
 		     + "\"";
@@ -605,8 +607,11 @@ string latexEnvCmdPrefix(string const & path)
 		return "env TEXINPUTS=\"." + sep + texinputs_prefix
 					  + sep + texinputs + "\" ";
 	else
-		return "cmd /d /c set TEXINPUTS=." + sep + texinputs_prefix
-						   + sep + texinputs + "&";
+		// NOTE: the dummy blank dir is necessary to force the
+		//       QProcess parser to quote the argument (see bug 9453)
+		return "cmd /d /c set \"TEXINPUTS=." + sep + " "
+						+ sep + texinputs_prefix
+						+ sep + texinputs + "\" & ";
 }
 
 
