@@ -29,12 +29,9 @@ time_t current_time();
 std::string const formatted_time(time_t t, std::string const & fmt);
 
 /**
- * Inverse of ctime().
- * Since ctime() outputs the local time, the caller needs to ensure that the
- * time zone and daylight saving time are the same as when \p t was created
- * by ctime().
+ * Inverse of asctime(gmtime()).
  */
-time_t from_ctime(std::string t);
+time_t from_asctime_utc(std::string t);
 
 } // namespace support
 } // namespace lyx
