@@ -10,16 +10,6 @@ using namespace lyx;
 
 using namespace std;
 
-namespace lyx {
-	// Dummy LyXRC support
-	struct LyXRC { string icon_set; } lyxrc;
-
-	// Keep the linker happy on Windows
-	void lyx_exit(int) {}
-
-	docstring const _(string const & s) { return from_ascii(s); }
-}
-
 void test_lowercase()
 {
 	cout << to_ascii(docstring(1, lowercase(char_type('A')))) << endl;
