@@ -46,6 +46,12 @@ enum CatCode {
 	catInvalid     // 15   <delete>
 };
 
+enum cat_type {
+	NORMAL_CATCODES,
+	VERBATIM_CATCODES,
+	UNDECIDED_CATCODES
+};
+
 
 enum {
 	FLAG_BRACE_LAST = 1 << 1,  //  last closing brace ends the parsing
@@ -111,6 +117,45 @@ std::ostream & operator<<(std::ostream & os, Token const & t);
 extern void debugToken(std::ostream & os, Token const & t, unsigned int flags);
 #endif
 
+/// A docstream version that supports putback even when not buffered
+class iparserdocstream
+{
+public:
+	typedef idocstream::int_type int_type;
+
+	iparserdocstream(idocstream & is) : is_(is) {}
+
+	/// Like std::istream::operator void*()
+	/// Do not convert is_ implicitly to bool, since that is forbidden in C++11.
+	/// FIXME: Convert to operator void*() in LyX 2.2
+	operator bool() const { return s_.empty() ? !is_.fail() : true; }
+
+	/// change the encoding of the input stream to \p e (iconv name)
+	void setEncoding(std::string const & e);
+
+	// add to the list of characters to read before actually reading
+	// the stream
+	void putback(char_type c);
+
+	// add to the list of characters to read before actually reading
+	// the stream
+	void putback(docstring s);
+
+	/// Like std::istream::get()
+	iparserdocstream & get(char_type &c);
+
+	/// Like std::istream::good()
+	bool good() const { return s_.empty() ? is_.good() : true; }
+
+	/// Like std::istream::peek()
+	int_type peek() const { return s_.empty() ? is_.peek() : s_[0]; }
+private:
+	///
+	idocstream & is_;
+	/// characters to read before actually reading the stream
+	docstring s_;
+};
+
 
 /*!
  * Actual parser class
@@ -129,16 +174,31 @@ class Parser {
 	Parser & operator=(Parser const & p);
 public:
 	///
-	Parser(idocstream & is);
+	Parser(idocstream & is, std::string const & fixedenc);
 	///
 	Parser(std::string const & s);
 	///
 	~Parser();
 
-	/// change the latex encoding of the input stream
-	void setEncoding(std::string const & encoding);
-	/// get the current latex encoding of the input stream
-	std::string getEncoding() const { return encoding_latex_; }
+	/** forget already parsed next tokens and put the
+	 * corresponding characters into the input stream for
+	 * re-reading. Useful when changing catcodes. */
+	void deparse();
+
+	/// change the encoding of the input stream according to \p encoding
+	/// (latex name) and package \p package
+	bool setEncoding(std::string const & encoding, int const & package);
+	/// change the encoding of the input stream to \p encoding (iconv name)
+	bool setEncoding(std::string const & encoding);
+	/// get the current iconv encoding of the input stream
+	std::string getEncoding() const { return encoding_iconv_; }
+
+	///
+	CatCode catcode(char_type c) const;
+	///
+	void setCatcode(char c, CatCode cat);
+	/// set parser to normal or verbatim mode
+	void setCatcodes(cat_type t);
 
 	///
 	int lineno() const { return lineno_; }
@@ -148,6 +208,8 @@ public:
 	void pushPosition();
 	/// restore previous position
 	void popPosition();
+	/// forget last saved position
+	void dropPosition();
 	/// dump contents to screen
 	void dump() const;
 
@@ -199,40 +261,58 @@ public:
 	/*!
 	 * \returns the contents of the environment \p name.
 	 * <tt>\begin{name}</tt> must be parsed already, <tt>\end{name}</tt>
-	 * is parsed but not returned.
+	 * is parsed but not returned. This parses nested environments properly.
 	 */
-	std::string const verbatimEnvironment(std::string const & name);
+	std::string const ertEnvironment(std::string const & name);
 	/*
-	* The same as verbatimEnvironment(std::string const & name) but
-	* \begin and \end commands inside the name environment are not parsed.
-	* This function is designed to parse verbatim environments.
-	*/
+	 * The same as ertEnvironment(std::string const & name) but
+	 * \begin and \end commands inside the name environment are not parsed.
+	 * This function is designed to parse verbatim environments.
+	 */
 	std::string const plainEnvironment(std::string const & name);
 	/*
-	* Basically the same as plainEnvironment(std::string const & name) but
-	* instead of \begin and \end commands the parsing is started/stopped
-	* at given characters.
-	* This function is designed to parse verbatim commands.
-	*/
-	std::string const plainCommand(char left, char right, std::string const & name);
-	/*!
-	 * Returns the character of the current token and increments
-	 * the token position.
+	 * Basically the same as plainEnvironment(std::string const & name) but
+	 * instead of \begin and \end commands the parsing is started/stopped
+	 * at given characters.
+	 * This function is designed to parse verbatim commands.
 	 */
-	char getChar();
+	std::string const plainCommand(char left, char right, std::string const & name);
+	/*
+	 * Basically the same as plainEnvironment() but the parsing is
+	 * stopped at string \p end_string. Contrary to the other
+	 * methods, this uses proper catcode setting. This function is
+	 * designed to parse verbatim environments and command. The
+	 * intention is to eventually replace all of its siblings. the
+	 * member \p first of the result tells whether the arg was
+	 * found and the member \p second is the value. If \p
+	 * allow_linebreak is false, then the parsing is limited to one line
+	 */
+	Arg verbatimStuff(std::string const & end_string, 
+			  bool allow_linebreak = true);
+	/*
+	 * \returns the contents of the environment \p name.
+	 * <tt>\begin{name}</tt> must be parsed already,
+	 * <tt>\end{name}</tt> is parsed but not returned. The string
+	 * is parsed with proper verbatim catcodes and one newline is
+	 * removed from head and tail of the string if applicable.
+	 */
+	std::string const verbatimEnvironment(std::string const & end_string);
+	///
+	std::string verbatim_item();
+	///
+	std::string verbatimOption();
 	///
 	void error(std::string const & msg);
-	/// Parses one token from \p is 
-	void tokenize_one();
-	///
-	void push_back(Token const & t);
 	/// The previous token.
 	Token const prev_token() const;
 	/// The current token.
 	Token const curr_token() const;
-	/// The next token.
+	/// The next token. Caution: If this is called, an encoding change is
+	/// only possible again after get_token() has been called.
 	Token const next_token();
-	/// The next but one token.
+	/// The next but one token. Caution: If this is called, an encoding
+	/// change is only possible again after get_token() has been called
+	/// twice.
 	Token const next_next_token();
 	/// Make the next token current and return that.
 	Token const get_token();
@@ -243,36 +323,42 @@ public:
 	bool skip_spaces(bool skip_comments = false);
 	/// puts back spaces (and comments if \p skip_comments is true)
 	void unskip_spaces(bool skip_comments = false);
-	///
-	void lex(std::string const & s);
-	///
+	/// Is any further input pending()? This is not like
+	/// std::istream::good(), which returns true if all available input
+	/// was read, and the next attempt to read would return EOF.
 	bool good();
-	///
-	std::string verbatim_item();
-	///
-	std::string verbatimOption();
 	/// resets the parser to initial state
 	void reset();
-	///
-	void setCatCode(char c, CatCode cat);
-	///
-	CatCode getCatCode(char c) const;
 
 private:
+	/// Setup catcode table
+	void catInit();
+	/// Parses one token from \p is
+	void tokenize_one();
+	///
+	void push_back(Token const & t);
 	///
 	int lineno_;
 	///
 	std::vector<Token> tokens_;
 	///
-	unsigned pos_;
+	size_t pos_;
 	///
 	std::vector<unsigned> positions_;
 	///
 	idocstringstream * iss_;
 	///
-	idocstream & is_;
-	/// latex name of the current encoding
-	std::string encoding_latex_;
+	iparserdocstream is_;
+	/// iconv name of the current encoding
+	std::string encoding_iconv_;
+	///
+	CatCode theCatcode_[256];
+	///
+	cat_type theCatcodesType_;
+	///
+	cat_type curr_cat_;
+	///
+	bool fixed_enc_;
 };
 
 
